#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web;

use strict;
use warnings;
use Moose;
use namespace::autoclean;

use lib "/usr/local/nagios/etc";
use I18N::LangTags::List;

# Uncomment the following two lines if you want to see a stack trace of warn messages that appear
#use Carp qw(cluck);
#$SIG{__WARN__} = \&cluck;

# Note: Set "CATALYST_DEBUG=1" in shell env to turn debugging on
#
# Set flags and add plugins for the application
#
#   ConfigLoader: will load the configuration from a YAML file in the
#                 application's home directory
# Static::Simple: will serve static files from the application's root
#                 directory
#        Session: session support (plus FastMmap storage and cookie state)
# Authentication: user login support + the CDBI/DBIC user store plugin

use Catalyst::Log::Log4perl;

use Catalyst qw/ConfigLoader Static::Simple
    Session Session::Store::DBI Session::State::Cookie
    Authentication
    Cache::FastMmap
    Snippets
    I18N
    Unicode
    AutoRestart
    /;

use CatalystX::RoleApplicator;

extends 'Catalyst';

__PACKAGE__->apply_request_class_roles(
    qw[
        Catalyst::TraitFor::Request::REST::ForBrowsers
        ]
);

our $VERSION = '0.01';

# This is a bug in C::P::Authentication. It returns an empty list if $c->user doesn't exist in
# a function call, thus the sub function doesn't think the value has been set
# A possible patch is in the works
# Also, if a session is valid but the user is not restoreable (because the contact has been removed from db),
# we catch the error and return undef
# TODO: delete the previously valid session?
# TODO: need to profile this as it could be a bottleneck
sub user {
    my ( $self, @args ) = @_;
    my $user;
    eval { $user = $self->SUPER::user(@args) };
    if ( $@ && $@ =~ /Store claimed to have a restorable user/ ) {
        return undef;
    }
    $user;
}

# Routine to add a menu option. Initial menu gets created at ->setup time
# { menu_name => "Status", name => "Super view", url => $url }
sub add_menu_option {
    my ( $self, $args ) = @_;
    my $menu_name = delete $args->{menu_name};
    my $menus     = $self->config->{"Controller::SideNav"}->{menus};
LOOP: foreach my $entry (@$menus) {
        if ( $entry->{name} eq $menu_name ) {
            push @{ $entry->{entries} }, $args;
            last LOOP;
        }
    }
}

# We're overriding Catalyst->prepare_path
# We override to add base prefix here, to allow relocatable apps
sub prepare_path {
    my $c = shift;
    $c->engine->prepare_path( $c, @_ );
    if ( $c->config->{override_base_prefix} ) {
        $c->request->base->path( $c->config->{override_base_prefix} . "/" );
    }
}

# Convenience function to return path only
sub uri_for_path {
    my ( $self, @args ) = @_;
    return $self->uri_for(@args)->path;
}

# Convenience funtion to always remove output from parameter. Used for /status uris, so any other common changes can go here
sub uri_for_params_status {
    my ( $self, $path, $params, $todelete ) = @_;
    $todelete ||= [];
    push @$todelete, "output";
    $self->uri_for_params( $path, $params, $todelete );
}

# Convenience function to create a uri_for, but using existing params
# Also allowed to pass an array for deleting params
# Also checks if the stash has a hashref for parameter overriding
#TODO: $opts should be after path - change for latest version
sub uri_for_params {
    my ( $self, $path, $params, $todelete, $opts ) = @_;
    $params   ||= {};
    $todelete ||= [];
    $opts     ||= {};
    $path = $self->request->uri->path unless $path;
    my $uri = $self->uri_for($path);
    if ( !$opts->{ignore_params_override}
        && exists $self->stash->{params_override} )
    {
        my $overrides = $self->stash->{params_override};
        foreach my $k ( keys %{$overrides} ) {
            $params->{$k} = $overrides->{$k};
        }
    }
    my $newparams = { %{ $self->req->params }, %$params };
    map { delete $newparams->{$_} } @$todelete;
    $uri->query_form($newparams);
    return $uri;
}

# Convenience function to get to the public role
sub public_role { shift->model("OpsviewSchema::Roles")->find(1); }

sub forward_if_exists {
    my $self = shift;

    # Stolen from Catalyst::Dispatcher.pm
    my ( $action, $args, $captures ) = $self->dispatcher->_command2action( $self, @_ );
    return 0 if ( !$action );
    $self->forward(@_);
}

sub is_community_edition {
    my ($self) = @_;
    return 1 if ( $self->config->{edition} eq "community" );
    return 0;
}

# For linking to document site
sub helpsite {
    my ( $self, $section ) = @_;
    $self->config->{help_url} . $section;
}

# Use i_default as the language that will be used for translate
# We use the phrase "translate" for audit log entries because Locale::Maketext::Extract::Plugin::Perl
# looks for that function. As audit log entries will always be in perl controllers, we
# hook into that to provide the maketext call here
# We could make it a configurable option in future at the yml level
my $i_default;

sub translate {
    my $c = shift;
    if ( !defined $i_default ) {
        $i_default = Opsview::Web::I18N->get_handle("i_default");
    }
    $i_default->maketext(@_);
}

# This is same as loc, but if a translation is not possible (ie, the translation is the same as the initial string), then
# return undef
sub ifloc {
    my $c           = shift;
    my $id          = $_[0];
    my $translation = $c->loc(@_);
    return undef if ( $translation eq $id );
    return $translation;
}

# These were used in /events, but not required anymore
# Should keep translations, so only remove later when other places use these language keys
sub loc_state_type {
    my ( $c, $statetypeid ) = @_;
    if ( $statetypeid == 0 ) {
        $c->loc("ui.status.label.soft");
    }
    else {
        $c->loc("ui.status.label.hard");
    }
}

sub loc_service_state {
    my ( $c, $state ) = @_;
    if ( $state == 0 ) {
        $c->loc("ui.status.label.ok");
    }
    elsif ( $state == 1 ) {
        $c->loc("ui.status.label.warning");
    }
    elsif ( $state == 2 ) {
        $c->loc("ui.status.label.critical");
    }
    elsif ( $state == 3 ) {
        $c->loc("ui.status.label.unknown");
    }
}

sub loc_host_state {
    my ( $c, $state ) = @_;
    if ( $state == 0 ) {
        $c->loc("ui.status.label.up");
    }
    elsif ( $state == 1 ) {
        $c->loc("ui.status.label.down");
    }
    elsif ( $state == 2 ) {
        $c->loc("ui.status.label.unreachable");
    }
}

#
# Configure the application
#
__PACKAGE__->config->{using_frontend_proxy} = 1;
__PACKAGE__->config( name => 'Opsview::Web', default_view => "TT" );
__PACKAGE__->config( file => "opsview_web" );    # Load the yml file
__PACKAGE__->config->{authentication}{http}{type} = "basic";

# Use these files from /usr/local/nagios/share
__PACKAGE__->config->{static}->{dirs}              = [qw/images stylesheets javascript help media/];
__PACKAGE__->config->{static}->{include_path}      = ['/usr/local/nagios/share'];
__PACKAGE__->config->{static}->{ignore_extensions} = [qw/tt/];

# General FastMmap cache - used by C::P::Snippets in snmpwalk
# Separate from session data (though also implemented as fastmmap)
# We delete it on startup. Need to create path otherwise complains on startup
# TODO: pass init_file instead? See Cache::FastMmap. Patch Catalyst::Plugin::Cache::FastMmap
# to allow
$_ = "/tmp/opsview/web/cache";
unlink $_;
Path::Class::Dir->new("/tmp/opsview/web/cache")->mkpath;
__PACKAGE__->config->{cache}->{storage}  = $_;
__PACKAGE__->config->{cache}->{expires}  = 3600;
__PACKAGE__->config->{'Plugin::Session'} = {
    expires           => 3600,
    dbi_dbh           => "OpsviewSchema",
    dbi_table         => "sessions",
    dbi_id_field      => "id",
    dbi_data_field    => "session_data",
    dbi_expires_field => "expires",
};

# Allow JSON output
# Disable the X-JSON header
__PACKAGE__->config(
    {   'View::JSON' => {
            allow_callback   => 1,
            expose_stash     => [qw(ResultSet)],
            no_x_json_header => 1,
        }
    }
);

my $log4perlconf = __PACKAGE__->path_to("etc/Log4perl.conf");

# Create a Log4perl object
__PACKAGE__->log( Catalyst::Log::Log4perl->new( "$log4perlconf", watch_delay => 60, ) );

# If debug is set (ie, run from dev server with -d), then change log4perl to use
# DEBUG level instead of whatever the config file says
# Only affects rootLogger - can change categories to allow/deny as required
use Log::Log4perl::Level;
Log::Log4perl->get_logger("")->level($DEBUG)
    if ( defined( $ENV{CATALYST_DEBUG} ) );

# Start using this as the separator, but not universally used yet
__PACKAGE__->config->{page_separator} = " > ";

# Start the application
#
__PACKAGE__->setup;

# All the authentication below has moved to ActionRoles, specified at the
# URL definition, rather than here
# Below can be removed when proven to work properly

# This allows these two roles through underneath, but you will need to split off
# anything with more specific access with denies further down the path
#__PACKAGE__->deny_access_unless( "/admin", [qw/CONFIGUREVIEW/], );

# Required to check /user/preference too
#__PACKAGE__->allow_access("/admin/check_save_access");

# Allow this for /user/preference - has its own access denied routines
#__PACKAGE__->allow_access("/admin/ajax/send_test_sms");

# View access required for /event
# Seems a silly way of specifying it, but this means you have to have VIEWSOME or VIEWALL to see
# t/720acl.t checks the correct accesses
#__PACKAGE__->allow_access_if( "/event",     [qw/VIEWSOME/] );
#__PACKAGE__->deny_access_unless( "/event",  [qw/VIEWALL/] );
#__PACKAGE__->allow_access_if( "/search",    [qw/VIEWSOME/] );
#__PACKAGE__->deny_access_unless( "/search", [qw/VIEWALL/] );

# Required to get the authentication if directly going to this page
#__PACKAGE__->allow_access("/event/auto");

#__PACKAGE__->allow_access_if( "/search",    [qw/VIEWSOME/] );
#__PACKAGE__->deny_access_unless( "/search", [qw/VIEWALL/] );
#__PACKAGE__->allow_access("/search/auto");

# Auto is required for all access to get to admin pages
#__PACKAGE__->allow_access("/admin/auto");
#__PACKAGE__->allow_access("/admin/meta_redirect");

# Need these two rules on /admin/host so that only CONFIGUREHOSTS is allowed
#__PACKAGE__->deny_access_unless( "/admin/host",                     [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/host",                        [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/strip_blank_servicechecks",   [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/convert_servicechecks_input", [qw/CONFIGUREHOSTS/], );

#__PACKAGE__->allow_access_if( "/admin/snmpwalk", [qw/CONFIGUREHOSTS/], );

#__PACKAGE__->allow_access_if( "/admin/ajax", [qw/CONFIGUREHOSTS/], );

# The two rules below are to allow CONFIGUREHOSTS to also get to these parts
# ADMINACCESS will inherit the /admin/servicecheck allow above
#__PACKAGE__->allow_access_if( "/admin/servicecheck/info_for_monitors_list", [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/serviceoption",                       [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/servicecheck/auto",                   [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/ajax/options_for_parent_hosts",       [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/servicegroup/auto",                   [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/snmpwalk/auto",                       [qw/CONFIGUREHOSTS/], );
#__PACKAGE__->allow_access_if( "/admin/servicegroup/services",               [qw/CONFIGUREHOSTS/], );

# Reload ability
#__PACKAGE__->allow_access_if( "/admin/reload",    [qw/RELOADACCESS/], );
#__PACKAGE__->deny_access_unless( "/admin/reload", [qw/RELOADACCESS/], );
#__PACKAGE__->allow_access_if( "/admin/status",    [qw/RELOADACCESS/], );
#__PACKAGE__->deny_access_unless( "/admin/status", [qw/RELOADACCESS/], );

# Block Audit log unless ADMINACCESS only
#__PACKAGE__->allow_access_if( "/admin/auditlog",    [qw/ADMINACCESS/], );
#__PACKAGE__->deny_access_unless( "/admin/auditlog", [qw/ADMINACCESS/], );

# This config can be applied after setup without issue as it doesn't provide
# any necessary info to any modules.  Cannot be run before setup since the
# info required isn't loaded from the yml file
# TODO: Is this the best way of doing this?

# Check that version has been read from config file. If not, then potentially the YML
# file is corrupted
$_ = __PACKAGE__->config->{version};
unless ($_) {
    die "Cannot find version string in opsview_web.yml - file is possibly invalid (remove any hard tabs)\n";
}

# Record the major and minor version in the config too, split up from version
# Possible version strings:
#  2.7.6
#  2.8HEAD
#  2.9
__PACKAGE__->config->{version} =~ m/^(\d+\.(\d+))(\.\d+|HEAD)?/;
__PACKAGE__->config->{major_version} = $1 or die "Invalid version number";
__PACKAGE__->config->{edition} = ( $2 % 2 == 0 ? 'enterprise' : 'community' );

# Generate the documentation URL
# Also switch community edition to use community reload image
my $doc_suffix;
if ( __PACKAGE__->is_community_edition ) {
    $doc_suffix = "-community";
    __PACKAGE__->config->{reload_image} = "/images/reload_progress_community.gif";
}
else {
    $doc_suffix = __PACKAGE__->config->{major_version};
}
__PACKAGE__->config->{doc_url} = __PACKAGE__->config->{help}->{base_url} . $doc_suffix;
undef $doc_suffix;

# and the help url
__PACKAGE__->config->{help_url} = __PACKAGE__->config->{doc_url} . __PACKAGE__->config->{help}->{separator};

# set build number if not already set (i.e. assume this is dev env)
if ( !__PACKAGE__->config->{build} ) {
    __PACKAGE__->config->{build} = "DEVEL";
}

# set the uuid
sub generate_uuid() {
    my ($self) = @_;
    my $sysprefs = $self->model("OpsviewSchema::Systempreferences")->find(1);
    if ( $sysprefs->uuid eq "" ) {
        use Data::UUID;
        my $ug = new Data::UUID;
        $sysprefs->uuid( $ug->create_str() );
        $sysprefs->update;
    }
}
__PACKAGE__->generate_uuid();

Opsview::Auditlog->system("Starting Opsview Web");

__PACKAGE__->model("OpsviewSchema::MonitorPacks")->install_new_monitorpacks( Log::Log4perl->get_logger("") );

#
# IMPORTANT: Please look into Opsview::Web::Controller::Root for more
#

1;
