#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# Need this so that anything under /admin is protected
package Opsview::Web::Controller::Admin;

use strict;
use warnings;

use base qw/Catalyst::Controller/;

use Opsview::Utils qw(convert_to_arrayref);

sub auto : Private {
    my ( $self, $c, $id ) = @_;

    return 0 unless $c->forward('/authenticate');

    $c->stash( additional_template_paths => [ $c->path_to( 'root', 'admin' )->stringify ] );

    return 1;
}

sub meta_redirect : Private {
    my ( $self, $c ) = @_;
    return if scalar @{ $c->error };
    my $from = $c->stash->{from} || "";

    # This bizarrely happens if you are editing an object, hit refresh and then save,
    # get returned back to edit page, and then save again.
    # Will then get stuck in a loop because $from will be /admin/host/save/1, so will
    # keep going back to save/1. We intercept and go to the list page
    if ( $from eq $c->req->uri || $from eq "default" ) {
        $from = $c->req->base . $c->stash->{list_page};
    }

    if ( $from eq "null" ) {
        $c->res->body(" ");
    }
    else {
        $c->res->body(
            qq!<html>
  <head>
    <meta http-equiv="refresh" content="0; URL=${from}" />
  </head>
</html>!
        );
    }
}

sub serviceoption : Private {
    my ( $self, $c, @args ) = @_;

    my $o = $c->stash->{object};
    my $services;

    if ($o) {
        my $objecttype = ref $o;
        if ( $objecttype =~ /Hosts?$/ ) {
            $services = $c->model("OpsviewDB::Servicecheck")->list_servicegroups( { hostid => $o->id } );
        }
        else {
            $services = $c->model("OpsviewDB::Servicecheck")->list_servicegroups( { hosttemplateid => $o->id } );
        }
    }
    else {

        # This is for a new hosttemplate - basically get list
        $services = $c->model("OpsviewDB::Servicecheck")->list_servicegroups( { hosttemplateid => 0 } );
    }
    $c->stash( service_list => $services );
}

sub strip_blank_servicechecks : Private {
    my ( $self, $c ) = @_;
    my $sc_list = $c->req->params->{servicechecks};
    $sc_list = convert_to_arrayref($sc_list);
    my $remove_blank_servicechecks = [ grep { $_ if $_ } @$sc_list ];
    $c->req->params->{servicechecks} = $remove_blank_servicechecks;
}

# Converts web form input of servicechecks into servicechecks and excludedservicechecks
# for DBIx::Class model to process
sub convert_servicechecks_input : Private {
    my ( $self, $c, $attrs ) = @_;

    # Get hashes, indexed by servicecheck id, of exceptions/timed_exceptions/event_handler information
    my $other_sc_attributes = {};
    foreach my $key ( keys %$attrs ) {
        if ( $key =~ m/^exception_(\d+)_add$/ ) {
            $other_sc_attributes->{$1}->{exception} = delete $attrs->{$key};
        }
        elsif ( $key =~ m/^timedoverride_(\d+)_cmd$/ ) {
            $other_sc_attributes->{$1}->{timed_exception} = {
                timeperiod => delete $attrs->{"timedoverride_${1}_time"},
                args       => delete $attrs->{$key},
            };
        }
        elsif ( $key =~ m/^event_handler_(\d+)_add$/ ) {
            $other_sc_attributes->{$1}->{event_handler} = delete $attrs->{$key};
        }
    }

    # Convert remove-{id} to be attrs of excludedservicechecks instead
    my $sc_input              = convert_to_arrayref( $attrs->{servicechecks} );
    my @servicechecks         = ();
    my @excludedservicechecks = ();
    foreach my $sc (@$sc_input) {
        my $hash = $other_sc_attributes->{$sc} || {};
        if ( $sc =~ /^remove-(\d+)$/ ) {
            $hash->{id}                  = $1;
            $hash->{remove_servicecheck} = 1;
        }
        elsif ( $sc =~ /^\d+$/ ) {
            $hash->{id} = $sc;
        }
        else {

            # Need this to ignore servicechecks that have not been selected in web UI
            next;
        }
        push @servicechecks, $hash;
    }
    $attrs->{servicechecks} = \@servicechecks;
}

sub check_save_access : Private {
    my ( $self, $c ) = @_;
    if ( !( $c->stash->{user_has_save_access} || $c->user->has_access("CONFIGURESAVE") ) ) {
        $c->detach("/access_denied");
    }
}

sub check_datetime : Local : Arg(0) : Does(ACL) : RequiredRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    $c->log->debug("in check_datetime");

    my $invalid = 0;

    foreach my $p ( keys %{ $c->req->params } ) {
        my $v = $c->req->param($p);
        $c->log->debug("checking :$p: with val :$v:");
        if ( defined $v && Opsview::Common::parse_string($v) ) {
            $invalid++;
        }
    }
    $c->log->debug("invalid = $invalid");
    $c->stash( ResultSet => ( $invalid == 0 ? 0 : 1 ) );
    $c->stash( output => "json" );
}

1;
