#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Graph;

use Moose;
use namespace::autoclean;

use Opsview::Common;

use Chart::Clicker;
use Chart::Clicker::Axis;
use Chart::Clicker::Axis::DateTime;
use Chart::Clicker::Data::DataSet;
use Chart::Clicker::Data::Series;
use Chart::Clicker::Decoration::Grid;
use Chart::Clicker::Decoration::Legend;
use Chart::Clicker::Decoration::Plot;
use Chart::Clicker::Drawing qw(:positions);
use Chart::Clicker::Drawing::Insets;
use Chart::Clicker::Renderer::Line;

BEGIN { extends qw/ Opsview::Web::ControllerBase::Admin / }

__PACKAGE__->config();

use Odw;

before 'auto' => sub {
    my ( $self, $c, @args ) = @_;
    my $title = $c->loc("ui.admin.graph.title");
    $c->stash( title_description => $title );
    $c->stash( submit_name       => "plain" );
};

sub edit       { }
sub save       { }
sub list       { }
sub delete     { }
sub save_new   { }
sub create_new { }
sub clone      { }
sub unique     { }

sub display : Path : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;

    my $end_dt = DateTime->now();
    my $start_dt = $end_dt->clone->subtract( days => 7 );

    $c->stash->{form_method}  = "GET";
    $c->stash->{form_enctype} = "";
    $c->stash->{end_time}     = $c->request->params->{end_time} || $end_dt->ymd . " " . $end_dt->hms;
    $c->stash->{start_time}   = $c->request->params->{start_time} || $start_dt->ymd . " " . $start_dt->hms;
    $c->stash->{host}         = $c->request->params->{host} ||= undef;
    $c->stash->{servicecheck} = $c->request->params->{servicecheck};
    $c->stash->{metric}       = $c->request->params->{metric};

    if ( $c->stash->{host} && $c->stash->{servicecheck} && defined( $c->stash->{metric} ) ) {
        ( my $metric = $c->stash->{metric} ) =~ s/(['\\])/\\$1/g;
        $c->log->debug( 'Metric changed from "', $c->stash->{metric}, '" to "', $metric, '"' ) if ( $c->debug );
        my $data = $c->model("OdwDB::Searches")->list_performance_data( $c->stash->{start_time}, $c->stash->{end_time}, $c->stash->{host}, $c->stash->{servicecheck}, $metric, );

        # make sure there is data to graph
        my @nodata;
        foreach my $d ( @{$data} ) {
            if ( !scalar( $d->{'values'} ) ) {
                push( @nodata, $d->{'metric'} );
            }
        }
        if ( !@{$data} ) {
            $c->stash( message => "No data was found" );
        }
        elsif (@nodata) {
            $c->stash( message => "The following datasets were empy: " . join( ",", @nodata ) );
        }
        else {
            $c->forward( "generate_graph", [$data] );
        }
    }

    $c->stash( action_uri => $c->uri_for("display") );

    $c->stash( template => 'edit_layout' );
}

sub generate_graph : Private {
    my ( $self, $c, $data ) = @_;

    my @serieses;
    foreach my $d ( @{$data} ) {
        my $units = "";
        $units = " (" . $d->{'units'} . ")" if ( $d->{'units'} );

        my $series = new Chart::Clicker::Data::Series(
            {   keys   => $d->{'keys'},
                values => $d->{'values'},
                name   => $d->{'metric'} . $units,
            }
        );
        push( @serieses, $series );
    }

    my $chart = new Chart::Clicker( { datasets => [ new Chart::Clicker::Data::DataSet( { series => \@serieses } ), ], } );

    $chart->add( new Chart::Clicker::Decoration::Legend( { margins => new Chart::Clicker::Drawing::Insets( { top => 3, } ), } ), $CC_BOTTOM );

    my $daxis = new Chart::Clicker::Axis::DateTime(
        {   orientation => $CC_HORIZONTAL,
            position    => $CC_BOTTOM,
            label       => "Date/Time",
            ticks       => 4,
            time_zone   => "local",
        }
    );

    $chart->add( $daxis, $CC_AXIS_BOTTOM );
    $chart->domain_axes( [$daxis] );

    my $raxis = new Chart::Clicker::Axis(
        {   orientation => $CC_VERTICAL,
            position    => $CC_LEFT,
            format      => '%0.5f',
            label       => "Units",
        }
    );

    $chart->add( $raxis, $CC_AXIS_LEFT );
    $chart->range_axes( [$raxis] );

    $chart->add( new Chart::Clicker::Decoration::Grid(), $CC_CENTER, 0 );

    my $renderer = new Chart::Clicker::Renderer::Line();

    my $plot = new Chart::Clicker::Decoration::Plot();
    $plot->renderers( [$renderer] );
    $chart->add( $plot, $CC_CENTER );
    eval { $chart->prepare(); };
    if ($@) {
        $c->stash->{message} = "Failed to prepare chart: $@";
        return 1;
    }

    eval { $chart->draw(); };
    if ($@) {
        $c->stash->{message} = "Failed to draw chart: $@";
        return 1;
    }

    my $img = eval { $chart->png; };
    if ($@) {
        $c->stash->{message} = "Failed to render chart: $@";
        return 1;
    }

    $c->flash( graph_image => $img );
    $c->stash->{image}     = 1;
    $c->stash->{post_form} = "post_form_graph";
}

sub get_graph : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;

    $c->response->content_type("image/png");
    $c->response->body( $c->flash->{graph_image} );
    delete $c->stash->{wrapper};
}

sub get_servicechecks : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $host         = $c->request->params->{host};
    my $servicecheck = $c->request->params->{check};
    my @names;

    my @checks = $c->model("OdwDB::Host")->list_host_servicechecks_with_perfdata($host);
    foreach my $check (@checks) {
        my $selected;
        $selected = "selected" if ( $check eq $servicecheck );
        push( @names, { name => $check, value => $check, selected => $selected } );
    }
    $c->log->_dump( \@names );
    $c->stash( wrapper  => "wrappers/blank" );
    $c->stash( template => "optionslist" );
    $c->stash( list     => \@names );
}

sub get_metrics : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $host  = $c->request->params->{host};
    my $check = $c->request->params->{check};
    my $met   = $c->request->params->{metric};
    my @names;

    my @metrics = $c->model("OdwDB::Servicecheck")->list_host_servicecheck_metrics( $host, $check );
    $c->log->debug("got metric $#metrics");
    if ( $#metrics > 0 ) {
        push( @names, { name => 'All', value => 0 } );
    }
    foreach my $metric (@metrics) {
        my $selected;
        $selected = "selected" if ( $met eq $metric );
        push( @names, { name => $metric, value => $metric, selected => $selected } );
    }
    $c->stash( wrapper  => "wrappers/blank" );
    $c->stash( template => "optionslist" );
    $c->stash( list     => \@names );
}

1;
