#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Host;

use Moose;
use namespace::autoclean;

use Opsview::Utils::QueryHost;
use Opsview::Utils qw(convert_to_arrayref);
use JSON;
use Try::Tiny;

BEGIN { extends 'Opsview::Web::ControllerBase::Admin' }

__PACKAGE__->config(
    model_name         => 'OpsviewSchema::Hosts',
    set_many_words     => [qw/keywords/],
    set_commasep       => [qw/notification_options/],
    query_host_command => "/usr/local/nagios/bin/query_host",
    search_column      => [qw(name ip)],
    page_search_box    => 1,
    page_add_new       => 1,
    use_synchronise    => {
        use_nmis               => 0,
        use_rancid             => 0,
        use_mrtg               => 0,
        enable_snmp            => 0,
        flap_detection_enabled => 0,
        parents                => [],
        servicechecks          => [],
        hosttemplates          => [],
        notification_options   => [],
        rancid_autoenable      => 0,
        tidy_ifdescr_level     => 0,
        snmp_max_msg_size      => 0,
    },
);

sub auto : Private {
    my ( $self, $c, @args ) = @_;
    $c->stash( title_description => $c->loc("ui.objecttype.host") );
    $self->next::method( $c, @args );
    if ( ( $c->stash->{object} ) && !$c->user->can_configurehost( $c->stash->{object}->id ) ) {
        $c->forward("/access_denied");
    }
    my $tabs = [qw(Host Notifications Monitors SNMP Attributes)];
    foreach my $plugin ( Opsview->plugins ) {
        next unless $plugin->can("display_host_tab");
        push @$tabs, $plugin->display_host_tab;
    }
    $c->stash( tabs => $tabs );
}

sub delete : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    shift->next::method(@_);
}

sub save : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, @args ) = @_;
    my $p = $c->req->params;

    # If there is no override, set to undef the arg.
    foreach my $key ( keys %{ $c->req->params } ) {
        if ( $key =~ /^(hostattributes\[\d+\]\[arg(\d+)\])/ ) {
            my $overridekey = "${key}[checkbox]";
            if ( !exists $p->{$overridekey} ) {
                $p->{$key} = undef;
            }
            delete $p->{$overridekey};
        }
    }

    # Loop again to delete keys. Can't do earlier as information could be out of order
    foreach my $key ( keys %{ $c->req->params } ) {
        if ( $key =~ /^(hostattributes\[\d+\]\[arg(\d+)\]\[checkbox\])$/ ) {
            delete $p->{$key};
        }
    }

    $self->next::method( $c, @args );
}

sub save_new : Local : Args(0) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    shift->next::method(@_);
}

sub edit : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, @args ) = @_;
    $self->next::method( $c, @args );
    $c->forward("/admin/serviceoption");
    $c->stash( interfaces => [ $c->stash->{object}->snmpinterfaces ] );
    if ( my $p = $c->req->param("tab") ) {
        if ( $p =~ /^[1-9]$/ ) {
            $p = lc( $c->stash->{tabs}[ $p - 1 ] );
        }
        $c->stash( "initial_tab" => "tab-$p" );
    }
}

sub edit_common {
    my ( $self, $c ) = @_;

    # Create information about each attribute
    my $attributes_values       = [];
    my $attributes_labels       = {};
    my $attributes_fields       = {};
    my $attributes_default_args = {};
    foreach my $a ( $c->model("OpsviewSchema::Attributes")->search( { internally_generated => 0 } ) ) {
        my $id = $a->id;
        push @$attributes_values, $a->name;
        $attributes_labels->{ $a->name } = $a->name;
        my @extra_fields;
        my $h = $attributes_default_args->{ $a->name } = {};
        foreach my $i ( 1 .. 9 ) {
            my $argname = "arg$i";
            $h->{$argname} = $a->$argname;
            my $label = "label$i";

            #last unless $a->$label;
            push @extra_fields, $a->$label;
        }
        $attributes_fields->{$id} = \@extra_fields;
    }
    $c->stash( attributes_values            => $attributes_values );
    $c->stash( attributes_labels            => $attributes_labels );
    $c->stash( attributes_fields            => $attributes_fields );
    $c->stash( attributes_fields_json       => to_json($attributes_fields) );
    $c->stash( attributes_default_args_json => to_json($attributes_default_args) );

    if ( $c->stash->{object} ) {
        push @{ $c->stash->{page_view_options_links} },
            {
            link => $c->uri_for( "/admin/host/services", $c->stash->{object}->id )->path,
            name => $c->loc("ui.admin.host.menu.resolveServiceChecks"),
            };
    }
}

sub list : Local : Args(0) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( title_description => $c->loc("ui.admin.host.list.title") );

    my $hg_paths = [ map { $_->matpath . "%" } $c->user->role->hostgroups ];
    $c->stash->{list_search_criteria} = { "hostgroup.matpath" => { "-like" => $hg_paths } };

    # Due to a warning in DBIx::Class, we've removed the prefetch of monitoringservers and monitoringclusters
    $c->stash->{page_attributes} = { join => "hostgroup", order_by => "me.name", prefetch => [ "icon", "hostgroup", ] };

    #$c->stash->{page_attributes} = { join => "hostgroup", order_by => "me.name", prefetch => [ "icon", "hostgroup", "monitoringservers", "monitoringclusternodes" ] };

    $self->next::method($c);
}

sub create_new : Path('new') : Args(0) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, @args ) = @_;
    $self->next::method( $c, @args );
    $c->stash( tabs_first_only => 1 );
    $c->stash( submit_name     => "next" );
}

sub clone : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, @args ) = @_;
    $self->next::method( $c, @args );
    $c->forward("/admin/serviceoption");
}

sub unique : Local : Arg(0) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    shift->next::method(@_);
}

# This is too late to do in save_new, so we override here instead
sub meta_redirect : Private {
    my ( $self, $c, @args ) = @_;
    if ( $c->action eq "admin/host/save_new" && !$c->stash->{clone_object} ) {
        $c->log->debug("Save new - redirecting to 2nd tab");
        $c->stash( from => $c->uri_for( "/admin/host/edit", $c->stash->{object}->id, { tab => 2, from => $c->stash->{from} } ) );
    }
    elsif ( exists $c->req->params->{".submit_and_redirect"} ) {
        $c->stash( from => $c->uri_for( "/admin/host/interfaces", $c->stash->{object}->id )->path_query );
    }
    $self->next::method( $c, @args );
}

sub do_synchronise {
    my ( $self, $c, $attrs ) = @_;
    $c->forward( "/admin/convert_servicechecks_input", [$attrs] );

    # Sort out snmp interfaces
    #$self->convert_snmpinterface_input($attrs);

    # Create new hostgroup if requested
    if ( ( my $new_hg = delete $attrs->{hostgroup_new} ) && $c->user->has_access("ADMINACCESS") ) {
        my $hg;
        try {
            $hg = $c->model("OpsviewSchema::Hostgroups")->find_or_create( { name => $new_hg } );
        }
        catch {
            $self->handle_exception( $c, $_ );
        };
        $attrs->{hostgroup} = $hg->id;
    }

    my $hash = Opsview::Web::Utils->convert_indexed_params_to_hash( $attrs, { delete => 1 } );
    my @hostattributes = values( %{ $hash->{hostattributes} } );
    $attrs->{hostattributes} = \@hostattributes;

    $self->next::method( $c, $attrs );
}

sub services : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $o = $c->stash->{object};
    my $services;
    unless ($o) {
        $c->res->redirect( $c->uri_for("/admin/host") );
        $c->detach;
    }
    $c->stash( service_objects => $o->resolved_servicechecks );
    $c->stash( management_urls => $o->list_managementurls );
    $c->stash( page_title      => " > " . $c->loc( "ui.admin.host.title.resolvedServices [_1]", $o->name ) );
    push @{ $c->stash->{page_view_options_links} },
        {
        link => $c->uri_for( "/admin/host/edit",          $o->id )->path,
        name => $c->loc( "ui.admin.list.label.edit [_1]", $o->name ),
        };
}

sub autocomplete : Local : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my @hosts = $c->model("OpsviewDB::Host")->search_snmp_hosts_by_name( "%" . $c->req->param(".host") . "%" );
    my @names = map { $_->name } @hosts;
    $c->stash( wrapper  => "wrappers/blank" );
    $c->stash( template => "unorderedlist" );
    $c->stash( list     => \@names );
}

sub query_host : Local : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, $credentials_via_host ) = @_;

    $c->forward("/admin/check_save_access");

    $c->stash( template => "snmpinterfaces" );
    $c->stash( wrapper  => "wrappers/blank" );
    if ( $c->req->param("testconnection") ) {
        $c->stash( testconnection => 1 );
    }

    my $query_host_args = {};
    $query_host_args->{command} = $self->config->{query_host_command};

    if ($credentials_via_host) {

        # Set credential info here
        my $host = $c->stash->{object};
        foreach my $k qw(snmp_version snmp_port) {
            $query_host_args->{$k} = $host->$k;
        }
        $query_host_args->{snmp_auth}->{community}    = $host->snmp_community;
        $query_host_args->{snmp_auth}->{username}     = $host->snmpv3_username;
        $query_host_args->{snmp_auth}->{authprotocol} = $host->snmpv3_authprotocol;
        $query_host_args->{snmp_auth}->{authpassword} = $host->snmpv3_authpassword;
        $query_host_args->{snmp_auth}->{privprotocol} = $host->snmpv3_privprotocol;
        $query_host_args->{snmp_auth}->{privpassword} = $host->snmpv3_privpassword;
        $query_host_args->{hostobject}                = $host;
        $query_host_args->{hostaddress}               = $host->ip;
        if ( $host->monitored_by->id != 1 ) {
            $query_host_args->{monitoringserver} = $host->monitored_by;
        }
    }
    else {

        my $hostname = $c->req->param("ip");
        if ( !$hostname ) {
            $c->stash( error => $c->loc("messages.noHostname") );
            return;
        }
        $query_host_args->{hostaddress} = $hostname;

        my $msid = $c->req->param("msid");

        if ( $c->req->param("hostid") ) {
            my $host = $c->model( $self->model_name )->find( $c->req->param("hostid") );
            if ($host) {
                $query_host_args->{hostobject} = $host;

                # Hosts used as a slave do not show the monitored_by field, but it needs to be checked via there
                if ( !$msid ) {
                    my $mc = $c->model("OpsviewSchema::Monitoringclusternodes")->search( { "host" => $host->id } )->first;
                    $msid = $mc->id if $mc;
                }
            }
        }

        my $snmp_version = $c->req->param("snmp_version");
        unless ($snmp_version) {
            $c->stash( error => $c->loc("messages.noSnmpVersion") );
            return;
        }

        my @args;
        if ( $snmp_version eq "2c" or $snmp_version eq "1" ) {
            my $snmp_community = $c->req->param("snmp_community");
            if ( !$snmp_community ) {
                $c->stash( error => $c->loc("messages.noSnmpCommunity") );
                return;
            }
            $query_host_args->{snmp_auth}->{community} = $snmp_community;
        }
        else {
            $query_host_args->{snmp_auth}->{username}     = $c->req->param("snmpv3_username");
            $query_host_args->{snmp_auth}->{authprotocol} = $c->req->param("snmpv3_authprotocol");
            $query_host_args->{snmp_auth}->{authpassword} = $c->req->param("snmpv3_authpassword");
            if ( $c->req->param("snmpv3_privpassword") ) {
                $query_host_args->{snmp_auth}->{privprotocol} = $c->req->param("snmpv3_privprotocol");
                $query_host_args->{snmp_auth}->{privpassword} = $c->req->param("snmpv3_privpassword");
            }
        }
        $query_host_args->{snmp_version} = $snmp_version;
        $query_host_args->{snmp_port}    = $c->req->param("snmp_port");

        if ($msid) {
            my $ms = $c->model("OpsviewDB::Monitoringserver")->retrieve($msid);
            $query_host_args->{monitoringserver} = $ms;
        }

    }

    if ( $c->req->param("testconnection") ) {
        $query_host_args->{testconnection} = 1;
    }

    if ( $c->req->param('tidy_ifdescr_level') ) {
        $query_host_args->{tidy_ifdescr_level} = $c->req->param('tidy_ifdescr_level');
    }

    if ( $c->req->param('snmp_max_msg_size') ) {
        $query_host_args->{snmp_max_msg_size} = $c->req->param('snmp_max_msg_size');
    }

    my $reply = Opsview::Utils::QueryHost->run($query_host_args);

    if ( $c->debug ) {
        my $command = $reply->{command};
        $c->log->debug("Running cmd=@$command");
        $c->log->debug( "Reply=" . Data::Dump::dump($reply) );
    }

    if ( my $err = $reply->{stderr} ) {
        $c->log->error("query_host error: $err");
        $c->stash( error => $c->loc( "messages.queryHost.error: [_1]", $err ) );
        return;
    }

    if ( !defined $reply->{output} ) {
        $c->stash( error => $c->loc("messages.queryHost.emptyOutput") );
        return;
    }

    if ( my $parse_error = $reply->{parse_error} ) {
        if ( $parse_error eq "invalidXML" ) {
            $c->stash( error => $c->loc( "messages.queryHost.invalidXMLData: [_1]", $reply->{output} ) );
        }
        elsif ( $parse_error eq "incorrectOutput" ) {
            $c->stash( error => $c->loc("messages.queryHost.incorrectOutput") );
        }
        return;
    }

    if ( $reply->{error} ) {
        $c->stash( error => $reply->{error} );
        return;
    }

    # Will get a hash like:
    # { sysDescr => "system description",
    #   interface => {
    #     1 => { ifAlias => "", ifLink => "up", ifSpeed => "10Mb/s", ifStatus => "up", ifDescr => "lo" },
    #     2 => { ...}
    #     ...
    #   },
    # }
    my $sys_descr = $reply->{system_description};
    $c->stash( system_description => $sys_descr );

    # Short circuit if we are only testing connection
    if ( $c->req->param("testconnection") ) {
        return;
    }

    my $warnings = $reply->{warnings};

    my $r          = [];
    my $interfaces = $reply->{interfaces};
INTERFACE: foreach my $interface_hash (@$interfaces) {
        my $extra_info = $c->loc( "ui.host.edit.snmp.interface.extraInfo [_1] [_2] [_3] [_4]", $interface_hash->{ifAlias}, $interface_hash->{ifSpeed}, $interface_hash->{ifLink}, $interface_hash->{ifStatus} );
        my $new_hash = { %$interface_hash, extra_info => $extra_info };
        push @$r, $new_hash;
    }
    $c->stash( interfaces => $r );

    if ( my $blank_interface_names = $reply->{blank_interface_names} ) {
        foreach my $interfaceid (@$blank_interface_names) {
            push @$warnings, $c->loc( "ui.host.edit.snmp.interface.blankNameForId [_1]", $interfaceid );
        }
    }

    if (@$warnings) {
        $c->stash( warn => join( "; ", @$warnings ) );
    }
}

sub menu : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( services => $c->stash->{object}->resolved_servicechecks );
    $c->stash( template => "menu" );
    $c->stash( wrapper  => "wrappers/menu" );
}

# Converts snmpinterface input parameters into the hash ref format for insert
sub convert_snmpinterface_input {
    my ( $self, $attrs ) = @_;
    my @new_interfaces        = ();
    my $snmpinterfaces_all    = convert_to_arrayref( delete $attrs->{snmpinterfaces_all} );
    my $throughput_criticals  = convert_to_arrayref( delete $attrs->{throughput_critical} );
    my $throughput_warnings   = convert_to_arrayref( delete $attrs->{throughput_warning} );
    my $errors_criticals      = convert_to_arrayref( delete $attrs->{errors_critical} );
    my $errors_warnings       = convert_to_arrayref( delete $attrs->{errors_warning} );
    my $discards_criticals    = convert_to_arrayref( delete $attrs->{discards_critical} );
    my $discards_warnings     = convert_to_arrayref( delete $attrs->{discards_warning} );
    my $indexids              = convert_to_arrayref( delete $attrs->{indexid} );
    my $actives               = convert_to_arrayref( $attrs->{snmpinterfaces} );
    my %active_snmpinterfaces = map { ( $_ => 1 ) } @$actives;
    foreach my $int (@$snmpinterfaces_all) {
        my $hash = {
            name                => $int,
            active              => $active_snmpinterfaces{$int} || 0,
            indexid             => shift @$indexids,
            throughput_warning  => shift @$throughput_warnings,
            throughput_critical => shift @$throughput_criticals,
            errors_warning      => shift @$errors_warnings,
            errors_critical     => shift @$errors_criticals,
            discards_warning    => shift @$discards_warnings,
            discards_critical   => shift @$discards_criticals,
        };
        push @new_interfaces, $hash;
    }
    $attrs->{snmpinterfaces} = \@new_interfaces;
}

sub interfaces : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;

    my $host = $c->stash->{object};

    if ( $c->req->method eq "POST" ) {
        my $attrs = { %{ $c->req->params } };
        $self->convert_snmpinterface_input($attrs);
        $attrs->{id} = $host->id;
        if ( $c->debug ) { warn( Data::Dump::dump($attrs) ) }
        $c->stash->{model_class}->synchronise( $attrs, { username => $c->user->username } );
        $self->meta_redirect($c);
    }
    else {

        # Get list
        $c->stash( interfaces => [ $c->stash->{object}->snmpinterfaces ] );

        $c->stash( page_title   => " > " . $c->loc("ui.objecttype.host") . ": " . $host->name . " > " . $c->loc("ui.admin.host.queryhost.title.editHostInterfaces") );
        $c->stash( help_section => "snmpinterfaces" );

        push @{ $c->stash->{page_view_options_links} },
            {
            link => $c->uri_for( "/admin/host/edit",          $c->stash->{object}->id )->path_query,
            name => $c->loc( "ui.admin.list.label.edit [_1]", $c->stash->{object}->name ),
            };
    }

}

1;
