#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Hostgroup;

use Moose;
use namespace::autoclean;

BEGIN { extends qw/ Opsview::Web::ControllerBase::Admin / }

__PACKAGE__->config(
    model_name             => 'OpsviewSchema::Hostgroups',
    draggables_list_method => "hosts",
    default_rows           => 100,
    page_search_box        => 1,
    page_add_new           => 1,
);

before 'auto' => sub {
    my ( $self, $c, @args ) = @_;
    $c->stash( title_description => $c->loc("ui.objecttype.hostgroup") );
};

before 'list' => sub {
    my ( $self, $c ) = @_;
    $c->stash( title_description => $c->loc("ui.admin.hostgroup.list.title") );

    # Only list leaves
    $c->stash->{list_search_criteria} = { lft => \"= rgt - 1" };

    # This used to add a join so that a search would also search the hostnames that are in the hostgroup.
    # However, the sql generated by Class::DBI::Sweet errors on mysql5 (the left join needs to be positioned
    # so that it joins with __TABLE(me)__ but you cannot change the order). You could use a custom SQL
    # But since this is not a killer feature, we'll just take this out
    #if (my $q = $c->req->param("q")) {
    #	$c->stash->{list_search_criteria} = { "hosts.name" => [ map { "%".$_."%" } split(" ", $q) ] };
    #}
};

sub change_hosts : Local : Args(0) {
    my ( $self, $c ) = @_;
    my ( $hg, $host );

    $c->forward("/admin/check_save_access");
    if ( $c->req->param("hgid") ) {
        $hg = Opsview::Hostgroup->retrieve( $c->req->param("hgid") );
    }
    if ( $c->req->param("hostid") ) {
        $host = Opsview::Host->retrieve( $c->req->param("hostid") );
    }

    if ( $hg && $host ) {
        my $old_hg = $host->hostgroup;
        $host->hostgroup($hg);
        $host->uncommitted(1);
        $host->update;
        Opsview::Auditlog->create(
            {   username => $c->user->username,
                text     => $c->translate( "messages.auditlog.hostgroup.updatingHost [_1] [_2] [_3]", $host->identity_string, $old_hg->name, $hg->name ),
            }
        );
        $c->stash( object => $hg );
    }
    $c->stash( template => "change_draggables" );
    $c->stash( wrapper  => "wrappers/blank" );
}

# All these routines below are for the hierarchical views
sub graph : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( top      => $c->model( $self->model_name )->find(1) );
    $c->stash( template => "graph" );
    $c->stash->{page_title} .= ' > ' . $c->loc('ui.admin.hostgroup.graph.title');
    $c->stash->{help_page_keyword} = "host_groups_hierarchy";
}

sub insert : Local : Args(0) {
    my ( $self, $c ) = @_;
    my ( $hierarchy_group, $name );

    $c->forward("/admin/check_save_access");
    my $m  = $c->model( $self->model_name );
    my $id = $c->req->param("parentid");

    if ($id) {
        $hierarchy_group = $m->retrieve($id);
    }
    $name = $c->req->param("name");

    $c->stash( wrapper => "wrappers/blank" );    # Needs to be set before the insert because of possible errors at insert
    if ( $hierarchy_group && $m->sql_duplicate_leaf_name->select_val($name) ) {
        $c->stash->{error} = $c->loc( "ui.admin.hostgroup.error.leafNameAlreadyUsed [_1]", $name );
    }
    else {
        eval { $m->insert( { parentid => $id, name => $name, uncommitted => 1 } ) };
        if ($@) {
            $c->forward( "/translate_error", [$@] );
        }
        else {
            Opsview::Auditlog->create(
                {   username => $c->user->username,
                    text     => $c->translate( "messages.auditlog.hostgroup.insertNewHostgroupIntoParent [_1] [_2]", $name, $hierarchy_group->name ),
                }
            );
        }
    }
    $c->forward("graph");
    $c->stash( template => "graph_diagram" );
}

sub rename : Local : Args(0) {
    my ( $self, $c ) = @_;
    my ( $hierarchy_group, $name );

    $c->forward("/admin/check_save_access");
    my $m  = $c->model( $self->model_name );
    my $id = $c->req->param("nodeid");

    $name = $c->req->param("name");

    $c->stash( wrapper => "wrappers/blank" );    # Needs to be set before the insert because of possible errors at insert
    if ( $id && $name ) {

        # search to make sure name isnt already used
        if ( $m->sql_duplicate_leaf_name->select_val($name) ) {
            $c->stash->{error} = $c->loc( "ui.admin.hostgroup.error.leafNameAlreadyUsed [_1]", $name );
        }
        else {
            my $group = $m->retrieve($id);
            my $audit_text = $c->translate( "messages.auditlog.hostgroup.renameHostgroup [_1] [_2]", $group->identity_string, $name );
            $group->name($name);
            eval { $group->update };
            if ($@) {
                $c->forward( "/translate_error", [$@] );
            }
            else {
                Opsview::Auditlog->create(
                    {   username => $c->user->username,
                        text     => $audit_text,
                    }
                );
            }
        }
    }
    $c->forward("graph");
    $c->stash( template => "graph_diagram" );
}

sub remove : Local {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    my $group = $c->model( $self->model_name )->retrieve( $c->req->param("group") );

    # add in as an extra step to make sure group exists to delete, rather than
    # propagate error to UI when it doesnt
    if ($group) {
        my $audit_text = $c->translate( "messages.auditlog.hostgroup.deleteHostgroup [_1]", $group->identity_string );
        $group->delete;
        Opsview::Auditlog->create(
            {   username => $c->user->username,
                text     => $audit_text,
            }
        );
    }
    $c->stash( wrapper => "wrappers/blank" );
    $c->forward("graph");
    $c->stash( template => "graph_diagram" );
}

sub adopt : Local {
    my ( $self, $c ) = @_;
    my ( $group, $parent );
    $c->forward("/admin/check_save_access");
    if ( my $groupid = $c->req->param("group") ) {
        $group = $c->model( $self->model_name )->retrieve($groupid);
    }
    if ( my $parentid = $c->req->param("parent") ) {
        $parent = $c->model( $self->model_name )->retrieve($parentid);
    }
    if ( $group && $parent ) {
        my $audit_text = $c->translate( "messages.auditlog.hostgroup.movingHostgroup [_1] [_2]", $group->name, $parent->name );
        $group->parentid($parent);
        $group->uncommitted(1);
        $group->update;
        Opsview::Auditlog->create(
            {   username => $c->user->username,
                text     => $audit_text,
            }
        );
    }
    $c->stash( wrapper => "wrappers/blank" );
    $c->forward("graph");
    $c->stash( template => "graph_diagram" );
}

sub menu : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, $id ) = @_;
    if ( $c->request->headers->referer =~ m/graph$/ ) {
        $c->stash( show_graph_options => 1 );
    }
    my $count_hosts = $c->model( $self->model_name )->find($id)->count_related("hosts");
    $c->stash( count_hosts => $count_hosts );
    $c->stash( template    => "menu" );
    $c->stash( wrapper     => "wrappers/menu" );
}

1;
