#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Notifications;

use strict;
use warnings;

use base qw/Catalyst::Controller/;

__PACKAGE__->config(
    model_name        => "Odw::Queries",
    title_description => "Notifications",
    default_rows      => 50,
);

sub auto : Private {
    my ( $self, $c, $id ) = @_;

    unless ( $c->user_exists && ( $c->user->has_access("ADMINACCESS") || $c->stash->{admin_override} ) ) {
        $c->stash( template => 'access_denied' );
        return 0;
    }

    $c->stash( title_description => "Notifications" );
    $c->stash( additional_template_paths => [ $c->path_to( 'root', $c->action->namespace )->stringify, $c->path_to( 'root', 'admin' )->stringify ] );

    # namespace used for contextual help
    $c->stash( namespace => $c->action->namespace );

    push( @{ $c->stash->{stylesheets} }, 'admin.css' );

    my $from = $c->req->params->{from} || $c->req->referer;

    $c->stash( from => $from );

    #$c->stash(add_line_numbers => \&add_line_numbers );
    #$c->stash(round => \&round );

    return 1;
}

=item list

Allowable query parameters:

=over 3

=item host

The hostname to search for. Repeatable

=item service

The service name to search for. Repeatable

If a hostname is selected, will also show host notifications. If hostname is not provided,
will only show service notifications.

=item start

Start date, in YYYY-MM-DD format. Default end - 7 days.

=item end

End date, in YYYY-MM-DD format. Default today.

=item page

Page number

=cut

use Data::Page;
use DateTime;
use DateTime::Format::Strptime;

sub list : Path {
    my ( $self, $c ) = @_;
    my $params   = $c->req->params;
    my $req_page = $params->{page} || 1;
    my $rows     = $params->{rows} || 200;
    if ( $req_page eq "all" ) {
        $req_page = 1;
        $rows     = 10000;
    }

    my $args = {};

    my $parser = DateTime::Format::Strptime->new( pattern => "%F %T", time_zone => "local" );
    my $dt;
    if ( my $end = $c->req->params->{end} ) {
        $dt = $parser->parse_datetime($end);
        if ($dt) {
            $args->{end} = $dt->epoch;
        }
    }
    unless ( $args->{end} ) {
        $args->{end} = time;
    }
    $c->stash( end_dt => DateTime->from_epoch( epoch => $args->{end}, time_zone => "local" ) );
    if ( my $start = $c->req->params->{start} ) {
        $dt = $parser->parse_datetime($start);
        if ($dt) {
            $args->{start} = $dt->epoch;
        }
    }
    unless ( $args->{start} ) {
        $args->{start} = $args->{end} - ( 7 * 24 * 60 * 60 );
    }
    $c->stash( start_dt => DateTime->from_epoch( epoch => $args->{start}, time_zone => "local" ) );

    $args->{host}        = [ $c->req->params->{host} ]    if defined $c->req->params->{host};
    $args->{service}     = [ $c->req->params->{service} ] if defined $c->req->params->{service};
    $args->{contactname} = [ $c->req->params->{contact} ] if defined $c->req->params->{contact};
    $args->{methodname}  = [ $c->req->params->{method} ]  if defined $c->req->params->{method};
    $args->{status}      = [ $c->req->params->{status} ]  if defined $c->req->params->{status};

    my $page    = Data::Page->new;
    my $results = Odw::Queries->list_notifications($args);
    my $count   = scalar @$results;
    if ($count) {
        $page->total_entries( scalar @$results );
        $page->entries_per_page($rows);
        $page->current_page($req_page);

        $c->stash( page => $page );
        my @visible = $page->splice($results);
        $c->stash( list => \@visible );
        $c->stash( page_link => sub { my $page = shift; return create_page_link( $c, $page, $params ) } );
    }
    $c->stash( count    => $page->total_entries );
    $c->stash( template => "list_layout" );
}

# See /usr/local/share/perl/5.8.4/Catalyst/Example/Controller/InstantCRUD.pm
# for more information about parameters. We assume no other parameters used
sub create_page_link {
    my ( $c, $page, $origparams ) = @_;
    my %params = %$origparams;
    $params{page} = $page;

    # Need eq because page could be "all"
    if ( $page eq $c->stash->{page}->current_page ) {
        return $page;
    }
    else {
        use HTML::Entities;
        return '<a href="' . decode_entities( $c->uri_for_params( "", { page => $page } ) ) . '">' . $page . '</a>';
    }
}

1;
