#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Reload;

use Moose;
use namespace::autoclean;

BEGIN { extends 'Catalyst::Controller::ActionRole' }

use Opsview::Nagios;
use Opsview::Run;
use Opsview::Reloadmessage;
use Fcntl;

my $reload_flag        = "/usr/local/nagios/var/rw/reload_flag";
my $gen_error          = "/usr/local/nagios/var/rw/config_output";
my $rrd_migration_flag = "/usr/local/nagios/var/rrd_migration.flag";

sub auto : Private {
    my ( $self, $c ) = @_;

    $c->stash( additional_template_paths => [ $c->path_to( 'root', $c->action->namespace )->stringify, $c->path_to( 'root', 'admin' )->stringify ] );
    push( @{ $c->stash->{stylesheets} }, 'admin.css' );

    $c->stash->{functions}->{parseInterval} = \&Time::Interval::parseInterval;
    $c->stash->{page_title} .= ' > ' . $c->loc('ui.admin.reload.title');
    $c->stash( page_search_box => 0 );

    return 1;
}

sub opsview : Path : Does(ACL) : RequiresRole(RELOADACCESS) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    if ( -e $rrd_migration_flag ) {
        $c->stash( rrd_migration_flag => 1 );
    }
    $c->stash( average_time => Opsview::Reloadtime->average_duration );

    my $last_backup_auditid = $c->model("OpsviewSchema::Auditlogs")->search( { reloadid => { ">" => 0 } }, { rows => 1, order_by => { "-desc" => "id" } } )->first;
    if ($last_backup_auditid) {
        my $changes = $c->model("OpsviewSchema::Auditlogs")->search( { "username" => { "!=" => "" }, "id" => { ">" => $last_backup_auditid->id } } )->count;
        $c->stash( number_of_auditlog_entries => $changes );
    }

    $c->stash( template => "opsview" );
}

sub do_reload : Local : Does(ACL) : RequiresRole(RELOADACCESS) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    unless ( sysopen( FLAG, $reload_flag, O_WRONLY | O_EXCL | O_CREAT ) ) {

        # For the API, a RETRY is returned, so the top level program can decide what to do next
        # For a web page, it will flick into a reloading state. But this is okay, because
        # a different invocation of reload has occurred, so the web ui doesn't need to change
        my $audit_text = $c->translate( "messages.auditlog.reload.invokedFromIp [_1]", $c->req->address );
        $c->log->warn($audit_text);
        Opsview::Auditlog->create( { username => $c->user->username, text => $audit_text } );
        $c->stash->{web_status} = { status => "RETRY" };
    }
    else {
        close FLAG;

        # File gets removed as part of rc.opsview web_reload
        if ( exists $c->stash->{synchronous} ) {
            my $audit_text = $c->translate("messages.auditlog.reload.runningSynchronous");
            $c->log->info($audit_text);
            Opsview::Auditlog->create( { username => $c->user->username, text => $audit_text } );
            system( "/usr/local/nagios/bin/rc.opsview", "web_reload" );
            $c->stash->{web_status} = Opsview::Nagios->web_status( { returns => "hash" } );
        }
        else {
            my $audit_text = $c->translate("messages.auditlog.reload.runningAsynchronous");
            $c->log->info($audit_text);
            Opsview::Auditlog->create( { username => $c->user->username, text => $audit_text } );
            my ( $rc, $out, $err ) = Opsview::Run->run_command( "/usr/local/nagios/bin/send_opsview_cmd", "web_reload" );
            if ( $rc != 0 ) {
                unlink($reload_flag) if ( -f $reload_flag );
                my $error_messages = join( "", @$err );
                Opsview::Reloadmessage->create(
                    {   utime    => time,
                        message  => $c->translate( "messages.auditlog.reload.cannotReload [_1]", $error_messages ),
                        severity => "critical",
                    }
                );
            }
        }
    }
    $c->res->body(" ");    # Do nothing
}

sub do_restart : Local : Does(ACL) : RequiresRole(RELOADACCESS) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    Opsview::Auditlog->create( { username => $c->user->username, text => $c->translate("messages.auditlog.reload.restartingWebServer") } );
    system( "/usr/local/nagios/bin/rc.opsview", "web_restart" );
    $c->res->body(" ");
}

sub config_errors : Local : Does(ACL) : RequiresRole(RELOADACCESS) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    $c->stash( wrapper => "wrappers/blank" );
    my $iterator = Opsview::Reloadmessage->retrieve_all;
    $c->stash( messages => $iterator );
    if ( -e $gen_error ) {
        open ERRORS, "$gen_error";
        my $errors;
        { local $/ = undef; $errors = <ERRORS>; }
        close ERRORS;
        $c->stash( gen_error => $errors );
    }
}

1;
