#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Report;

use Moose;
use namespace::autoclean;

BEGIN { extends qw/ Opsview::Web::ControllerBase::Admin / }

__PACKAGE__->config(
    model_name        => 'ReportsDB::Report',
    title_description => "Report",
    set_many_on_off   => [ { name => "hostgroups", parameters => [qw/warning critical/], }, ],
    zero_if_empty     => [ qw/scheduled/, ],
    page_add_new      => 1,
);

sub clone { }

after 'auto' => sub {
    my ( $self, $c, @args ) = @_;
    $c->stash( tabs => [qw(Report Selection Schedule)] );
};

after 'create_new' => sub {
    my ( $self, $c ) = @_;
    $c->stash( tabs        => [qw(Report empty)] );
    $c->stash( submit_name => "next" );
};

after 'edit' => sub {
    my ( $self, $c, @args ) = @_;
    $self->custom_setup( $c, $c->stash->{object}, $c->stash->{object}->selection );
    if ( my $p = $c->req->param("tab") ) {
        if ( $p =~ /^[1-9]$/ ) {
            $p = lc( $c->stash->{tabs}[ $p - 1 ] );
        }
        $c->stash( "initial_tab" => "tab-$p" );
    }
};

before 'save' => sub {
    my ( $self, $c, @args ) = @_;

    # Set hostgroups_all be same here, rather than in view
    # Needed for auditing in ControllerBase/Admin.pm
    my @a = $c->req->param("hostgroups");
    $c->req->param( hostgroups_all => @a );

    if ( $c->action eq "admin/report/save_new" ) {
        my $typename = $c->stash->{object}->type->fixedname;

        # Bit of magic here - creates the object in the might_have relationship table
        my $fk = $c->stash->{object}->meta_info->{might_have}->{$typename}->{foreign_class};
        $fk->create( { reportid => $c->stash->{object}->id } ) if defined $fk;

        $c->log->debug("Save new - redirecting to 2nd tab");
        my @action = $c->stash->{object}->actionname;
        $c->stash( from => $c->uri_for( "/admin/report", @action, "edit", $c->stash->{object}->id, { tab => 2, from => $c->stash->{from} } ) );
    }
};

sub custom_setup : Private {
    my ( $self, $c, $report, $selection ) = @_;
    $c->stash( hostgroups => $report->custom($selection) );
}

sub custom_information : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    if ( defined $c->req->params->{selection} ) {
        my $selectionid = $c->req->params->{selection};
        my $selection   = $c->model("ReportsDB::Selection")->retrieve($selectionid);
        unless ($selection) {
            my $text = $c->loc("messages.error.noSelection");
            $c->stash( error => $text );
            $c->forward("/soft_error");
        }

        #$c->forward("custom_setup", $c->stash->{object}, $selection);
        $self->custom_setup( $c, $c->stash->{object}, $selection );
        $c->stash( template => "hostgroup_extra" );
        $c->stash( wrapper  => "wrappers/blank" );
    }
}

sub menu : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( wrapper => "wrappers/menu" );
}

sub regenerate : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
}

use MIME::Lite;

sub do_regenerate : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $o = $c->stash->{object};
    if ( exists $c->req->params->{annotation} && $o->capability("annotations") ) {
        $o->annotation( $c->req->params->{annotation} );
    }
    $o->update;
    my $start_date = $c->req->params->{report_date};
    my $audit_text = $c->translate( "messages.auditlog.regeneratingReportWithStartDate [_1] [_2]", $c->stash->{object}->name, $start_date );
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $audit_text,
        }
    );

    my $filename = "/tmp/report.$$";
    {
        local $SIG{CHLD} = 'DEFAULT';
        my $reportid    = $o->id;
        my $report_type = $o->type->invoker;
        my @cmd         = ( "/usr/local/nagios/bin/run_opsview_report", "-S$start_date", "-N$reportid", "-f$filename", "-t$report_type" );
        $c->log->debug("Running command: @cmd");
        system(@cmd) == 0 or die "Error invoking reports";
        $c->log->debug("Finished running command");
    }
    my $email = $c->req->params->{email};
    $c->log->debug("Got email of $email");
    if ($email) {
        my $name = $o->name;
        my $msg = MIME::Lite->new( To => $email, Subject => "Report: $name", Type => "multipart/mixed" );
        $msg->attach( Path => $filename, Filename => "report.pdf", Type => "application/pdf" );
        $msg->send;
        $c->log->debug("Sent message");
    }
    unlink $filename;
    $c->res->body(" ");
}

# This is the initial page, which then calls via ajax....
sub start_generate : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
}

# ... this page actually starts the generation
sub generate : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    unless ( $c->stash->{object} ) {
        my $text = $c->loc("messages.error.invalidReportId");
        $c->stash( error => $text );
        $c->detach("/soft_error");
    }
    $c->forward("adhoc");
}

use Opsview::Run;

sub adhoc : Private {
    my ( $self, $c ) = @_;

    my $reportid = $c->stash->{object}->id;

    my $value = $c->snippet( adhoc_report_progress => $reportid );
    if ( defined $value && $value =~ /^\d+$/ ) {
        my $text = $c->loc( "messages.error.reportIdAlreadyRunning [_1]", $reportid );
        $c->stash( error => $text );
        $c->detach("/soft_error");
    }
    $c->snippet( adhoc_report_progress => $reportid => 0 );

    my $audit_text = $c->translate( "messages.report.runningAdhocFor [_1]", $c->stash->{object}->name );
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $audit_text,
        }
    );

    my $filename = "/tmp/report.$$";
    {
        local $SIG{CHLD} = 'DEFAULT';

        my $progress = 0;
        my $error;
        my $output;
        my $report_type = $c->stash->{object}->type->invoker;
        my @cmd = ( "/usr/local/nagios/bin/run_opsview_report", "-N$reportid", "-f$filename", "-t$report_type" );
        $c->log->debug("Running command: @cmd");
        my $rc = run_command_subcap(
            \@cmd,
            sub {
                $output .= shift;
                $progress = $output =~ tr/\n//;
                $c->snippet( adhoc_report_progress => $reportid => $progress );
            },
            sub {
                $error .= shift;
            },
        );
        if ($rc) {
            my $text = $c->loc("Error running report") . "\n";
            if ($output) {
                $text .= "Output: $output";
            }
            if ($error) {
                $text .= "Error: $error";
            }
            $c->snippet( adhoc_report_progress => $reportid => $text );
            $c->res->body(" ");
            $c->detach;
        }
    }

    $c->snippet( adhoc_report_progress => $reportid => $filename );
    $c->res->body(" ");
}

sub poll : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $reportid = $c->stash->{object}->id;

    my $value = $c->snippet( "adhoc_report_progress" => $reportid );
    if ( !defined $value ) {
        $c->stash->{error} = "Report not being generated";
        $c->forward("/soft_error");
    }
    elsif ( $value =~ m#^/.*# ) {
        $c->stash( output => "json" );
        $c->stash( ResultSet => { state => "end" } );
    }
    elsif ( $value =~ /^\d+$/ ) {
        $c->stash( output => "json" );
        $c->stash( ResultSet => { state => "running", progress => $value } );
    }
    else {
        $c->stash( output => "json" );
        $c->stash( ResultSet => { state => "failed", detail => $value } );
    }
}

sub show_report : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $reportid = $c->stash->{object}->id;
    my $value = $c->snippet( "adhoc_report_progress" => $reportid );
    unless ( defined $value && $value =~ m#^/# ) {
        $c->stash->{error} = "Report not generated properly: $value";
        $c->detach("/soft_error");
    }
    my $filename = $value;
    unless ( -r $filename ) {
        $c->stash->{error} = "Cannot read file $filename";
        $c->detach("/soft_error");
    }
    open F, "$filename" or die "Can't read $filename";
    my $output;
    { undef $/; $output = <F>; }
    close F;
    $c->res->header( 'Content-Disposition', qq[attachment; filename="report.pdf"] );
    $c->res->body($output);
    $c->res->content_type("application/pdf");
}

1;
