#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Servicecheck;

use Moose;
use namespace::autoclean;
use Try::Tiny;

BEGIN { extends qw/Opsview::Web::ControllerBase::Admin/ }

__PACKAGE__->config(
    model_name      => 'OpsviewSchema::Servicechecks',
    set_commasep    => [qw/notification_options/],
    set_many_words  => [qw/keywords/],
    default_rows    => 100,
    page_search_box => 1,
    page_add_new    => 1,
    search_column   => [qw(name plugin)],
    use_synchronise => {
        invertresults          => 0,
        flap_detection_enabled => 0,
        check_freshness        => 0,
        markdown_filter        => 0,
        calculate_rate         => 0,
        dependencies           => [],
        notification_options   => [],
    },
);

sub auto : Private {
    my ( $self, $c, @args ) = @_;
    $c->stash( title_description => $c->loc("ui.objecttype.servicecheck") );
    $self->next::method( $c, @args );
    my $tabs = [ "Service Check", "Notifications", "Advanced" ];
    $c->stash( tabs => $tabs );
    $c->forward("/common/nagios_states");
}

sub do_synchronise {
    my ( $self, $c, $attrs ) = @_;
    if ( ( my $new_sg = delete $attrs->{servicegroup_new} ) && $c->user->has_access("ADMINACCESS") ) {
        my $sg;
        try {
            $sg = $c->model("OpsviewSchema::Servicegroups")->find_or_create( { name => $new_sg } );
        }
        catch {
            $self->handle_exception( $c, $_ );
        };
        $attrs->{servicegroup} = $sg->id;
    }
    if ( defined $attrs->{notification_interval} && $attrs->{notification_interval} eq "" ) {
        $attrs->{notification_interval} = undef;
    }
    if ( $c->action->name eq "save_new" && $c->stash->{clone_object} ) {
        my @cloned_rules;
        my $cloned_object = $c->stash->{clone_object}->serialize_to_hash( { columns => ["snmptraprules"] } );
        foreach my $rule ( @{ $cloned_object->{snmptraprules} } ) {
            my $newrule = {%$rule};
            delete $newrule->{id};
            delete $newrule->{uncommitted};
            push @cloned_rules, $newrule;
        }
        $attrs->{snmptraprules} = \@cloned_rules;
    }

    $self->next::method( $c, $attrs );

    # Redirect to the rule page, if an SNMPtrap type
    if ( $c->action->name eq "save_new" && $c->stash->{object}->checktype->id == 4 ) {
        $c->stash->{from} = $c->uri_for( "/admin/snmptraprule/list", { servicecheck => $c->stash->{object}->id } );
    }
}

before 'redirect_to_list' => sub {
    my ( $self, $c ) = @_;
    $_ = $c->req->params->{name};

    # Fancy trick to redirect embeddedservices to their own page, rather than servicechecks
    # This needs to be updated for host variables
    if ( defined $_ && /^([^:]+?):/ ) {
        $c->req->params->{name} = $1;
    }
};

sub minilist : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c, $id, $type ) = @_;

    my $o = $c->stash->{object};
    if ( $type eq "hosts" ) {
        my $list = $o->all_hosts;
        $c->stash( list       => $list );
        $c->stash( page_title => "hosts" );
    }
    if ( $type eq "exceptions" ) {
        my @list;
        foreach my $o ( $o->retrieve_all_exceptions_by_service($id) ) {
            push( @list, $o->{h_id} );
        }
        $c->log->debug("id=$id");
        $c->stash( list       => \@list );
        $c->stash( page_title => "exceptions" );
    }
    $c->stash( template => "minilist" );
}

before 'edit' => sub {
    my ( $self, $c, @args ) = @_;
    $c->stash->{exceptions_count} = $c->model("OpsviewDB::Servicecheck")->count_all_exceptions_by_service( $args[0] );
};

sub edit_common {
    my ( $self, $c ) = @_;
    $c->stash->{all_servicechecks_iterator} = $c->model("OpsviewDB::Servicecheck")->search( { attribute => undef }, { sql_method => "joined_servicegroup", statement_order => [qw/from where order_by limit/], order_by => "servicegroups.name, me.name" } );
}

before 'list' => sub {
    my ( $self, $c ) = @_;
    $c->stash( title_description => $c->loc("ui.admin.servicecheck.list.title") );
    $c->stash->{page_attributes} = {
        join     => "servicegroup",
        order_by => "servicegroup.name, me.name",
        prefetch => ["servicegroup"],
    };

    # This used to add a join so that a search would also search the hostnames that are in the hostgroup.
    # However, the sql generated by Class::DBI::Sweet errors on mysql5 (the left join needs to be positioned
    # so that it joins with __TABLE(me)__ but you cannot change the order). You could use a custom SQL
    # But since this is not a killer feature, we'll just take this out
    #if (my $q = $c->req->param("q")) {
    #       $c->stash->{list_search_criteria} = { "hosts.name" => [ map { "%".$_."%" } split(" ", $q) ] };
    #}

};

sub menu : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( exceptions => $c->stash->{object}->count_all_exceptions_by_service( $c->stash->{object}->id ) );
    $c->stash( template   => "menu" );
    $c->stash( wrapper    => "wrappers/menu" );
}

sub info_for_monitors_list : Local : Args(1) : Does(ACL) : AllowedRole(CONFIGUREVIEW) : AllowedRole(CONFIGUREHOSTS) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( via                                => scalar $c->req->param("via") );
    $c->stash( checked                            => ( ( scalar $c->req->param("checked") ) eq "true" ? 1 : 0 ) );
    $c->stash( state                              => $c->req->params->{state} );
    $c->stash( central_servicecheck_configuration => 1 );
    $c->stash( wrapper                            => "wrappers/blank" );
}

1;
