#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Snmptrap;

use Moose;
use namespace::autoclean;

BEGIN { extends qw/Opsview::Web::ControllerBase::Admin/ }

__PACKAGE__->config(
    model_name        => 'OpsviewDB::Snmptrap',
    title_description => "SNMP Trap",
);

sub summary : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    $c->stash( title_description => $c->loc("ui.admin.snmptrap.summary.title") );
    my @sc = Opsview::Servicecheck->search( checktype => 4 );
    $c->stash( hosts                    => Opsview::Host->with_advanced_snmptrap_arrayref );
    $c->stash( servicechecks            => \@sc );
    $c->stash( snmptrapexceptions_count => Opsview::Snmptrapexception->count_all );
    $c->stash( snmptrapdebug_count      => Opsview::Snmptrapdebug->count_all );
    $c->stash( hosts_tracing            => Opsview::Host->search( snmptrap_tracing => 1 )->count );
    $_ = $c->flash->{number_of_saved_tracing_hosts};
    $_ = "" unless ( defined $_ );
    $c->stash( number_of_saved_tracing_hosts => $_ );
    $c->stash->{page_title} = " > " . $c->stash->{title_description};
}

sub tracing : Local : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) : {
    my ( $self, $c ) = @_;
    my $hosts = Opsview::Host->with_advanced_snmptrap_arrayref;
    my @host_names = map { $_->name } @$hosts;
    $c->stash( hosts => \@host_names );
    my @hosts_with_tracing = Opsview::Host->search( snmptrap_tracing => 1 );
    $c->stash( hosts_with_tracing => \@hosts_with_tracing );
    $c->stash( title_description  => $c->loc("ui.admin.snmptrap.tracing.title") );
    $c->stash->{page_title}        = " > " . $c->stash->{title_description};
    $c->stash->{help_page_keyword} = "snmp_trap_tracing";
}

sub save_tracing : Local {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    my @hostnames = $c->req->param("hostname");
    $c->flash->{number_of_saved_tracing_hosts} = Opsview::Host->set_snmptrap_tracing( \@hostnames );
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $c->translate( "ui.admin.snmptrap.tracing.setForHosts [_1]", join( " ", @hostnames ) ),
        }
    );
    $c->res->redirect( $c->uri_for("summary") );
}

sub collect : Local {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    my $hostname = $c->req->param("hostname");
    unless ( defined $hostname ) {
        $c->res->body( $c->loc("messages.noHostname") );
        $c->detach;
    }
    my $host = Opsview::Host->search( name => $hostname )->first;
    unless ($host) {
        $c->res->body( $c->loc("messages.host.notFound [_1]") );
        $c->detach;
    }
    my $ms  = $host->monitored_by;
    my @cmd = ("/usr/local/nagios/bin/collect_traps $hostname");
    if ( $ms->is_master ) {
        $self->collect_traps( $c, \@cmd );
    }
    else {
        foreach my $node ( $ms->nodes ) {
            my @sshcmd = $node->ssh_command(@cmd);
            $self->collect_traps( $c, \@sshcmd );
        }
    }

    my @traps = @{ $c->stash->{traps} };
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $c->translate( "messages.auditlog.collectingTrapsForHost [_1] [_2]", $hostname, scalar @traps ),
        }
    );
    unless (@traps) {
        $c->res->body( $c->loc("ui.admin.snmptrap.tracing.noTrapsCollected") );
        $c->detach;
    }

    local $SIG{CHLD} = 'DEFAULT';

    # This bit is effectively reprocess_snmptraps - that script should probably go
    # and be replaced by a controller
    my $tmp_file = "/tmp/reprocess_snmptraps.cfg.$$";
    unless ( system( "/usr/local/nagios/bin/nagconfgen.pl", "-s", $tmp_file ) == 0 ) {
        $c->res->body( $c->loc("ui.admin.snmptrap.tracing.cannotGenerateNewConfigurationFile") );
        $c->detach;
    }
    open F, "| /usr/local/nagios/bin/snmptrap2nagios -d -r -c $tmp_file";

    # ---next trap--- separator should already be in array
    print F @traps;
    close F;
    if ( $? != 0 ) {
        $c->res->body( $c->loc( "messages.generic.errorRunning [_1]", "snmptrap2nagios" ) );
        $c->detach;
    }
    $c->res->body( $c->loc( "ui.admin.snmptrap.tracing.trapsCollected [_1]", scalar @traps ) );
}

sub collect_traps : Private {
    my ( $self, $c, $cmd ) = @_;
    my @traps;
    local $SIG{CHLD} = 'DEFAULT';
    local $/ = "#---next trap---#\n";
    unless ( open F, "-|", @$cmd ) {
        $c->res->body( $c->loc( "messages.generic.cannotExecute [_1]", "collect_traps" ) );
        $c->detach;
    }
    push @traps, <F>;
    close F;
    if ($?) {
        $c->res->body( $c->loc( "messages.generic.errorRunning [_1]", "collect_traps" ) );
        $c->detach;
    }
    push @{ $c->stash->{traps} }, @traps;
}

1;
