#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Snmptrapexception;

use Moose;
use namespace::autoclean;

use Opsview::Run;

BEGIN { extends qw/Opsview::Web::ControllerBase::Admin/ }

__PACKAGE__->config(
    model_name      => 'OpsviewDB::Snmptrapexception',
    search_column   => "packet",
    page_search_box => 1,
);

# Unused
sub edit       { }
sub create_new { }
sub clone      { }
sub save_new   { }
sub save       { }

after 'auto' => sub {
    my ( $self, $c, @args ) = @_;
    $c->stash( scalar_localtime => sub { scalar localtime(shift) } );
};

# Override delete
sub delete : Local : Args(1) {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    if ( $c->req->param("filter") eq "unique" ) {
        my $o = $c->stash->{object};
        my $audit_text = $c->translate( "messages.auditlog.snmptrapexception.deleteSimilar [_1]", $o->identity_string );
        $o->delete_all_similar;
        Opsview::Auditlog->create(
            {   username => $c->user->username,
                text     => $audit_text,
            }
        );
        $c->forward('/admin/meta_redirect');
    }
    else {
        $self->next::method($c);
    }
}

before 'list' => sub {
    my ( $self, $c, $filter ) = @_;
    $c->stash( title_description => $c->loc("ui.objecttype.snmptrapexception") );
    if ( $filter && $filter eq "unique" ) {
        $c->stash->{page_attributes} = {
            sql_method         => "unique",
            statement_order    => [qw/from where limit order_by/],
            order_by           => "time desc",
            disable_sql_paging => 1,
        };
        $c->stash( filter => "unique" );
    }
    else {
        $c->stash( filter => "all" );
    }
    $c->stash( total => Opsview::Snmptrapexception->count_all );

    # Need to find all current promotions and create a hash so that web page doesn't get
    # hit with lots of polls
    my @keys       = $c->cache->get_keys(2);
    my $promotions = {};
    foreach my $hash (@keys) {
        $_ = $hash->{key};

        # WARNING: This is completely dependent on C::P::Snippets' naming convention
        my ($exception_id) = /^snippet:promote_mib_(\d+):state$/;
        $promotions->{$exception_id} = 1 if $exception_id;
    }
    $c->stash( promotions => $promotions );
};

sub reprocess : Local {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $c->translate("messages.auditlog.snmptrapexception.reprocessingAll"),
        }
    );
    system( "/usr/local/nagios/bin/send_opsview_cmd", "reprocess_snmptraps" );
    $c->res->body(" ");
}

sub promote_mib : Local : Args(1) {
    my ( $self, $c, $id ) = @_;
    $c->forward("/admin/check_save_access");
    my $value = $c->snippet( "promote_mib_$id", "state" );

    $c->res->body(" ");

    if ( $value && $value =~ /^running$/ ) {

        # no need to do anything
    }
    else {
        $c->snippet( "promote_mib_$id", "state", "running" );
        $c->forward( "run_promote_mib", [$id] );
    }
}

sub poll_promote_mib : Local : Args(1) {
    my ( $self, $c, $id ) = @_;
    $c->forward("/admin/check_save_access");

    $c->stash( output => "json" );

    my $value  = $c->snippet( "promote_mib_$id", "state" )  || "unknown";
    my $output = $c->snippet( "promote_mib_$id", "output" ) || "";

SWITCH: {
        foreach ($value) {
            /^(:?end|running|error)$/ && do {
                $c->stash(
                    ResultSet => {
                        state  => $value,
                        output => $output,
                    }
                );
                last SWITCH;
            };
            do {
                $c->stash( ResultSet => { state => "unknown", } );
                last SWITCH;
            };
        }
    }

    $c->res->body(" ");
}

sub run_promote_mib : Private {
    my ( $self, $c, $id ) = @_;

    my $trap = $c->model("OpsviewDB::Snmptrapexception")->retrieve($id);

    my $command;
    my $daemon_check;

    unless ($trap) {
        my $error = $c->loc( "ui.admin.snmptrapexception.help.noSnmpTrapExceptionById [_1]", $id );
        $c->snippet( "promote_mib_$id", "state",  "error" );
        $c->snippet( "promote_mib_$id", "output", $error );
        $c->detach;
    }

    if ( $trap->reason() == 7 ) {
        my $error = $c->loc("ui.admin.snmptrapexception.help.mibAlreadyPromoted");
        $c->snippet( "promote_mib_$id", "state",  "end" );
        $c->snippet( "promote_mib_$id", "output", $error );
        $c->detach;
    }

    my $oid = $trap->trapname;

    my ( $rc, $out, $err ) = Opsview::Run->run_command( "/usr/local/nagios/bin/send_opsview_cmd", "daemoncheck" );

    if ( $@ || $err ) {
        $c->snippet( "promote_mib_$id", "state",  "error" );
        $c->snippet( "promote_mib_$id", "output", @$err );
        $c->log->error( "Unable to promote MIB: ", $@, @$err );
        $c->detach;
    }

    $_ = $c->snippet( "promote_mib_$oid", "state" );
    if ( $_ && $_ eq "in progress" ) {
        my $error = $c->loc("ui.admin.snmptrapexception.help.mibAlreadyPromoted");
        $c->snippet( "promote_mib_$id", "state",  "end" );
        $c->snippet( "promote_mib_$id", "output", $error );
        $c->detach;
    }

    $c->snippet( "promote_mib_$oid", "state",  "in progress" );
    $c->snippet( "promote_mib_$id",  "output", "" );

    $out = $err = undef;

    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $c->translate( "messages.auditlog.snmptrapexception.runningPromoteMibForOid [_1]", $oid ),
        }
    );
    $rc = run_command_subcap(
        [ "/usr/local/nagios/bin/send_opsview_cmd", "promote_mib", $oid ],
        sub {

            #TODO: Should not have <pre> in this - do at the view
            my $output = $c->snippet( "promote_mib_$id", "output" );
            $output =~ s!</?PRE>!!g if ($output);
            $output .= $_[0];

            $c->snippet( "promote_mib_$id", "output", "<PRE>" . $output . "</PRE>" );
        },
        undef
    );

    $c->snippet( "promote_mib_$oid", "state", "" );

    if ( $@ || $err ) {
        $c->snippet( "promote_mib_$id", "state",  "error" );
        $c->snippet( "promote_mib_$id", "output", @$err );
        $c->log->error( "Unable to promote MIB: ", $@, @$err );
        $c->detach;
    }
    $c->snippet( "promote_mib_$id", "state", "end" );
}

1;
