#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Admin::Snmpwalk;

use Moose;
use namespace::autoclean;
use Opsview::Utils;
use Opsview::Run;

BEGIN { extends qw/Catalyst::Controller/ }

sub auto : Private {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    1;
}

sub scan : Local {
    my ( $self, $c ) = @_;

    $c->forward("get_hostid");
    my $value = $c->snippet( "poll" => $c->stash->{hostid} );
    if ( $value && $value =~ /^(running|start)$/ ) {

        # Do nothing
    }
    else {
        $c->snippet( "poll" => $c->stash->{hostid} => 0 );
        $c->forward("run_snmpwalk");
    }
    $c->res->body(" ");    # To ignore output
}

sub run_snmpwalk : Private {
    my ( $self, $c ) = @_;

    $c->forward("get_host_object");
    my $host = $c->stash->{host};

    my $ip = $host->ip;
    my $ms = $host->monitored_by;

    my $output;
    {
        local $SIG{CHLD} = 'DEFAULT';
        my @cmd;

        if ( $host->snmp_version eq "2c" or $host->snmp_version eq "1" ) {
            my $snmp_community = $host->snmp_community;

            # If 2c, attempt to work out correct SNMP version to query device
            my @try_versions = qw(2c 1);
            if ( $host->snmp_version eq "1" ) {
                @try_versions = (1);
            }
            if ( $host->snmp_port ) {
                $ip .= ':' . $host->snmp_port;
            }
            my $version;
            my ( $rc, $out, $err );
            foreach my $v (@try_versions) {
                $version = $v;    # cannot use $version directly in foreach
                my @cmd = ( "snmpget", "-v", $version, "-c", $snmp_community, $ip, "system.sysDescr.0" );
                if ( $ms->is_slave ) {
                    @cmd = $ms->nodes->first->ssh_command( \@cmd );
                }
                $c->log->info("snmpwalk precmd=@cmd");
                ( $rc, $out, $err ) = Opsview::Run->run_command(@cmd);
                if ($@) {
                    $c->log->fatal( "snmpget: ", $@ );
                    $c->snippet( poll => $c->stash->{hostid} => $c->loc("messages.snmp.unableToRunSnmpget") );
                    $c->res->body(" ");
                    $c->detach;
                }
                last if ($out);
            }

            if ( $rc || $err ) {
                my $text = $c->loc( "messages.snmp.agentNotResponding [_1]", join( ", ", @try_versions ) );
                if ( defined($err) ) {
                    $text .= join( "", " - ", @$err );
                }
                $c->log->warn( "snmpget: ", $text );
                $c->snippet( poll => $c->stash->{hostid} => $text );
                $c->res->body(" ");
                $c->detach;
            }

            # .iso required for full walk
            @cmd = ( "snmpwalk", "-v", $version, "-c", $snmp_community, $ip, ".iso" );
        }
        else {
            my @args = ( "-u", $host->snmpv3_username, "-a", $host->snmpv3_authprotocol, "-A", $host->snmpv3_authpassword, );
            if ( $host->snmpv3_privpassword ) {
                push @args, "-x", $host->snmpv3_privprotocol, "-X", $host->snmpv3_privpassword,;
                unshift @args, "-l", "authPriv";
            }
            else {
                unshift @args, "-l", "authNoPriv";
            }
            my @test = ( "snmpget", "-v", "3", @args, $ip, "system.sysDescr.0" );
            if ( $ms->is_slave ) {
                @test = $ms->nodes->first->ssh_command( \@test );
            }
            $c->log->info("snmpwalk precmd=@cmd");
            my ( $rc, $out, $err ) = Opsview::Run->run_command(@test);
            if ( $rc || $err || $@ ) {
                my $text = $c->loc("messages.snmp.unableToRunSnmpget");
                if ( defined($err) ) {
                    $text = join( "", @$err );
                }
                $c->log->fatal( "snmpget: ", $@ ) if ($@);
                $c->log->fatal( "snmpget: ", $text );
                $c->snippet( poll => $c->stash->{hostid} => $text );
                $c->res->body(" ");
                $c->detach;
            }
            if ( !$out ) {
                $c->snippet( poll => $c->stash->{hostid} => $c->loc("messages.snmp.agentNotRespondingToVersion3") );
                $c->res->body(" ");
                $c->detach;
            }
            @cmd = ( "snmpwalk", "-v", "3", @args, $ip, ".iso" );
        }

        if ( $ms->is_slave ) {
            @cmd = $ms->nodes->first->ssh_command( \@cmd );
        }
        $c->log->info("snmpwalk cmd=@cmd");

        my $progress = 0;
        my $error;
        my $rc = run_command_subcap(
            \@cmd,
            sub {
                $output .= $_[0];
                $progress = $output =~ tr/\n//;
                $c->snippet( poll => $c->stash->{hostid} => $progress );
            },
            sub {
                $error .= $_[0];
            },
        );

        # only detach if exit code not 0 and no output
        if ( $rc && $progress == 0 ) {
            my $text = $c->loc("messages.snmp.errorRunningSnmpwalk");
            if ($error) {
                $text .= " - $error";
            }
            $c->snippet( poll => $c->stash->{hostid} => $text );
            $c->res->body(" ");
            $c->detach;
        }

        # Catch possible errors
        if ( !$output ) {
            $c->snippet( poll => $c->stash->{hostid} => $c->loc("messages.snmp.noOutputSnmpwalk") );
            $c->res->body(" ");
            $c->detach;
        }

        if ($rc) {

            # Add = signs so that output is like snmpwalk
            $output .= " = $error" if ($error);
            $output .= " = " . $c->loc("messages.snmp.connectionTerminated");
        }
    }

    # I think this is to remove funny characters in the system description
    # This probably should be done at the view of the snmpwalk output (filter html)
    # but we'll retain here in the absence of test cases
    $output =~ s/^(.*?Descr.*?\n)//;
    my $firstline = $1;    # Could be null
    $firstline =~ s/\(/- /g;
    $firstline =~ s/\)|'|"//g;
    $output = $firstline . $output;

    my $cache_row = $c->model("OpsviewDB::Snmpwalkcache")->find_or_create( { hostid => $c->stash->{hostid} } );
    $cache_row->text($output);
    $cache_row->last_updated(time);
    $cache_row->update;

    $c->snippet( poll => $c->stash->{hostid} => "end" );
}

sub get_hostid : Private {
    my ( $self, $c ) = @_;
    my $host;
    my $hid = $c->req->param("hostid");
    if ($hid) {
        $c->stash( hostid => $hid );
    }
    else {
        my $hostname = $c->req->param("host");
        unless ($hostname) {
            $c->res->body( $c->loc("messages.noHostname") );
            $c->detach;
        }
        $host = $c->model("OpsviewDB::Host")->search( name => $hostname )->first;
        unless ($host) {
            $c->res->body( $c->loc( "messages.host.notFound [_1]", $hostname ) );
            $c->detach;
        }
        $c->stash( host   => $host );
        $c->stash( hostid => $host->id );
    }

    # Check host is allowed to be seen by user
    $c->forward("/access_denied") unless $c->user->can_configurehost( $c->stash->{hostid} );
}

sub get_host_object : Private {
    my ( $self, $c ) = @_;
    my $host;
    if ( !$c->stash->{host} ) {
        my $hid = $c->stash->{hostid};
        $host = $c->model("OpsviewDB::Host")->retrieve($hid);
        unless ($host) {
            $c->res->body( $c->loc( "messages.host.notFoundById [_1]", $hid ) );
            $c->detach;
        }
        $c->stash( host => $host );
    }
}

sub poll : Local {
    my ( $self, $c ) = @_;
    $c->forward("get_hostid");
    my $hostid = $c->stash->{hostid};

    $c->stash( output => "json" );
    my $value = $c->snippet( "poll" => $hostid );
    if ( !defined $value ) {
        my $cache_row = $c->model("OpsviewDB::Snmpwalkcache")->search( hostid => $hostid )->first;
        if ($cache_row) {
            $c->snippet( "poll" => $hostid => "end" );
            $c->stash( ResultSet => { state => "end", last_update => scalar localtime $cache_row->last_updated } );
        }
        else {
            $c->snippet( "poll" => $hostid => 0 );
            $c->stash( ResultSet => { state => "start" } );
        }
    }
    elsif ( $value eq "end" ) {

        # TODO: This cache_row could be empty if the db has been changed with the sessions not cleared
        # Fix is probably to put sessions in the db as well
        my $cache_row = $c->model("OpsviewDB::Snmpwalkcache")->search( hostid => $hostid )->first;
        $c->stash( ResultSet => { state => "end", last_update => scalar localtime $cache_row->last_updated } );
    }
    elsif ( $value =~ /^\d+$/ ) {
        $c->stash( ResultSet => { state => "running", lines => $value } );
    }
    else {
        delete $c->stash->{output};
        $c->res->body("$value");
    }
}

sub display : Local {
    my ( $self, $c ) = @_;
    $c->forward("get_hostid");
    my $hostid = $c->stash->{hostid};

    my $model = $c->model("OpsviewDB::Snmpwalkcache");
    my ($cache_row) = $model->search( hostid => $hostid );

    my $a = [];
    foreach $_ ( split( "\n", $cache_row->text ) ) {
        my $h = {};
        my ( $oid, $value ) = m/^(.*?) = (.*)$/o;
        next unless $value;    # Ignore lines that don't look like correct results
        $h->{oid}   = $oid;
        $h->{value} = $value;
        push @$a, $h;
    }
    $c->stash( snmpwalk_data => $a );
    $c->stash( last_update   => scalar localtime( $cache_row->last_updated ) );
    $c->stash( wrapper       => "wrappers/blank" );
    $c->stash( template      => "snmpwalk" );
}

1;
