#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Api;

use strict;
use warnings;
use base qw/Catalyst::Controller/;
use Opsview::Role;    # Needed for constants in Opsview::Role
use Opsview::Reloadmessage;
use Opsview::Common;

sub begin : ActionClass('Deserialize') {
}

sub end : ActionClass('Serialize') {
}

__PACKAGE__->config->{map}->{'text/xml'} = "XML::Opsview";

sub do_api : Path {
    my ( $self, $c ) = @_;
    unless ( $c->res->status == 200 ) {
        $c->detach;
    }
    unless ( exists $c->stash->{xml} ) {
        $c->res->status(400);
        $c->stash->{result} = { detail => "XML data required", status => "OPTIONS-ERROR" };
        $c->detach;
    }
    my $xml = $c->stash->{xml};
    if ( $xml->error ) {
        $c->res->status(400);
        $c->stash->{result} = { detail => $xml->error, status => "OPTIONS-ERROR" };
        $c->detach;
    }

    # opsview.rng allows <host> with no action=.. element. This is so we can return status in a similar XML structure
    # However, at the API level, we need to have an action defined, so raise a new error condition here
    my $action = $xml->action;
    unless ($action) {
        $c->res->status(400);
        $c->stash->{result} = { detail => "No action specified", status => "OPTIONS-ERROR" };
        $c->detach;
    }
    if ( exists $xml->{authentication} ) {
        $c->req->params->{login_username} = $xml->{authentication}->{username};
        $c->req->params->{login_password} = $xml->{authentication}->{password};
    }
    unless ( $c->forward("/login_api") ) {
        $c->stash->{result} = { status => "NO-ACCESS", detail => ( $c->stash->{login_error} || "Need to login" ) };
        $c->res->status(401);
        $c->detach;
    }
    unless ( $c->user_exists && $c->user->has_access("ADMINACCESS") ) {
        $c->stash->{result} = { status => "NO-ACCESS", detail => "Insufficient permissions" };
        $c->res->status(401);
        Opsview::Auditlog->create( { username => $c->user->username, text => $c->translate("messages.auditlog.accessDeniedNoAdmin") } );
        $c->detach;
    }

    if ( $action eq "reload" ) {
        $c->stash( synchronous => 1 );
        $c->forward("/admin/reload/do_reload");
        $c->res->body(undef);    # Need to do this because /admin/reload/do_reload sets body to " "

        my $web_status = $c->stash->{web_status};
        if ( $web_status->{status} eq "RETRY" ) {
            $c->stash->{result} = { status => "RETRY", detail => ["Reload already running"] };
            $c->res->status(400);
        }
        elsif ( $web_status->{status} == 0 ) {
            $c->stash->{result} = { status => "OK" };
        }
        else {
            my @errors = map { ( $_->monitoringcluster ? $_->monitoringcluster->name : "" ) . ":" . $_->severity . ":" . $_->message } ( Opsview::Reloadmessage->retrieve_all );
            my $h = { detail => \@errors };
            if ( $web_status->{status} == 4 ) {
                $h->{status} = "WARNING";
            }
            else {
                $h->{status} = "FAILED";
                $c->res->status(400);
            }
            $c->stash->{result} = $h;
        }
    }
    else {
        $action = "do_$action";
        if ( my $list = $xml->list ) {
            $c->log->debug( "About to call $action against " . join( ",", map { ref($_) } @$list ) ) if ( $c->debug );
            eval {
                foreach my $o (@$list)
                {

                    # The two lines below are to pass some context info
                    # to the model. However, this feels wrong
                    # A fresh look at this is probably needed
                    # This seems like dispatch to me - maybe should be some
                    # private actions that this goes through, rather than this code
                    $o->username( $c->user->username );

                    #$o->loc( $c->loc );
                    $o->$action( $c->req->data );
                }
                die( $@, $/ ) if ($@);    # get the error out of the eval
            };
        }
        else {
            my $o = $c->stash->{object};
            $o->username( $c->user->username );

            #$o->loc( $c->loc );
            eval { $_ = $c->stash->{object}->$action( $c->req->data ) };
        }
        if ($@) {
            chomp($@);
            $c->stash->{result} = { detail => $@, status => "FAILED" };
            $c->res->status(400);
        }
        else {
            $c->stash->{result} = { detail => "", status => "OK" };
        }
    }
}

1;
