#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Common;

use strict;
use warnings;
use Opsview::Utils qw(convert_to_arrayref convert_uoms);
use Utils::Nagiosgraph;
use RRDs;

use base qw/Catalyst::Controller/;

# This is so load1, load5, load15 returns in the right order
sub sort_by_length_then_alphabetically {
    length $a <=> length $b || $a cmp $b;
}

# Stub to allow testing of sort routine
sub _test_sort { shift; sort sort_by_length_then_alphabetically @_; }

sub get_styles_list : Private {
    my ( $self, $c, $arg, $type ) = @_;
    $type ||= "Style";
    my $style_prefix = "Opsview::Web::Controller::${arg}::${type}";
    my @style_list = grep {s/^${style_prefix}:://} keys %{ $c->components };
    my %style_labels;
    foreach my $style (@style_list) {
        my $style_label = $c->loc("ui.viewport.style.$style");
        if ( $style_label eq "ui.viewport.style.$style" ) {
            $style_label = $c->components->{"${style_prefix}::$style"}->{description};
        }
        $style_labels{$style} = $style_label;
    }
    my @order_by_description = sort { $style_labels{$a} cmp $style_labels{$b} } @style_list;
    $c->stash( style_list   => \@order_by_description );
    $c->stash( style_labels => \%style_labels );
    my @return_list;
    foreach my $s (@order_by_description) {
        push @return_list, { name => $s, label => $style_labels{$s}, reload_page => $c->components->{"${style_prefix}::$s"}->{reload_page} || 0 },;
    }
    $c->stash( style_list_detailed => \@return_list );
}

# Similar to Runtime::Searches->list_services. Returns a hashref of { metric_hash => {}, errors => [] }
# This needs to be "our" to allow testing
our $rrddir = "/usr/local/nagios/var/rrd";

sub list_rrds : Private {
    my ( $self, $c ) = @_;
    my $params            = $c->req->params;
    my $public_rrdgraphs  = $c->public_role->has_access("RRDGRAPHS");
    my $includethresholds = $params->{includethresholds} || 0;

    # Normalise parameters for hostname and servicename into array refs
    my $hosts        = convert_to_arrayref( $params->{host} );
    my $services     = convert_to_arrayref( $params->{service} );
    my $metrics      = convert_to_arrayref( $params->{metric} );
    my %want_metrics = map { ( $_ => 1 ) } @$metrics;

    my @host_service_metrics;
    my $hsm_array = convert_to_arrayref( $params->{hsm} );
    foreach my $hsm (@$hsm_array) {
        my ( $hostname, $servicename, $metricname ) = split( "::", $hsm );
        push @host_service_metrics, { hostname => $hostname, servicename => $servicename, metricnames => { $metricname => 1 } };
    }
    foreach my $hostname (@$hosts) {
        foreach my $servicename (@$services) {
            push @host_service_metrics, { hostname => $hostname, servicename => $servicename, metricnames => \%want_metrics };
        }
    }
    return $self->metadata_of_rrds( $c, \@host_service_metrics );
}

sub metadata_of_rrds {
    my ( $self, $c, $hsmetric_list ) = @_;
    my $public_rrdgraphs = $c->public_role->has_access("RRDGRAPHS");

    my $includethresholds = $c->req->params->{includethresholds} || 0;

    # List of datapoints
    my $ds = {};
    my %hosts;
    my %services;
    my %host_services;
    my %host_service_metrics;
    my $uoms = {};
    my @auth_errors;

    foreach my $hash (@$hsmetric_list) {
        my $hostname     = $hash->{hostname};
        my $servicename  = $hash->{servicename};
        my %want_metrics = %{ $hash->{metricnames} };

        # If security is required, filter according to object settings
        unless ($public_rrdgraphs) {

            # Check in case user is not defined. Possible if top controller doesn't ensure user exists
            unless ( $c->user && $c->user->can_view_object_by_name( { hostname => $hostname, servicename => $servicename } ) ) {
                push @auth_errors, $c->loc( "messages.accessDeniedToHostService [_1] [_2]", $hostname, $servicename );
                next;
            }
        }

        my $metrics_dir = "$rrddir/" . urlencode($hostname) . "/" . urlencode($servicename);

        if ( -d $metrics_dir ) {
            my @saved_metrics;
            opendir METRICS, $metrics_dir or Catalyst::Exception->throw( $c->translate( "messages.error.cannotReadDirectory [_1]", $metrics_dir ) );

            # Ignore all files with a . at beginning
            # Need to sort here so that the uoms list is deterministic
            my @metrics = sort grep !/^\./, readdir METRICS;
            closedir METRICS;
            foreach my $metricname (@metrics) {
                if (%want_metrics) {
                    next unless $want_metrics{$metricname};
                }
                $hosts{$hostname}                                             = 1;
                $services{$servicename}                                       = 1;
                $host_services{"$hostname||$servicename"}                     = 1;
                $host_service_metrics{"$hostname||$servicename||$metricname"} = 1;

                my $uom      = "";
                my $uom_file = "$metrics_dir/$metricname/uom";
                if ( -e $uom_file && -s $uom_file ) {
                    open UOM, "$uom_file";
                    $uom = <UOM>;
                    chomp $uom;
                    close UOM;
                }

                # Normalise uom value - this will be in the JSON response too
                if ($uom) {
                    ($uom) = convert_uoms($uom);
                }

                my $metricinfo = { name => $metricname, file => "$metrics_dir/$metricname/value.rrd", host => $hostname, service => $servicename, uom => $uom };
                if ( $includethresholds && -e "$metrics_dir/$metricname/thresholds.rrd" ) {
                    $metricinfo->{threshold}->{file} = "$metrics_dir/$metricname/thresholds.rrd";

                    # Open up the rrd and check for warning_end and critical_end
                    my $rrdinfo = RRDs::info("$metrics_dir/$metricname/thresholds.rrd");
                    if (RRDs::error) {
                        Catalyst::Exception->throw( $c->loc( "messages.error.rrdInfoError [_1]", RRDs::error ) );
                    }
                    my $dsnames = {};
                    map { $metricinfo->{threshold}->{$_} = 1 } ( grep s/^ds\[(.*?)\].*/$1/, keys %$rrdinfo );
                }
                push @{ $ds->{$metricname} }, $metricinfo;
                push @{ $uoms->{$uom} },      $metricinfo;
            }
        }
    }
    my @metric_order = sort sort_by_length_then_alphabetically keys %$ds;
    my $summary      = {
        metrics              => scalar @metric_order,
        hosts                => scalar keys %hosts,
        services             => scalar keys %services,
        uoms                 => scalar keys %$uoms,
        host_services        => scalar keys %host_services,
        host_service_metrics => scalar keys %host_service_metrics,
    };
    return {
        errors       => \@auth_errors,
        metrics      => $ds,
        metric_order => \@metric_order,
        summary      => $summary,
        hosts        => \%hosts,
        services     => \%services,
        uoms         => $uoms,
    };
}

sub calendar_setup : Private {
    my ( $self, $c ) = @_;
    push( @{ $c->stash->{extra_javascript} }, "calendar", "calendar-en", "calendar-setup" );
    push( @{ $c->stash->{stylesheets} }, 'skins/aqua/theme.css' );
}

# Very basic meta_redirect. Needed for forms where you want to save the data on them with a back button
sub meta_redirect : Private {
    my ( $self, $c, $url ) = @_;
    $c->res->body(
        qq!<html>
  <head>
    <meta http-equiv="refresh" content="0; URL=${url}" />
  </head>
</html>!
    );
}

sub setup_languages_list : Private {
    my ( $self, $c ) = @_;
    my $languages_list = $c->installed_languages;

    # Take a copy of the hashref as we will be changing it to add the browser option
    $languages_list = {%$languages_list};
    my @languages_values = sort { $languages_list->{$a} cmp $languages_list->{$b} } ( keys %$languages_list );
    unshift( @languages_values, "" );
    $languages_list->{""} = $c->loc("ui.languages.label.useBrowser");
    $c->stash( languages_list   => $languages_list );
    $c->stash( languages_values => \@languages_values );
}

# Need to copy it twice because want to be able to find the $c->loc and $c->translate calls
sub nagios_states : Private {
    my ( $self, $c, $i_default ) = @_;
    my $nagios_states_by_id;
    if ($i_default) {
        $nagios_states_by_id = {
            "0" => $c->translate("ui.status.label.ok"),
            "1" => $c->translate("ui.status.label.warning"),
            "2" => $c->translate("ui.status.label.critical"),
            "3" => $c->translate("ui.status.label.unknown"),
        };
    }
    else {
        $nagios_states_by_id = {
            "0" => $c->loc("ui.status.label.ok"),
            "1" => $c->loc("ui.status.label.warning"),
            "2" => $c->loc("ui.status.label.critical"),
            "3" => $c->loc("ui.status.label.unknown"),
        };
    }
    $c->stash( nagios_states_by_id => $nagios_states_by_id );
}

sub apply_restrictions : Private {
    my ( $self, $c, $rs, $join_args ) = @_;
    my $errors = [];
    my $asuser = $c->user;

    if ( $c->req->params->{asuser} && $c->user->can_view_all ) {
        my $ascontactname = $c->req->params->{asuser};
        my $ascontact = $c->model("OpsviewSchema::Contacts")->find( { name => $ascontactname } );
        if ( !$ascontact ) {
            $c->stash( error => $c->loc( "messages.contact.notFound [_1]", $ascontactname ) );
            $c->detach("/soft_error");
        }
        $asuser = $ascontact;
        $c->stash( asuser => $ascontact );
    }

    if ( !$asuser->can_view_all ) {
        $rs = $rs->search( { "contacts.contactid" => $asuser->id }, { join => $join_args } );
    }

    $rs;
}

1;
