#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Event;

use Moose;
use namespace::autoclean;

use Opsview::Common;
use Opsview::Utils qw(convert_to_arrayref convert_state_to_text convert_host_state_to_text convert_state_type_to_text);
use JSON;

BEGIN { extends 'Catalyst::Controller::ActionRole' }

sub auto : Private {
    my ( $self, $c, $id ) = @_;

    $c->stash( additional_template_paths => [ $c->path_to( 'root', 'event' )->stringify ] );
    $c->stash( page_view_include => "view_options" );

    $c->stash( page_title               => " > " . $c->loc("ui.event.title") );
    $c->stash( page_search_box          => 1 );
    $c->stash( page_search_box_override => "page_search_box" );

    my $loggedin = $c->forward('/authenticate');
    return $loggedin;
}

sub index : Path : Does(ACL) : AllowedRole(VIEWSOME) : AllowedRole(VIEWALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    # if export/json
    if ( $c->req->params->{output} ) {
        my $outtype = $c->req->params->{output};

        # do search
        if (   $outtype eq "json"
            || $outtype eq "xml"
            || $outtype eq "xml"
            || $outtype eq "csv"
            || $outtype eq "excel" )
        {
            $c->forward("/event/json");
            $c->stash( output => $outtype );
        }

        # setup headers
        if ( $outtype eq 'csv' ) {
            $c->res->headers->header( "Content-Type"        => "text/csv" );
            $c->res->headers->header( "Content-Disposition" => 'attachment; filename="events.csv"' );
        }
        elsif ( $outtype eq 'excel' ) {
            $c->res->headers->header( "Content-Disposition" => 'attachment; filename="events.xls"' );
        }
        elsif ( $outtype eq 'xml' ) {
            $c->res->headers->header( "Content-Disposition" => 'attachment; filename="events.xml"' );
        }

        # tranform into xml
        if ( $outtype eq "xml" ) {

            # add data
            $c->stash->{$outtype} = {

                # data itself
                'data' => $c->stash->{'ResultSet'}
            };

            # metadata defining schema to use
            $c->stash->{$outtype}->{template} = {
                'type'     => 'hash',
                'name'     => 'opsview',
                'children' => {
                    'summary' => {
                        'type'     => 'hash',
                        'name'     => 'summary',
                        'children' => {
                            'maxeventid' => {
                                'type' => 'scalar',
                                'name' => 'maxeventid'
                            }
                        }
                    },
                    'list' => {
                        'type' => 'array',
                        'name' => 'list',
                        'item' => {
                            'type'     => 'hash',
                            'name'     => 'event',
                            'attrs'    => { 'eventid' => { 'name' => 'id' } },
                            'children' => {
                                'time' => {
                                    'type' => 'scalar',
                                    'name' => 'time'
                                },
                                'hostname' => {
                                    'type' => 'scalar',
                                    'name' => 'hostname'
                                },
                                'host_group' => {
                                    'type' => 'scalar',
                                    'name' => 'host_group'
                                },
                                'servicename' => {
                                    'type' => 'scalar',
                                    'name' => 'servicename'
                                },
                                'state' => {
                                    'type' => 'scalar',
                                    'name' => 'state'
                                },
                                'state_type' => {
                                    'type' => 'scalar',
                                    'name' => 'state_type'
                                },
                                'output' => {
                                    'type' => 'scalar',
                                    'name' => 'output'
                                }
                            }
                        }
                    }
                }
            };

        }

        # transform for spreadsheets
        elsif ( $outtype eq "csv" || $outtype eq "excel" ) {

            # put data in 2D array
            my $resultset  = $c->stash->{'ResultSet'};
            my $col_lookup = $c->stash->{'columns_requested_lookup'};
            my $data       = [];

            my @row = ();
            push( @row, 'Event ID' )     if $col_lookup->{eventid};
            push( @row, 'Time' )         if $col_lookup->{time};
            push( @row, 'Host Name' )    if $col_lookup->{hostname};
            push( @row, 'Host Group' )   if $col_lookup->{host_group};
            push( @row, 'Service Name' ) if $col_lookup->{servicename};
            push( @row, 'State' )        if $col_lookup->{state};
            push( @row, 'State Type' )   if $col_lookup->{state_type};
            push( @row, 'Output' )       if $col_lookup->{output};
            push( @{$data}, \@row );

            foreach ( @{ $resultset->{list} } ) {
                my $event = $_;
                my @row   = ();
                push( @row, $event->{eventid} )     if exists $event->{eventid};
                push( @row, $event->{time} )        if exists $event->{time};
                push( @row, $event->{hostname} )    if exists $event->{hostname};
                push( @row, $event->{host_group} )  if exists $event->{host_group};
                push( @row, $event->{servicename} ) if exists $event->{servicename};
                push( @row, $event->{state} )       if exists $event->{state};
                push( @row, $event->{state_type} )  if exists $event->{state_type};
                push( @row, $event->{output} )      if exists $event->{output};
                push( @{$data}, \@row );
            }

            # add data
            $c->stash->{$outtype} = { 'data' => $data };
        }

    }
    else {

        # js & css
        push( @{ $c->stash->{stylesheets} }, 'dataTables.css', 'skins/aqua/theme.css', 'ajaxcommon.css', 'events.css' );
        push( @{ $c->stash->{extra_javascript} }, "serialization", "prototypextensions", "jquery", "jquery.dataTables", "ajaxdatatable", "timeline/timeline_js/timeline-api", "calendar", "calendar-en", "calendar-setup", "events", "sound", "showdown" );

        # auto refresh
        $c->stash->{auto_refresh} = $c->req->params->{auto_refresh};
        if ( exists $c->req->params->{show_timeline} ) {
            $c->stash->{show_timeline} = $c->req->params->{show_timeline};
        }
        else {
            $c->stash->{show_timeline} = $c->stash->{system_preference}->show_timeline;
        }

        # Get list of hosts
        my $rs = $c->model("RuntimeSchema::OpsviewHosts")->search(
            {},
            {   order_by => "name",
                columns  => ["name"],
            }
        );

        # Add authentication
        if ( $c->user->has_access("VIEWSOME") ) {
            $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
        }
        my @hostnames = map { $_->name } ( $rs->search );

        # Get list of services
        $rs = $c->model("RuntimeSchema::OpsviewHostServices")->search(
            undef,
            {   order_by => "servicename",
                columns  => ["servicename"],
                group_by => "servicename",
            }
        );

        # Add authentication
        if ( $c->user->has_access("VIEWSOME") ) {
            $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
        }
        my @servicenames = map { $_->servicename } ( $rs->search );

        # get list of all live keywords
        my @keyword_rs = $c->model("OpsviewSchema::Keywords")->search(
            undef,
            {   order_by => \'name',
                columns  => ['name']
            }
        );
        my @keywords = ();
        foreach (@keyword_rs) { push( @keywords, $_->name ); }

        # get list of all host groups
        # This uses the configuration information, just because it is quicker
        $rs = $c->model("OpsviewSchema::Hostgroups")->search(
            undef,
            {   order_by => \'matpath',
                columns  => [ 'id', 'matpath' ],
            }
        );

        # Filter list of host groups if they don't have access to everything
        # Bit different due to linking to Opsview configuration db
        if ( $c->user->has_access("VIEWSOME") && !$c->user->role->all_hostgroups ) {
            $rs = $rs->search( { "role_access_hostgroups.roleid" => $c->user->role->id }, { join => "role_access_hostgroups" } );
        }
        my @hostgroup_rs      = $rs->search;
        my @hostgroup_names   = ();
        my @hostgroup_ddnames = ();
        my @hostgroup_values  = ();
        my $maxlen            = 35;
        foreach (@hostgroup_rs) {
            push( @hostgroup_values, $_->id );
            my $name = "/" . $_->matpath;
            chop($name);    # Remove trailing ,
            $name =~ s/,/\//g;
            my $ddname = $name;
            $ddname =~ s/[^\/]*\//\-/g;
            $ddname =~ s/\-([^\-])/> $1/;    # Don't use /g because this affects hostgroups with a "-" in the name
            push( @hostgroup_ddnames, $ddname );
            my $l = length($name);
            if ( $l > $maxlen ) { $name = '...' . substr( $name, $l - $maxlen, $maxlen ); }
            push( @hostgroup_names, $name );
        }

        # info for all multiselects in filter options
        my $filter_columns = [
            {   "id"     => "host",
                "name"   => $c->loc("ui.event.label.host"),
                "values" => \@hostnames
            },
            {   "id"             => "host_group",
                "name"           => $c->loc("ui.event.label.hostgroup"),
                "names"          => \@hostgroup_names,
                "dropdown_names" => \@hostgroup_ddnames,
                "values"         => \@hostgroup_values
            },
            {   "id"     => "service",
                "name"   => $c->loc("ui.event.label.service"),
                "values" => \@servicenames
            },
            {   "id"     => "host_state",
                "name"   => $c->loc("ui.event.label.hostState"),
                "names"  => [ $c->loc("ui.status.label.up"), $c->loc("ui.status.label.down"), $c->loc("ui.status.label.unreachable") ],
                "values" => [ "UP", "DOWN", "UNREACHABLE" ]
            },
            {   "id"     => "service_state",
                "name"   => $c->loc("ui.event.label.serviceState"),
                "names"  => [ $c->loc("ui.status.label.ok"), $c->loc("ui.status.label.warning"), $c->loc("ui.status.label.critical"), $c->loc("ui.status.label.unknown") ],
                "values" => [ "OK", "WARNING", "CRITICAL", "UNKNOWN" ]
            },
            {   "id"     => "state_type",
                "name"   => $c->loc("ui.event.label.stateType"),
                "names"  => [ "SOFT", "HARD" ],
                "values" => [ "SOFT", "HARD" ]
            },
            {   "id"     => "keyword",
                "name"   => $c->loc("ui.event.label.keyword"),
                "values" => \@keywords
            }
        ];
        $c->stash->{filter_columns} = $filter_columns;
    }
}

# Lookups - should probably move somewhere else (Opsview::Utils?) later
my $lookup_state_type = {
    "soft" => 0,
    "hard" => 1,
};
my $lookup_host_state = {
    "up"          => 0,
    "down"        => 1,
    "unreachable" => 2,
};
my $lookup_service_state = {
    "ok"       => 0,
    "warning"  => 1,
    "critical" => 2,
    "unknown"  => 3,
};
my $lookup_columns = {
    "hostname"        => "object_hosts.hostname",
    "time"            => "state_time",
    "objectid"        => "me.object_id",
    "servicename"     => "object_hosts.name2",
    "state"           => "state",
    "state_type"      => "state_type",
    "output"          => "output",
    "eventid"         => "statehistory_id",
    "host_group"      => "hostgroup.matpath",
    "host_group_id"   => "hostgroup.id",
    "markdown_filter" => "object_hosts.markdown_filter",
};

sub json : Private {
    my ( $self, $c ) = @_;

    # Convert to perl structure
    # Use URL parameters for now
    my $params             = $c->req->params;
    my $rows               = $params->{rows};
    my $page               = $params->{page} || 1;
    my $sortby             = $params->{sortby} ? from_json( $params->{sortby} ) : [ { col => "time", order => "desc" } ];
    my $host_states        = convert_to_arrayref( $params->{host_state} );
    my $service_states     = convert_to_arrayref( $params->{service_state} );
    my $hostnames          = convert_to_arrayref( $params->{host} );
    my $servicenames       = convert_to_arrayref( $params->{service} );
    my $state_types        = convert_to_arrayref( $params->{state_type} );
    my $hostgroups         = convert_to_arrayref( $params->{host_group} );
    my $keywords           = convert_to_arrayref( $params->{keyword} );
    my $search_strings_ref = convert_to_arrayref( $params->{search} );
    my $columns_requested  = convert_to_arrayref( $params->{cols} );
    my $saved_maxeventid   = $params->{saved_maxeventid} || 0;
    my $start_dt;
    my $end_dt;

    # Calculate ordering. Assumes $sortby is of form:
    # [ { col => "name", order => "desc" }, { col => "name2", order => "asc" } ]
    my $order_by = [];
    foreach my $sorter (@$sortby) {
        my $col = $lookup_columns->{ lc $sorter->{col} };
        Catalyst::Exception->throw("Invalid $col") unless $col;
        $col .= " DESC" if ( defined $sorter->{order} && $sorter->{order} eq "desc" );
        push @$order_by, \$col;
    }

    my $columns_requested_lookup = {};
    map { $columns_requested_lookup->{$_}++ } @$columns_requested;

    # Add host_group_id if host_group requested
    $columns_requested_lookup->{host_group_id}++ if $columns_requested_lookup->{host_group};

    # Always return time and eventid - Event viewer errors if time not set
    $columns_requested_lookup->{time}++;
    $columns_requested_lookup->{eventid}++;

    # Need servicename to distinguish between host and service
    $columns_requested_lookup->{servicename}++;

    # TODO: OPS-561: Find out why these aren't getting through
    $columns_requested_lookup->{output}++;
    $columns_requested_lookup->{markdown_filter}++;
    $columns_requested_lookup->{state_type}++;
    $columns_requested_lookup->{objectid}++;
    $c->stash( columns_requested_lookup => $columns_requested_lookup );

    my $columns_to_get = [];
    my $columns_as     = [];
    foreach my $col ( keys %$columns_requested_lookup ) {
        my $c = $lookup_columns->{ lc $col };
        if ( defined $c ) {
            push @$columns_to_get, $c;
            $col = "state_time" if ( $col eq "time" );    # DBIC requires that the accessor already exists
            push @$columns_as, $col;
        }
    }

    # Setup initial resultset
    my $opts = {
        order_by => $order_by,
        +select  => $columns_to_get,
        +as      => $columns_as,
    };

    # This sets up an expensive join to hostgroups, but only if we want to search it or return the hostgroup name
    if ( $columns_requested_lookup->{host_group} || @$hostgroups ) {
        $opts->{join} = { "object_hosts" => { "host" => "hostgroup" } };
    }
    else {
        $opts->{join} = "object_hosts";
    }

    my $rs = $c->model("RuntimeSchema::NagiosStatehistory")->search( {}, $opts, );

    # Filter by time
    if ( $params->{startTime} ) {
        $start_dt = Opsview::Common->parse_datetime_string( $params->{startTime} );
        $rs = $rs->search( { "state_time" => { ">=" => $start_dt } } );
    }
    if ( $params->{endTime} ) {
        $end_dt = Opsview::Common->parse_datetime_string( $params->{endTime} );
        $rs = $rs->search( { "state_time" => { "<" => $end_dt } } );
    }

    # Add authentication
    if ( $c->user->has_access("VIEWSOME") ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }

    # Add keyword filtering if set
    $rs = $rs->search( { "keywords.keyword" => $keywords }, { join => "keywords" } ) if @$keywords;

    # Filter on hostgroup
    if (@$hostgroups) {
        my @hostgroups_like = map { $c->model("RuntimeSchema::OpsviewHostgroups")->find($_)->matpath . "%" } @$hostgroups;
        $rs = $rs->search( { "hostgroup.matpath" => { "-like" => \@hostgroups_like } } );
    }

    # Add state_types
    my $state_types_numeric = [];
    foreach my $key (@$state_types) {
        push @$state_types_numeric, $lookup_state_type->{ lc $key };
    }
    $rs = $rs->search( { "state_type" => $state_types_numeric } ) if @$state_types_numeric;

    # Add host_states or service_states. Must work out the object type otherwise will get service states too
    # Need to do an OR condition, to allow hosts and services to be seen
    my $host_states_numeric = [];
    foreach my $key (@$host_states) {
        push @$host_states_numeric, $lookup_host_state->{ lc $key };
    }
    my $service_states_numeric = [];
    foreach my $key (@$service_states) {
        push @$service_states_numeric, $lookup_service_state->{ lc $key };
    }
    my $condition = [];
    push @$condition, "-and" => [ "state" => $host_states_numeric, "object_hosts.name2" => undef ] if @$host_states_numeric;
    push @$condition, "-and" => [ "state" => $service_states_numeric, "object_hosts.name2" => { "!=" => undef } ] if @$service_states_numeric;
    $rs = $rs->search( { -or => $condition } ) if @$condition;

    # Add hostnames and servicenames
    my $servicenames_search = [@$servicenames];
    if (@$hostnames) {
        $rs = $rs->search( { "object_hosts.hostname" => $hostnames } );

        # Include the host event data, only if there are some hosts requested
        push @$servicenames_search, undef;
    }
    if (@$servicenames) {
        $rs = $rs->search( { "object_hosts.name2" => $servicenames_search } );
    }

    # Search parameters for hostname, service name, output
    if (@$search_strings_ref) {
        my @where_or_terms;
        foreach my $search_term (@$search_strings_ref) {
            push @where_or_terms,
                {
                -or => [
                    "object_hosts.hostname" => { "like" => "%" . $search_term . "%" },
                    "object_hosts.name2"    => { "like" => "%" . $search_term . "%" },
                    "output"                => { "like" => "%" . $search_term . "%" },
                ]
                };
        }
        $rs = $rs->search( { -and => \@where_or_terms, } );
    }

    # Remove unnecessary ordering to allow db to avoid sorting
    my $unordered_rs = $rs->search( {}, { order_by => undef } );
    my $filtered_maxeventid = $unordered_rs->get_column("statehistory_id")->max;

    my $filtered_new_event_count = 0;
    if ( $saved_maxeventid > 0 && $saved_maxeventid < $filtered_maxeventid ) {
        $filtered_new_event_count = $unordered_rs->count( { statehistory_id => { '>' => $saved_maxeventid } } );
    }
    if ($rows) {
        $rs = $rs->search( {}, { rows => $rows, page => $page } );
    }

    # Convert rows to perl structure
    my $dt_formatter = $c->stash->{dt_formatter};
    my @list         = ();
    while ( my $obj = $rs->next ) {

        my $data = {};
        $data->{time}        = $dt_formatter->( $obj->state_time ) if $columns_requested_lookup->{time};
        $data->{eventid}     = $obj->get_column("eventid")         if $columns_requested_lookup->{eventid};
        $data->{objectid}    = $obj->get_column("objectid")        if $columns_requested_lookup->{objectid};
        $data->{hostname}    = $obj->get_column("hostname")        if $columns_requested_lookup->{hostname};
        $data->{servicename} = $obj->get_column("servicename")     if $columns_requested_lookup->{servicename};

        if ( $columns_requested_lookup->{output} ) {
            $data->{output}   = $obj->get_column("output");
            $data->{markdown} = $obj->get_column("markdown_filter");
        }

        if ( $columns_requested_lookup->{host_group} ) {
            if ( $c->req->param("output") eq "json" ) {
                $data->{host_group} = $obj->get_column("host_group_id");
            }
            else {

                # Return textual string. We convert "," to "/" and prefix with "/"
                my $hg = $obj->get_column("host_group");
                chop $hg;
                $hg =~ s/,/\//g;
                $data->{host_group} = "/" . $hg;
            }
        }

        if ( $columns_requested_lookup->{state_type} ) {
            $data->{state_type} = uc( convert_state_type_to_text( $obj->get_column("state_type") ) );
        }

        if ( $columns_requested_lookup->{state} ) {
            if ( defined $obj->get_column("servicename") ) {
                $data->{state} = uc( convert_state_to_text( $obj->get_column("state") ) );
            }
            else {
                $data->{state} = uc( convert_host_state_to_text( $obj->get_column("state") ) );
            }
        }

        push @list, $data;
    }

    # Return JSON
    my $resultset = { list => \@list, };

    # If rows is set, and hence paging, return data
    if ($rows) {
        $resultset->{summary} = {
            total_entries => $rs->pager->total_entries,
            pages         => $rs->pager->last_page,
            page          => $rs->pager->current_page,
        };
    }

    # This is an overall max value, which is used as a timestamp on the last events processed
    # Cannot use same resultset as will try to use max value based on previous search parameters
    $resultset->{summary}->{maxeventid}          = $c->model("RuntimeSchema::NagiosStatehistory")->get_column("statehistory_id")->max;
    $resultset->{summary}->{filtered_maxeventid} = $filtered_maxeventid;
    $resultset->{summary}->{entries}             = scalar @list;

    if ( $filtered_new_event_count > 0 ) {
        $resultset->{summary}->{filtered_new_event_count} = $filtered_new_event_count;
    }

    $c->stash( ResultSet => $resultset );
    $c->stash( output    => "json" );
}

1;
