#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Graph;

use strict;
use warnings;

use Clone qw(clone);
use base qw/Catalyst::Controller/;

use Opsview::Utils;
use Time::Zone;

sub auto : Private {
    my ( $self, $c ) = @_;
    $c->stash(
        additional_template_paths => [
            $c->path_to( 'root', 'graph', 'style' )->stringify,    # Must come before graph/ to pick up most specific first
            $c->path_to( 'root', 'graph' )->stringify,
        ]
    );
    $c->stash->{page_title} .= " > " . $c->loc("ui.graph.title");
    return $c->forward( "/check_public_or_user_access", ["RRDGRAPHS"] );
}

sub layout : Path : Args(0) {
    my ( $self, $c ) = @_;

    # Need to remove this because you don't want subsequent URLs to take this output value
    my $out = delete $c->req->params->{output};

    # Change start and end to epoch values
    my $query_params = clone( $c->req->params );
    $c->stash( query_params => $query_params );
    my ( $start_dt, $end_dt );
    if ( $query_params->{start} ) {
        $start_dt = Opsview::Common->parse_datetime_string( $query_params->{start} );
    }
    if ( $query_params->{end} ) {
        $end_dt = Opsview::Common->parse_datetime_string( $query_params->{end} );
    }
    if ( defined $start_dt ) {
        $c->stash->{params_override}->{start} = $start_dt->epoch;
    }
    else {
        delete $query_params->{start};
    }
    if ( defined $end_dt ) {
        $c->stash->{params_override}->{end} = $end_dt->epoch;
    }
    else {
        delete $query_params->{end};
    }

    # Include Javascript libs & CSS
    push( @{ $c->stash->{extra_javascript} }, "prototypextensions", "serialization", "destruction", "resizing", "closing", "excanvas", "jquery", "jquery.flot", "jquery.noConflict", "ajaxglobals", "ajaxgraph", "ajaxtimegraph", "opsviewgraphs" );
    push( @{ $c->stash->{stylesheets} }, 'ajaxgraph.css', 'opsviewgraphs.css' );

    # if ie8 put in ie7 compataibility mode, otherwise main graph and overview are displayed incorrectly
    if ( $c->stash->{useragent} =~ m/MSIE 8\.0/ ) {
        $c->res->headers->header( "X-UA-Compatible" => "IE=7" );
    }

    # Pass graph state/settings string on
    $c->stash( 'graphState'      => $c->req->params->{state} );
    $c->stash( 'timezone_offset' => tz_local_offset() );

    # Get style
    my $style;
    unless ( $style = $c->req->param("style") ) {
        $style = "summary";
    }
    $c->forward( "/common/get_styles_list", ["Graph"] );
    unless ( exists $c->stash->{style_labels}->{$style} ) {
        $c->stash( error => $c->loc( "messages.graph.styleNotExist [_1]", $style ) );
        $c->detach("/soft_error");
    }

    my $results = $c->forward("/common/list_rrds");

    #	if (@{$results->{errors}}) {
    #		$c->stash->{error} = join("; ", @{$results->{errors}});
    #		$c->detach("/soft_error");
    #	}
    #	if (! $results->{summary}->{metrics} ) {
    #		$c->stash->{error} = $c->loc("messages.graph.noDatasets");
    #		$c->detach("/soft_error");
    #	}
    if ( not defined $query_params->{'legend'} ) {
        $query_params->{'legend'} = 1;
    }
    $c->stash( style        => $style );
    $c->stash( template     => "graph_wrapper" );
    $c->stash( graph_header => 1 );
    $c->stash( results      => $results );
    $c->stash( now          => time );

    # Get default graph settings
    $c->stash(
        settings => {
            show_legend      => Opsview::Config->graph_show_legend,
            auto_max_metrics => Opsview::Config->graph_auto_max_metrics,
        }
    );

    $c->forward("/graph/options_setup");

    # RRDs do not need to be refreshed at a shorter rate
    # Only refresh if end is now
    my $refresh = 0;
    if ( !$query_params->{end} || $query_params->{end} eq "now" ) {
        $refresh = 1;
        $c->stash( refresh_rate => 600 );
    }

    # Allow changing of templates or processing of data
    $c->forward("/graph/style/$style/override");

    if ( defined $out ) {
        if ( $out eq "ajax" ) {
            $c->stash( wrapper => "wrappers/blank" );
        }
        elsif ( $out eq "json" ) {
            $c->stash( output => "json" );
        }
    }
    else {

        # Set breadcrumbs
        if ( $c->stash->{results}->{summary}->{hosts} == 1 ) {
            my ($hostname) = keys %{ $c->stash->{results}->{hosts} };
            my $host = $c->model("RuntimeSchema::Hosts")->search( { name => $hostname } )->first;
            my $breadcrumbs = $c->forward( "/status/hostgroup/breadcrumbs", [ $host->hostgroup ] );
            push @$breadcrumbs, $c->forward( "/status/host/breadcrumb", [$host] );
            $c->stash( breadcrumbs => $breadcrumbs );
        }
        $c->forward("/refresh_page") if $refresh;
    }
}

sub options_setup : Private {
    my ( $self, $c ) = @_;
    $c->forward("/common/calendar_setup");
}

sub menu : Local {
    my ( $self, $c ) = @_;

    # Get style
    my $style;
    unless ( $style = $c->req->param("style") ) {
        $style = "summary";
    }
    $c->forward( "/common/get_styles_list", ["Graph"] );
    unless ( exists $c->stash->{style_labels}->{$style} ) {
        $c->stash( error => $c->loc( "messages.graph.styleNotExist [_1]", $style ) );
        $c->detach("/soft_error");
    }
    $c->stash( template => $style . "_menu" );
    $c->stash( wrapper  => "wrappers/menu" );
}

1;
