#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Graphrrd;

use strict;
use warnings;

use base qw/Catalyst::Controller/;

use Opsview::Utils;

sub auto : Private {
    my ( $self, $c ) = @_;
    $c->stash(
        additional_template_paths => [
            $c->path_to( 'root', 'graphrrd', 'style' )->stringify,    # Must come before graph/ to pick up most specific first
            $c->path_to( 'root', 'graphrrd' )->stringify,
        ]
    );
    $c->stash->{page_title} .= " > " . $c->loc("ui.graph.title");
    return $c->forward( "/check_public_or_user_access", ["RRDGRAPHS"] );
}

sub layout : Path : Args(0) {
    my ( $self, $c ) = @_;

    # Need to remove this because you don't want subsequent URLs to take this output value
    my $out = delete $c->req->params->{output};

    # Change start and end to epoch values
    my $query_params = $c->req->params;
    my ( $start_dt, $end_dt );
    if ( $query_params->{start} ) {
        $start_dt = Opsview::Common->parse_datetime_string( $query_params->{start} );
    }
    if ( $query_params->{end} ) {
        $end_dt = Opsview::Common->parse_datetime_string( $query_params->{end} );
    }
    if ( defined $start_dt ) {
        $c->stash->{params_override}->{start} = $start_dt->epoch;
    }
    else {
        delete $query_params->{start};
    }
    if ( defined $end_dt ) {
        $c->stash->{params_override}->{end} = $end_dt->epoch;
    }
    else {
        delete $query_params->{end};
    }

    # Include Javascript libs & CSS
    push( @{ $c->stash->{extra_javascript} }, "prototypextensions", "serialization", "destruction", "resizing", "closing", "excanvas", "jquery", "jquery.flot", "jquery.noConflict", "ajaxglobals", "ajaxgraph", "ajaxtimegraph", "opsviewgraphs" );
    push( @{ $c->stash->{stylesheets} }, 'ajaxgraph.css', 'opsviewgraphs.css' );

    # if ie8 put in ie7 compataibility mode
    if ( $c->stash->{useragent} =~ m/MSIE 8\.0/ ) {
        $c->res->headers->header( "X-UA-Compatible" => "IE=7" );
    }

    # Pass graph state/settings string on
    $c->stash( 'graphState' => $c->req->params->{state} );

    # Get style
    my $style;
    unless ( $style = $c->req->param("style") ) {
        $style = "summary";
    }
    $c->forward( "/common/get_styles_list", ["Graph"] );
    unless ( exists $c->stash->{style_labels}->{$style} ) {
        $c->stash( error => $c->loc( "messages.graph.styleNotExist [_1]", $style ) );
        $c->detach("/soft_error");
    }

    my $results = $c->forward("/common/list_rrds");

    #	if (@{$results->{errors}}) {
    #		$c->stash->{error} = join("; ", @{$results->{errors}});
    #		$c->detach("/soft_error");
    #	}
    #	if (! $results->{summary}->{metrics} ) {
    #		$c->stash->{error} = $c->loc("messages.graph.noDatasets");
    #		$c->detach("/soft_error");
    #	}
    if ( not defined $c->req->param('legend') ) {
        $c->req->params->{'legend'} = 1;
    }
    $c->stash( style        => $style );
    $c->stash( template     => "graph_wrapper" );
    $c->stash( graph_header => 1 );
    $c->stash( results      => $results );
    $c->stash( now          => time );

    $c->forward("/graphrrd/options_setup");
    $c->stash( tabs => [qw(Options)] );

    # RRDs do not need to be refreshed at a shorter rate
    # Only refresh if end is now
    my $refresh = 0;
    if ( !$c->req->params->{end} || $c->req->params->{end} eq "now" ) {
        $refresh = 1;
        $c->stash( refresh_rate => 120 );
    }

    # Allow changing of templates or processing of data
    $c->forward("/graphrrd/style/$style/override");

    if ( defined $out ) {
        if ( $out eq "ajax" ) {
            $c->stash( wrapper => "wrappers/blank" );
        }
        elsif ( $out eq "json" ) {
            $c->stash( output => "json" );
        }
    }
    else {
        $c->forward("/refresh_page") if $refresh;
    }
}

sub options_setup : Private {
    my ( $self, $c ) = @_;
    $c->forward("/common/calendar_setup");
}

sub options : Local {
    my ( $self, $c ) = @_;
    push( @{ $c->stash->{stylesheets} }, 'admin.css' );
    my $tabs = [qw(Options Objects)];
    $c->stash( tabs => $tabs );
    $c->forward("/graphrrd/options_setup");
    $c->stash( disable_cache => 1 );
}

# This serves two functions:
#  Merge query parameters with body parameters, with body overwriting query params - this is so the form takes precedence
#  Remove some invalid data due to the form structure
#  Then redirect to allow the URL to be bookmarkable
sub validate : Local {
    my ( $self, $c ) = @_;
    my $query_params = { %{ $c->req->query_parameters } };
    my $body_params  = $c->req->body_parameters;
    foreach my $k ( keys %$body_params ) {
        $query_params->{$k} = $body_params->{$k};
        if ( $k eq "host" || $k eq "service" || $k eq "metric" ) {
            my %objs = map { ( $_ => 1 ) } ( grep !/^$/, @{ convert_to_arrayref( $query_params->{$k} ) } );
            my $newlist = [ keys %objs ];
            $query_params->{$k} = $newlist;
        }
    }
    map { $query_params->{$_} = 0 unless exists $body_params->{$_} } (qw(includethresholds stack legend showlimits fixedscale));

    # Remove submit and blank hostnames, service names
    delete $query_params->{".submit"};

    $c->res->redirect( $c->uri_for( "/graphrrd", $query_params ) );
}

sub menu : Local {
    my ( $self, $c ) = @_;

    # Get style
    my $style;
    unless ( $style = $c->req->param("style") ) {
        $style = "summary";
    }
    $c->forward( "/common/get_styles_list", ["Graphrrd"] );
    unless ( exists $c->stash->{style_labels}->{$style} ) {
        $c->stash( error => $c->loc( "messages.graph.styleNotExist [_1]", $style ) );
        $c->detach("/soft_error");
    }
    $c->stash( template => $style . "_menu" );
    $c->stash( wrapper  => "wrappers/menu" );
}

1;
