#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

# Although this is called "public", it is really "authenticated"
# Use Pub.pm for actually publicly available resources through Catalyst
package Opsview::Web::Controller::Public;

use strict;
use warnings;
use Algorithm::Closest::NetworkAddress;
use Opsview::Utils::Time;

use base qw/Catalyst::Controller/;

sub auto : Private {
    my ( $self, $c ) = @_;

    return 0 unless $c->forward("/authenticate");

    $c->stash( additional_template_paths => [ $c->path_to( 'root', $c->action->namespace )->stringify, $c->path_to( 'root', 'admin' )->stringify ] );
    return 1;
}

sub plugin_help : Local {
    my ( $self, $c ) = @_;
    my $name = $c->req->param("name");
    unless ($name) {
        $c->res->body("No plugin name specified");
        return;
    }
    my $p = Opsview::Plugin->search( name => $name )->first;

    if ($p) {
        $c->stash( plugin => $p );
        if ( defined $c->req->param('popup') ) {
            $c->stash( title => "Plugin help for " . $c->req->param('name') );
        }
        $c->stash( wrapper => "wrappers/blank" );
    }
    else {
        $c->res->body("Cannot find plugin: $name");
        return;
    }
}

sub closest_monitoringserver : Local : Args(1) {
    my ( $self, $c, $network_address ) = @_;
    my $nodes = Opsview::Monitoringserver->retrieve_all_nodes_hash;

    my $l = Algorithm::Closest::NetworkAddress->new( network_address_list => [ keys %$nodes ] );

    my $closest = $l->compare($network_address);
    my $node;
    if ( exists $nodes->{$closest} ) {
        $c->res->body( '"' . $nodes->{$closest}->id . '"' );
    }
    else {
        $c->res->body('""');
    }

    $c->res->content_type("text/javascript; charset=utf-8");
    $c->detach;
}

sub normalise_minutes : Local {
    my ( $self, $c ) = @_;
    my $minutes = $c->req->param("atom_max_age");
    my $a = Time::Interval::parseInterval( minutes => $minutes, String => 1 );
    $c->res->body($a);
}

use DateTime;
use DateTime::Duration;
use DateTime::Format::Strptime;

sub normalise_duration : Local {
    my ( $self, $c ) = @_;
    my $string = $c->req->param("q");
    my $result = {};
    my $seconds;
    eval { $seconds = Opsview::Utils::Time->jira_duration_to_seconds($string) };
    if ($@) {
        $result->{error} = $@;
    }
    else {
        $result->{seconds}             = $seconds;
        $result->{normalised_duration} = Opsview::Utils::Time->seconds_to_jira_duration($seconds);
    }
    $c->stash( output    => "json" );
    $c->stash( ResultSet => $result );
}

sub check_downtime_start_end_times : Local {
    my ( $self, $c ) = @_;
    $c->forward("/rest/downtime/downtime_validate");
    $c->stash( output => "json" );
    if ( $c->stash->{downtime_data} ) {
        $c->stash(
            ResultSet => {
                starttime => $c->stash->{downtime_data}->{start_time}->strftime("%F %T"),
                endtime   => $c->stash->{downtime_data}->{end_time}->strftime("%F %T"),
            }
        );
    }
    else {
        $c->res->status(400);
    }
}

sub validate_time : Local {
    my ( $self, $c ) = @_;
    $c->stash( output => "json" );
    if ( my $time = $c->req->param("starttime") or $c->req->param("endtime") ) {
        my $strp = new DateTime::Format::Strptime(
            pattern   => "%F %T",
            time_zone => "local",
        );
        my $dt = $strp->parse_datetime($time);
        unless ($dt) {
            $c->stash( ResultSet => { result => 0, detail => "Invalid time" } );
        }
        else {

            # Check > 1 hour ago (because time defaults to now)
            my $now = DateTime->now->subtract( hours => 1 );
            if ( $dt->subtract_datetime($now)->is_negative ) {
                $c->stash( ResultSet => { result => 0, detail => "Time is past" } );
            }
            else {
                $c->stash( ResultSet => { result => 1 } );
            }
        }
    }
    else {
        $c->stash( ResultSet => { result => 0, detail => "No parameter found" } );
    }
}

1;
