#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::REST;

use Moose;
use namespace::autoclean;
use Try::Tiny;

BEGIN { extends 'Catalyst::Controller::ActionRole', 'Opsview::Web::ControllerBase::REST' }

# The api_version number is linked to the Opsview version number
# Increment this number when backwards compatibility is broken
my $api_min_version = "2.0";

# Note: No user is here. Chain to authenticated_root if you require a user
sub root : Chained("/") : PathPart("rest") : CaptureArgs(0) {
    my ( $self, $c ) = @_;

    if ( $c->debug ) {
        $c->log->debug( "Headers: " . Data::Dump::dump( $c->req->headers ) );
    }

    # If a deserialize request fails, it doesn't detach from begin. We force it here
    if ( $c->res->status != 200 ) {
        $c->detach;
    }
}

# This means that authentication is required and will be setup for ACLs
sub authenticated_root : Chained("root") : PathPart("") : CaptureArgs(0) {
    my ( $self, $c ) = @_;
    $c->detach unless $c->forward("/rest/authenticate");
}

# Can't use config as sub name due to Catalyst
sub config_root : Chained("authenticated_root") : PathPart("config") : CaptureArgs(0) {
}

sub status_root : Chained("authenticated_root") : PathPart("status") : CaptureArgs(0) {
}

sub api_version : Chained("root") : PathPart("") : ActionClass('REST') {
}

use version;

sub api_version_GET {
    my ( $self, $c, @args ) = @_;
    if (@args) {

        # This catches all requests going to a wrong URL
        $c->res->status(404);
        $c->stash( "rest" => { message => "Request not found for " . $c->uri_for_path(@args), } );
    }
    else {
        my $api_version = "";
        if ( $_ = $c->config->{version} ) {
            $api_version = version->declare( "v" . $_ )->numify;
        }
        my $easyxdm_version = $c->forward("/easyxdm_version");
        $c->stash(
            "rest" => {
                api_version     => $api_version,
                api_min_version => $api_min_version,
                easyxdm_version => $easyxdm_version . "",
            }
        );
    }
}

sub info : Chained("authenticated_root") : PathPart("info") : Args(0) : ActionClass('REST') {
}

sub info_GET {
    my ( $self, $c ) = @_;

    $c->stash(
        "rest" => {
            opsview_version => $c->config->{version},
            opsview_build   => $c->config->{build},
            opsview_edition => $c->config->{edition},
            uuid            => $c->model("OpsviewSchema::Systempreferences")->find(1)->uuid,
        }
    );
}

# From C::P::Session
my $counter = 0;

sub session_hash_seed {
    return join( "", ++$counter, time, rand, $$, {}, );
}

sub generate_token {
    my ($self) = @_;
    my $digest = Digest->new("SHA-1");
    $digest->add( $self->session_hash_seed );
    return $digest->hexdigest;
}

my $session_expires_after = 3600;

sub generate_session {
    my ( $self, $c, $other_params ) = @_;
    my $count = 0;
    my $new_session;

    while ( !$new_session ) {
        $count++;
        my $token = $self->generate_token;
        try {
            $new_session = $c->model("OpsviewSchema::ApiSessions")->create(
                {   token => $token,
                    %$other_params,
                }
            );
        }
        catch {
            $c->log->warn("Failed to generate session on attempt $count: $_");
        };
        if ( $count >= 5 ) {
            $c->res->status(503);
            $c->stash( "rest" => { message => "Error creating session token after 5 attempts" } );
            $c->detach;
        }
    }
    return $new_session;
}

sub set_token : Private {
    my ( $self, $c ) = @_;

    my $now         = time();
    my $expiry_time = $now + $session_expires_after;

    # Add into db with username, access time, expiry time and IP address
    my $session = $self->generate_session(
        $c,
        {   expires_at     => $expiry_time,
            accessed_at    => $now,
            username       => $c->user->username,
            ip             => $c->req->address,
            one_time_token => 0,
        }
    );

    my $token_response = { token => $session->token };

    if ( $c->req->params->{include_one_time_token} ) {

        # One time token is valid for 30 days, because not sure when it will be used
        my $token_expiry_seconds = 30 * 24 * 60 * 60;
        my $one_time_token       = $self->generate_session(
            $c,
            {   expires_at     => $now + $token_expiry_seconds,
                accessed_at    => $now,
                username       => $c->user->username,
                ip             => $c->req->address,
                one_time_token => 1,
            }
        );
        $token_response->{one_time_token} = $one_time_token->token;
    }

    $self->status_ok( $c, entity => $token_response );
}

=head2 /rest/authenticate

Private function to check authentication for a user. Uses the X-Opsview-Username and X-Opsview-Token header variables.
Returns 0 if user cannot authenticate, otherwise returns 1 and sets the user

=cut

sub authenticate : Private {
    my ( $self, $c ) = @_;

    # Get token and username from headers
    # Also use the parameters, though this should be a last choice option
    my $username 
        = $c->req->header("X-Opsview-Username")
        || $c->req->params->{"x-opsview-username"}
        || "";
    my $token 
        = $c->req->header("X-Opsview-Token")
        || $c->req->params->{"x-opsview-token"}
        || "";
    my $error;
    my $now = time();

    # Confirm token exists and is not out of date. Otherwise 401
    my $session = $c->model("OpsviewSchema::ApiSessions")->find( { token => $token } );
    if ( !$session ) {
        $error = "Token invalid";
    }
    elsif ( lc( $session->username ) ne lc($username) ) {
        $error = "Token does not match username";
    }
    elsif ( $session->expires_at <= $now ) {
        $error = "Token has expired";
    }
    elsif ( $session->one_time_token ) {
        $error = "One time token cannot be used for REST API";
    }

    # Confirm user exists. Do this after token checks because a hack attempt would need to guess a valid token before
    # getting this far
    my $uobj;
    if ( !$error ) {
        $uobj = $c->find_user( { name => $username }, "local" );
        if ( !defined $uobj ) {
            $error = "User not found";
        }
    }

    if ($error) {
        $c->log->info("REST authenticate error: $error. Username: '$username', Token: '$token'");
        $c->res->status(401);
        $c->stash( "rest" => { message => $error } );
        return 0;
    }

    $c->set_authenticated($uobj);

    # Update access time
    $session->expires_at( $now + $session_expires_after );
    $session->accessed_at($now);
    $session->ip( $c->req->address );
    $session->update;

    $c->stash( session_object => $session );

    return 1;
}

sub login : Chained("root") : PathPart("login") : Args(0) : ActionClass('REST') {
}

sub login_DELETE {
    my ( $self, $c ) = @_;

    # /rest/login is public, so we need to authenticate for a session deletion
    $c->detach unless $c->forward("authenticate");

    $c->stash->{session_object}->delete;
}

sub login_POST {
    my ( $self, $c ) = @_;

    my $logged_in = 0;

    $c->stash( login_via => "REST API" );

    # If basic auth, use that
    my ( $u, $p ) = $c->request->headers->authorization_basic;
    if ( $u && $p ) {
        $c->stash( ".login_username" => $u );
        $c->stash( ".login_password" => $p );
        $logged_in = $c->forward("/login_via_cat_plugins");

        # Return a basic auth request failure if not logged in
        if ( !$logged_in ) {
            $c->authenticate( { realm => "Opsview API" }, "http" );
        }
    }

    # Else use parameters
    elsif ( ( $u = delete $c->req->params->{username} ) && ( $p = delete $c->req->params->{password} ) ) {
        $c->stash( ".login_username" => $u );
        $c->stash( ".login_password" => $p );
        $logged_in = $c->forward("/login_via_cat_plugins");
    }

    # Else use content-type input
    elsif ( ( my $data = $c->req->data ) ) {
        if ( ( $u = $data->{username} ) && ( $p = $data->{password} ) ) {
            $c->stash( ".login_username" => $u );
            $c->stash( ".login_password" => $p );
            $logged_in = $c->forward("/login_via_cat_plugins");
        }
    }

    # If okay, generate token
    if ($logged_in) {
        $c->forward("set_token");
    }
    else {
        $c->res->status(401);
        $c->stash( "rest" => { message => "Invalid login" } );
    }
}

sub login_tkt : Chained("root") : PathPart("login_tkt") : Args(0) : ActionClass('REST') {
}

sub login_tkt_POST {
    my ( $self, $c ) = @_;

    $c->stash( login_via => "REST API using authtkt" );

    my $username = delete $c->req->params->{username} || ( $c->req->data && $c->req->data->{username} );

    unless ($username) {
        $c->res->status(401);
        $c->stash( "rest" => { message => "No username specified for REST login" } );
        $c->detach;
    }

    $c->stash(
        validate_ticket_extra_checks => sub {
            my ( $c, $ticket ) = @_;
            if ( $username ne $ticket->{uid} ) {
                $c->stash( login_error => $c->loc( "messages.authentication.usernameDoesNotMatchTicket [_1] [_2]", $username, $ticket->{uid} ) );
                return 0;
            }
            return 1;
        }
    );

    my $logged_in = $c->forward("/validate_ticket");

    # If okay, generate token
    if ($logged_in) {
        $c->forward("set_token");
    }
    else {
        $c->res->status(401);
        $c->stash( "rest" => { message => "Invalid login using authtkt", detail => $c->stash->{login_error} } );
        Opsview::Auditlog->system( "Invalid login using REST authtkt: " . $c->stash->{login_error} );
    }
}

sub reload : Chained("authenticated_root") : PathPart("reload") : Args(0) : Does(ACL) : RequiresRole(RELOADACCESS) : ACLDetachTo(/access_denied) : ActionClass('REST') {
}

sub reload_GET {
    my ( $self, $c ) = @_;
    my $web_status = Opsview::Nagios->web_status( { returns => "hash" } );
    $c->forward("/admin/status/get_configuration_status_information");
    $c->forward("/admin/reload/opsview");
    my $restdata = {
        server_status        => $web_status->{status},
        configuration_status => $c->stash->{configuration_status},
        lastupdated          => $web_status->{lastupdated},
        average_duration     => $c->stash->{average_time},
        auditlog_entries     => $c->stash->{number_of_auditlog_entries},
    };

    my @messages = map { { monitoringserver => ( $_->monitoringcluster ? $_->monitoringcluster->name : "" ), severity => $_->severity, detail => $_->message } } ( Opsview::Reloadmessage->retrieve_all );
    $restdata->{messages} = \@messages;

    $c->stash( "rest" => $restdata );
}

sub reload_POST {
    my ( $self, $c ) = @_;

    # Below is similar to O::W::C::Api->do_api where $action="reload"
    $c->stash( synchronous => 1 );
    $c->forward("/admin/reload/do_reload");
    $c->res->body(undef);    # Need to do because /admin/reload/do_reloads sets body to " "

    my $result     = {};
    my $web_status = $c->stash->{web_status};
    if ( $web_status->{status} eq "RETRY" ) {
        $result->{status}   = 1;
        $result->{messages} = ["Reload already running"];
        $c->res->status(409);
    }
    else {
        $self->reload_GET($c);
        $result = $c->stash->{rest};
        if ( $result->{server_status} == 2 || $result->{server_status} == 3 ) {
            $c->res->status(400);
        }
    }
    $c->stash( rest => $result );
}

sub serverinfo : Chained("authenticated_root") : PathPart("serverinfo") : Args(0) : Does(ACL) : RequiresRole(RELOADACCESS) : ACLDetachTo(/access_denied) : ActionClass('REST') {
}

# These mappings can probably be moved elsewhere, but we add here for simplicity
my $map_server_status_to_nagios_state = {
    0 => "ok",
    1 => "warning",
    2 => "critical",
    3 => "critical",
    4 => "warning",
};
my $map_status_to_nagios_code = {
    "ok"       => 0,
    "warning"  => 1,
    "critical" => 2,
    "unknown"  => 3,
};

sub serverinfo_GET {
    my ( $self, $c ) = @_;

    $self->reload_GET($c);
    my @msdata;
    my $restdata = {
        reload            => $c->stash->{rest},
        monitoringservers => \@msdata,
    };

    my $num_hosts = $c->model("OpsviewSchema::Hosts")->count;
    $restdata->{num_hosts} = $num_hosts;

    my $num_monitoringservers = $c->model("OpsviewSchema::Monitoringservers")->count;
    $restdata->{num_monitoringservers} = $num_monitoringservers;

    my $nodes_rs = $c->model("OpsviewSchema::Monitoringclusternodes");
    $restdata->{num_nodes} = $nodes_rs->count;

    my $nagios_status_servicename = "Nagios Stats";
    my $master                    = $c->model("OpsviewSchema::Monitoringservers")->find(1);

    my $perfdata = Runtime::Searches->list_services( $c->user, { includeperfdata => 1, servicecheck => [$nagios_status_servicename], from_opsview_web => 1 } );
    my $status = Runtime::Searches->list_services( $c->user, { servicecheck => ["Slave-node: %"], from_opsview_web => 1 } );

    my $nodestatus_by_name  = {};
    my $nodelatency_by_name = {};

    # Set master status based on server_status value
    $nodestatus_by_name->{ $master->host->name } = $map_server_status_to_nagios_state->{ $restdata->{reload}->{server_status} };

    # Parse through to pick out Slave-node status
    foreach my $host ( @{ $status->{list} } ) {
        foreach my $service ( @{ $host->{services} } ) {
            if ( $service->{name} =~ /^Slave-node: (.*)$/ ) {
                $nodestatus_by_name->{$1} = $service->{state};
            }
        }
    }
    foreach my $host ( @{ $perfdata->{list} } ) {
        foreach my $service ( @{ $host->{services} } ) {
            if ( lc( $service->{name} ) eq lc($nagios_status_servicename) ) {
                foreach my $metric ( @{ $service->{metrics} || [] } ) {
                    if ( $metric->{name} eq "avgactsvclatency" ) {
                        $nodelatency_by_name->{ $host->{name} } = $metric->{value};
                    }
                }
            }
        }
    }

    my $info_per_slavenode = {};

    # Create slave information
    my @nodes = $nodes_rs->search();
    unshift @nodes, $master;
    my @systems = $c->model("OpsviewSchema::Monitoringservers")->search();
    foreach my $system (@systems) {
        my @nodes = ( $system->is_master ? ($master) : $system->nodes );
        my $system_info = {
            name      => $system->name,
            activated => $system->activated,
            nodes     => [],
        };
        foreach my $node (@nodes) {
            my $nodename           = $node->host->name;
            my $nodeip             = $node->host->ip;
            my $info_per_slavenode = {
                name    => $nodename,
                ip      => $nodeip,
                status  => '3',
                latency => undef,
            };
            if ( exists $nodestatus_by_name->{$nodename} ) {
                $info_per_slavenode->{status} = $map_status_to_nagios_code->{ $nodestatus_by_name->{$nodename} } if exists $nodestatus_by_name->{$nodename};
            }
            if ( exists $nodelatency_by_name->{$nodename} ) {
                $info_per_slavenode->{latency} = $nodelatency_by_name->{$nodename} if exists $nodelatency_by_name->{$nodename};
            }
            push @{ $system_info->{nodes} }, $info_per_slavenode;
        }
        push @msdata, $system_info;
    }

    $c->stash( "rest" => $restdata );
}

1;
