#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::REST::Acknowledge;

use Moose;
use namespace::autoclean;
use Opsview::Utils qw(convert_to_arrayref unique_objects);
use Opsview::Common;
use Try::Tiny;

BEGIN {
    extends 'Catalyst::Controller::ActionRole', 'Opsview::Web::ControllerBase::REST', 'Opsview::Web::ControllerBase::REST::NagiosAction';
}

sub acknowledge : Chained("/rest/authenticated_root") : PathPart("acknowledge") : Args(0) : ActionClass('REST') {
}

# Lists all services that can be acknowledged based on state + user permissions
sub acknowledge_GET : Does(ACL) : AllowedRole(ACTIONSOME) : AllowedRole(ACTIONALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    my $args = $c->stash->{status_params} || $c->req->params;
    $args                          = {%$args};
    $args->{filter}                = "unhandled";
    $args->{includeunhandledhosts} = 1;
    my $asuser = $c->user;    # This could be switched to asuser later

    my $rs = $c->model("RuntimeSchema::OpsviewHostObjects");
    if ( !$asuser->has_access("ACTIONALL") ) {
        $rs = $rs->search( { "contacts.contactid" => $asuser->id }, { join => "contacts" } );
    }

    my $list_result = $rs->list_summary($args);

    $c->stash( 'rest' => $list_result );
}

sub acknowledge_POST : Does(ACL) : AllowedRole(ACTIONALL) : AllowedRole(ACTIONSOME) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    # Validate acknowledge parameters
    $c->forward("acknowledge_validate");

    if ( $_ = $c->stash->{validation_errors} ) {
        $c->res->status(403);
        $c->stash( "rest" => { message => "Error validating acknowledge parameters", detail => join( "; ", @$_ ) } );
        $c->detach;
    }

    # See which objects this should apply to
    $c->forward("find_objects");

    # If no objects, error
    if ( !@{ $c->stash->{hosts} } && !@{ $c->stash->{services} } ) {
        $c->res->status(403);
        $c->stash( "rest" => { message => "No objects chosen for acknowledgements" } );
        $c->detach;
    }

    $c->forward("acknowledge_set");

    # Any errors will result in an error returned. Cannot be sure which objects did work
    my @all_errors = @{ $c->stash->{all_errors} };
    if (@all_errors) {
        $c->res->status(403);
        $c->stash(
            "rest" => {
                message => "Errors setting acknowledgements - some objects may have been acknowledged",
                detail  => join( "", @all_errors ),
            }
        );
        $c->detach;
    }

    $c->forward("action_results");
}

# By the time the request gets here, assume all the security permissions have passed
# Also called from the web UI
sub acknowledge_set : Private {
    my ( $self, $c ) = @_;

    my @hosts    = @{ $c->stash->{hosts}    || [] };
    my @services = @{ $c->stash->{services} || [] };

    my $username = $c->user->name;
    my $ackdata  = $c->stash->{acknowledge_data};

    # Set acknowledgement for each object type
    my ( @done_hosts, @done_services );
    my @all_errors;
    foreach my $h (@hosts) {
        my $cmd = Opsview::Externalcommand->new(
            command => "ACKNOWLEDGE_HOST_PROBLEM",
            args    => join( ";", $h->name, $ackdata->{acknowledgement_type}, $ackdata->{notify}, $ackdata->{persistent}, $username, $ackdata->{comment} ),
        );
        my $errors = [];
        try { $cmd->submit } catch { $errors = convert_to_arrayref($_) };

        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_hosts, $h;
        }
    }
    foreach my $s (@services) {
        my $cmd = Opsview::Externalcommand->new(
            command => "ACKNOWLEDGE_SVC_PROBLEM",
            args    => join( ";", $s->hostname, $s->name, $ackdata->{acknowledgement_type}, $ackdata->{notify}, $ackdata->{persistent}, $username, $ackdata->{comment} ),
        );
        my $errors = [];
        try { $cmd->submit } catch { $errors = convert_to_arrayref($_) };

        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_services, $s;
        }
    }

    $c->stash->{done_hosts}    = \@done_hosts;
    $c->stash->{done_services} = \@done_services;

    $c->stash->{all_errors} = \@all_errors;
    if (@all_errors) {
        $c->log->warn("Got error during acknowledgement setting: @all_errors");
    }
    Opsview::Auditlog->create(
        {   username => $username,
            text     => $c->translate( "messages.auditlog.massAcknowledgement [_1] [_2] [_3] [_4] [_5]", $ackdata->{comment}, $ackdata->{notify}, $ackdata->{persistent}, scalar @done_hosts, scalar @done_services )
        }

        )

}

# Potentially could be called from web UI
sub acknowledge_validate : Private {
    my ( $self, $c ) = @_;

    my @errors;

    my $params  = $c->req->params;
    my $data    = $c->stash->{data} || $c->req->data || {};
    my $comment = $data->{comment} || $params->{comment};
    $comment = Opsview::Common->clean_comment($comment);
    unless ($comment) {
        push @errors, $c->loc("ui.downtime.help.commentEmpty");
    }

    my $notify = ( exists $data->{notify} ? $data->{notify} : ( $params->{notify} || 0 ) );

    # Nagios' persistent flag means comments remain after the ack is removed. This doesn't make sense, so we remove it every time
    my $persistent = 0;

    my $acknowledgement_type = ( exists $data->{sticky} ? $data->{sticky} : ( $params->{sticky} || 0 ) );

    if (@errors) {
        $c->stash( validation_errors => \@errors );
    }
    else {
        $c->stash(
            acknowledge_data => {
                comment              => $comment,
                notify               => $notify,
                persistent           => $persistent,
                acknowledgement_type => $acknowledgement_type,
            }
        );
    }
}

1;
