#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::REST::Downtime;

use Moose;
use namespace::autoclean;
use Opsview::Utils qw(convert_to_arrayref unique_objects);
use Opsview::Common;

BEGIN { extends 'Catalyst::Controller::ActionRole', 'Opsview::Web::ControllerBase::REST', 'Opsview::Web::ControllerBase::REST::NagiosAction' }

sub downtime : Chained("/rest/authenticated_root") : PathPart("downtime") : Args(0) : ActionClass('REST') {
}

sub downtime_GET : Does(ACL) : AllowedRole(VIEWSOME) : AllowedRole(VIEWALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    my $args = $c->stash->{downtime_params} || $c->req->params;
    my $rs = $c->model("RuntimeSchema::OpsviewHostObjects");

    if ( !$c->user->has_access("VIEWALL") ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }
    my $list_result = $rs->list_downtimes($args);

    $c->stash( 'rest' => $list_result );
}

sub downtime_DELETE : Does(ACL) : AllowedRole(DOWNTIMEALL) : AllowedRole(DOWNTIMESOME) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    my $start_time_dt = "";
    if ( $_ = $c->req->params->{start_time} ) {
        $start_time_dt = Opsview::Common->parse_datetime_string($_);
    }

    # Find objects
    $c->forward( "find_objects", ["DOWNTIMEALL"] );

    # If no objects, error
    if ( !@{ $c->stash->{hostgroups} } && !@{ $c->stash->{hosts} } && !@{ $c->stash->{services} } ) {
        $c->res->status(403);
        $c->stash( "rest" => { message => "No objects chosen for downtime deletion" } );
        $c->detach;
    }

    # Delete downtimes based on objects - add filtering options if present
    $c->stash->{downtime_data} = {
        start_time_dt => $start_time_dt,
        comment       => Opsview::Common->clean_comment( $c->req->params->{comment} ),
    };
    $c->forward("downtime_deletion");

    # Any errors will result in an error returned. Cannot be sure which objects did work
    my @all_errors = @{ $c->stash->{all_errors} };
    if (@all_errors) {
        $c->res->status(403);
        $c->stash(
            "rest" => {
                message => "Errors deleting downtime - some objects may have downtime deleted",
                detail  => join( "", @all_errors ),
            }
        );
        $c->detach;
    }

    $c->forward("downtime_results");
}

sub downtime_POST : Does(ACL) : AllowedRole(DOWNTIMEALL) : AllowedRole(DOWNTIMESOME) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    # Validate downtime parameters
    $c->forward("downtime_validate");

    if ( $_ = $c->stash->{validation_errors} ) {
        $c->res->status(403);
        $c->stash( "rest" => { message => "Error validating downtime parameters", detail => join( "; ", @$_ ) } );
        $c->detach;
    }

    # See which objects this should apply to
    $c->forward( "find_objects", ["DOWNTIMEALL"] );

    # If no objects, error
    if ( !@{ $c->stash->{hostgroups} } && !@{ $c->stash->{hosts} } && !@{ $c->stash->{services} } ) {
        $c->res->status(403);
        $c->stash( "rest" => { message => "No objects chosen for downtime creation" } );
        $c->detach;
    }

    # Set the downtime
    $c->forward("downtime_set");

    # Any errors will result in an error returned. Cannot be sure which objects did work
    my @all_errors = @{ $c->stash->{all_errors} };
    if (@all_errors) {
        $c->res->status(403);
        $c->stash(
            "rest" => {
                message => "Errors setting downtime - some objects may have downtime set",
                detail  => join( "", @all_errors ),
            }
        );
        $c->detach;
    }

    $c->forward("downtime_results");
}

sub downtime_results : Private {
    my ( $self, $c ) = @_;
    my @done_hostgroups = @{ $c->stash->{done_hostgroups} };
    my @done_hosts      = @{ $c->stash->{done_hosts} };
    my @done_services   = @{ $c->stash->{done_services} };

    my $result = {
        summary => {
            num_hostgroups => scalar @done_hostgroups,
            num_hosts      => scalar @done_hosts,
            num_services   => scalar @done_services,
        },
    };

    $result->{list} = {
        hostgroups => [],
        hosts      => [],
        services   => [],
    };
    foreach my $hg (@done_hostgroups) {
        push @{ $result->{list}->{hostgroups} }, { name => $hg->name, id => $hg->id };
    }
    foreach my $h (@done_hosts) {
        push @{ $result->{list}->{hosts} }, { hostname => $h->name, id => $h->id };
    }
    foreach my $s (@done_services) {
        push @{ $result->{list}->{services} }, { hostname => $s->hostname, servicename => $s->servicename, id => $s->id };
    }
    $c->stash( 'rest' => $result );
}

# By the time the request gets here, assume all the security permissions have passed
# Also called from the web UI for downtime deletions
sub downtime_deletion : Private {
    my ( $self, $c ) = @_;

    my @hostgroups = @{ $c->stash->{hostgroups} || [] };
    my @hosts      = @{ $c->stash->{hosts}      || [] };
    my @services   = @{ $c->stash->{services}   || [] };

    # Set downtimes for each object type
    my ( @done_hostgroups, @done_hosts, @done_services );
    my @all_errors;
    foreach my $hg (@hostgroups) {
        my $errors = $hg->delete_downtime( $c->user, $c->stash->{downtime_data} );
        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_hostgroups, $hg;
        }
    }
    foreach my $h (@hosts) {
        my $errors = $h->delete_downtime( $c->user, $c->stash->{downtime_data} );
        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_hosts, $h;
        }
    }
    foreach my $s (@services) {
        my $errors = $s->delete_downtime( $c->user, $c->stash->{downtime_data} );
        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_services, $s;
        }
    }

    $c->stash->{done_hostgroups} = \@done_hostgroups;
    $c->stash->{done_hosts}      = \@done_hosts;
    $c->stash->{done_services}   = \@done_services;

    $c->stash->{all_errors} = \@all_errors;
    if (@all_errors) {
        $c->log->warn("Got error during downtime deletion: @all_errors");
    }
}

# This is not currently used. Left here for deletions of comments that affect other items
sub downtime_delete_by_start_time_comment : Private {
    my ( $self, $c ) = @_;

    # Check if user has access to do deletion by comment

    # Send comment
    $c->stash->{all_errors} = $c->model("RuntimeSchema::NagiosScheduleddowntimes")->delete_downtimes_by_start_time_comment( $c->user, $c->stash->{downtime_data}, );
}

# By the time the request gets here, assume all the security permissions have passed
# Also called from the web UI for downtimes
sub downtime_set : Private {
    my ( $self, $c ) = @_;

    my @hostgroups = @{ $c->stash->{hostgroups} || [] };
    my @hosts      = @{ $c->stash->{hosts}      || [] };
    my @services   = @{ $c->stash->{services}   || [] };

    # Set downtimes for each object type
    my ( @done_hostgroups, @done_hosts, @done_services );
    my @all_errors;
    foreach my $hg (@hostgroups) {
        my $errors = $hg->set_downtime( $c->user, $c->stash->{downtime_data} );
        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_hostgroups, $hg;
        }
    }
    foreach my $h (@hosts) {
        my $errors = $h->set_downtime( $c->user, $c->stash->{downtime_data} );
        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_hosts, $h;
        }
    }
    foreach my $s (@services) {
        my $errors = $s->set_downtime( $c->user, $c->stash->{downtime_data} );
        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_services, $s;
        }
    }

    $c->stash->{done_hostgroups} = \@done_hostgroups;
    $c->stash->{done_hosts}      = \@done_hosts;
    $c->stash->{done_services}   = \@done_services;

    $c->stash->{all_errors} = \@all_errors;
    if (@all_errors) {
        $c->log->warn("Got error during downtime setting: @all_errors");
    }

}

# Also called from the web UI for downtimes
sub downtime_validate : Private {
    my ( $self, $c ) = @_;

    my @errors;

    my $params  = $c->req->params;
    my $data    = $c->stash->{data} || $c->req->data || {};
    my $comment = $data->{comment} || $params->{comment};
    if ( !defined $comment ) {
        push @errors, $c->loc("ui.downtime.help.commentEmpty");
    }
    $comment = Opsview::Common->clean_comment($comment);

    my ( $start_dt, $end_dt ) = parse_downtime_strings( $data->{starttime} || $params->{starttime}, $data->{endtime} || $params->{endtime} );

    if ( !$start_dt || !$end_dt ) {
        push @errors, $@;
    }

    if (@errors) {
        $c->stash( validation_errors => \@errors );
    }
    else {
        $c->stash(
            downtime_data => {
                start_time  => $start_dt,
                start_timev => $start_dt->epoch,
                end_time    => $end_dt,
                end_timev   => $end_dt->epoch,
                comment     => $comment
            }
        );
    }
}

1;
