#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::REST::Recheck;

use Moose;
use namespace::autoclean;
use Opsview::Utils qw(convert_to_arrayref unique_objects);
use Opsview::Common;
use Try::Tiny;

BEGIN { extends 'Catalyst::Controller::ActionRole', 'Opsview::Web::ControllerBase::REST', 'Opsview::Web::ControllerBase::REST::NagiosAction' }

sub recheck : Chained("/rest/authenticated_root") : PathPart("recheck") : Args(0) : ActionClass('REST') {
}

# Lists all services that can be rechecked based on user permissions
sub recheck_GET : Does(ACL) : AllowedRole(ACTIONSOME) : AllowedRole(ACTIONALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    my $args = $c->stash->{status_params} || $c->req->params;
    $args = {%$args};
    my $asuser = $c->user;    # This could be switched to asuser later

    my $rs = $c->model("RuntimeSchema::OpsviewHostObjects");
    if ( !$asuser->has_access("ACTIONALL") ) {
        $rs = $rs->search( { "contacts.contactid" => $asuser->id }, { join => "contacts" } );
    }

    my $list_result = $rs->list_summary($args);

    $c->stash( 'rest' => $list_result );
}

sub recheck_POST : Does(ACL) : AllowedRole(ACTIONALL) : AllowedRole(ACTIONSOME) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    # See which objects this should apply to
    $c->forward("find_objects");

    # If no objects, error
    if ( !@{ $c->stash->{hosts} } && !@{ $c->stash->{services} } ) {
        $c->res->status(403);
        $c->stash( "rest" => { message => "No objects chosen for recheck" } );
        $c->detach;
    }

    $c->forward("recheck_do");

    # Any errors will result in an error returned. Cannot be sure which objects did work
    my @all_errors = @{ $c->stash->{all_errors} };
    if (@all_errors) {
        $c->res->status(403);
        $c->stash(
            "rest" => {
                message => "Errors requesting recheck - some objects may have been rechecked",
                detail  => join( "", @all_errors ),
            }
        );
        $c->detach;
    }

    $c->forward("action_results");
}

# By the time the request gets here, assume all the security permissions have passed
# Also called from the web UI
sub recheck_do : Private {
    my ( $self, $c ) = @_;

    my @hosts    = @{ $c->stash->{hosts}    || [] };
    my @services = @{ $c->stash->{services} || [] };

    my $username = $c->user->name;

    # Set recheck for each object type
    my ( @done_hosts, @done_services );
    my @all_errors;
    my $ts = time;
    foreach my $h (@hosts) {
        my $cmd = Opsview::Externalcommand->new(
            command => "SCHEDULE_HOST_CHECK",
            args    => join( ";", $h->name, $ts ),
        );
        my $errors = [];
        try { $cmd->submit } catch { $errors = convert_to_arrayref($_) };

        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_hosts, $h;
        }
    }
    foreach my $s (@services) {
        my $cmd = Opsview::Externalcommand->new(
            command => "SCHEDULE_SVC_CHECK",
            args    => join( ";", $s->hostname, $s->name, $ts ),
        );
        my $errors = [];
        try { $cmd->submit } catch { $errors = convert_to_arrayref($_) };

        if (@$errors) {
            push @all_errors, @$errors;
        }
        else {
            push @done_services, $s;
        }
    }

    $c->stash->{done_hosts}    = \@done_hosts;
    $c->stash->{done_services} = \@done_services;

    $c->stash->{all_errors} = \@all_errors;
    if (@all_errors) {
        $c->log->warn("Got error during recheck: @all_errors");
    }
    Opsview::Auditlog->create(
        {   username => $username,
            text     => $c->translate( "messages.auditlog.massRecheck [_1] [_2]", scalar @done_hosts, scalar @done_services )
        }
    );

}

1;
