#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::RRDfetch;

use strict;
use warnings;
use Opsview::Utils qw(convert_to_arrayref convert_uoms);
use Opsview::Systempreference;
use RRDs;
use Statistics::Lite qw(max min mean);
use List::MoreUtils qw(uniq);
use Utils::Nagiosgraph qw(urlencode);

# Timezone support via tz_local_offset
# Flot wants data in epoch seconds, but always renders UTC. So we need to add offset information
use Time::Zone;

use base qw/Catalyst::Controller/;

=head1 /rrdfetch

This returns back json or csv data based on the parameters passed

=head2 auto

Checks authentication

=cut

sub auto : Private {
    my ( $self, $c, $id ) = @_;
    $c->res->status(401);    # Set status code to 401 unless authenticated
    my $access_okay = $c->forward( "/check_public_or_user_access", ["RRDGRAPHS"] );
    if ($access_okay) {
        $c->res->status(200);
    }
    return $access_okay;
}

=head2 default - for /rrdfetch

Returns back data from RRD.

The time value returned is offset by the server timezone.

The json data is for consumption by the jsgraphs. Flot expects the time to be "seconds since epoch", but based on a local time zone

The csv output is structured to allow reading in a spreadsheet. WARNING: The CSV routine assumes that multiple data points
have the same time index - I think this is not true if RRDs are created with different ranges

=cut

# This only returns the data, so doesn't need all the attributes that /rrdgraph uses
sub default : Path {
    my ( $self, $c ) = @_;

    # NOTE: start and end times are UTC+offset
    # We calculate the offset based on now, so there may be a discrepency when asking for times
    # either side of a DST change
    my $tz_offset = tz_local_offset();

    # Basic RRD attributes
    my $params = $c->req->params;
    my $end = $params->{end} || time;
    $end = $end - $tz_offset;
    my $start = $params->{start};

    if ( !defined $start ) {
        my $duration = $params->{duration} || "1d";
        $start = "end-$duration";
    }
    else {
        $start = $start - $tz_offset;
    }
    $c->stash( 'start' => $start );
    $c->stash( 'end'   => $end );

    my $output = $params->{output} || "json";

    my $hsm_list = convert_to_arrayref( $params->{hsm} );

    # Sort list of hsm, to make thresholds lower than metrics
    $hsm_list = [ uniq sort @$hsm_list ];

    my $rs = $c->model("RuntimeSchema::OpsviewPerformanceMetrics");

    # Add auth. Need to check user actually exists due to possibly public view
    if ( $c->user_exists && $c->user->has_access("VIEWSOME") ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }

    my $rrd_files = {};
    my $rrddir    = "/usr/local/nagios/var/rrd";

    my $metric_specific_information = sub {
        my ( $rrd_files, $perfmetric, $thresholdname ) = @_;
        my $hostname    = $perfmetric->hostname;
        my $servicename = $perfmetric->servicename;
        my $metricname  = $perfmetric->metricname;
        my $rrd_filename;
        if ($thresholdname) {
            $rrd_filename = "$rrddir/" . join( "/", urlencode($hostname), urlencode($servicename), urlencode($metricname), "thresholds.rrd" );
            if ( !-e $rrd_filename ) {
                return;
            }
            if ( !exists $rrd_files->{$rrd_filename} ) {
                $rrd_files->{$rrd_filename} = {
                    filename   => $rrd_filename,
                    uom        => $perfmetric->uom,
                    label      => join( "::", $hostname, $servicename, $metricname ),
                    thresholds => [$thresholdname],
                };
            }
            else {
                push @{ $rrd_files->{$rrd_filename}->{thresholds} }, $thresholdname;
            }
        }
        else {
            $rrd_filename = "$rrddir/" . join( "/", urlencode($hostname), urlencode($servicename), urlencode($metricname), "value.rrd" );
            if ( !-e $rrd_filename ) {
                return;
            }
            $rrd_files->{$rrd_filename} = {
                filename => $rrd_filename,
                uom      => $perfmetric->uom,
                label    => join( "::", $hostname, $servicename, $metricname ),
            };
        }
    };

    # Foreach hsm, search table if exists
    foreach my $hsm (@$hsm_list) {
        my ( $hostname, $servicename, $metricname, $thresholdname ) = split( "::", $hsm );
        if ( my $perfmetric = $rs->search( { hostname => $hostname, servicename => $servicename, metricname => $metricname } )->first ) {
            $metric_specific_information->( $rrd_files, $perfmetric, $thresholdname );
        }
    }
    if ( my $keyword = $c->req->param("keyword") ) {
        $keyword = convert_to_arrayref($keyword);
        foreach my $perfmetric ( $rs->search( { "keywords.keyword" => $keyword }, { join => "keywords", distinct => 1 } ) ) {
            $metric_specific_information->( $rrd_files, $perfmetric );
        }
    }

    #  If not exists, error if hsm, ignore if threshold - ?????
    #  Else, add to list of files, with hsmt label, indexed by file. Add uom info
    #
    # Foreach keys file, read data and add label to it and summary data. Push onto results
    # We sort the files so that the graph has a deterministic order for the graph lines
    my @lines    = ();
    my @csv_data = ();
    foreach my $rrd_file ( sort keys %$rrd_files ) {

        # Routine to extract data is very similar for values + thresholds, so make a sub here to reuse
        my $line_data_info = sub {
            my ( $start, $step, $data, $index, $uom ) = @_;

            # Work out if you should translate the uom to something else
            my ( $uom_new, $uom_multiplier ) = convert_uoms($uom);

            my $time       = $start;
            my @data_plots = ();
            my @raw_data   = ();
            foreach my $line (@$data) {
                my $val = $line->[$index];
                $val *= $uom_multiplier if defined $val;
                push @data_plots, [ $time + tz_local_offset($time), $val ];
                push @raw_data, $val;
                $time += $step;
            }
            my $min = min(@raw_data);
            my $max = max(@raw_data);
            my $avg = mean(@raw_data);

            my $line_info = {};
            $line_info->{data}        = \@data_plots;
            $line_info->{description} = "Min: " . ( defined $min ? sprintf( "%.3f", $min ) : "n/a" ) . "<br />Max: " . ( defined $max ? sprintf( "%.3f", $max ) : "n/a" ) . "<br />Average: " . ( defined $avg ? sprintf( "%.3f", $avg ) : "n/a" );
            $line_info->{uom}         = $uom_new;
            return $line_info;
        };

        # I thought you could get the current value from RRDs::info, but the value there doesn't
        # seem to match up correctly. Removing this for the moment
        # A possibility is working out the current value from the last service result
        #my $rrd_info = RRDs::info( $rrd_file );
        #my $current = exists $rrd_info->{ "ds[value].value" } ? $rrd_info->{ "ds[value].value" } : "unknown";

        my ( $data_start, $step, $names, $data ) = RRDs::fetch( $rrd_file, "AVERAGE", "-s", $start, "-e", $end );
        if (RRDs::error) {
            $c->log->warn( "RRDs::fetch error: " . (RRDs::error) );
            next;
        }
        if ( exists $rrd_files->{$rrd_file}->{thresholds} ) {
            my $name_to_index = {};
            my $i             = 0;
            foreach my $name (@$names) {
                $name_to_index->{$name} = $i++;
            }
            foreach my $threshold ( @{ $rrd_files->{$rrd_file}->{thresholds} } ) {
                my $index = $name_to_index->{"${threshold}_end"};
                my $line_info = $line_data_info->( $data_start, $step, $data, $index, $rrd_files->{$rrd_file}->{uom} );
                $line_info->{label} = join( "::", $rrd_files->{$rrd_file}->{label}, $threshold );
                push @lines, $line_info;
            }
        }
        else {

            if ( $output eq "json" ) {
                my $line_info = $line_data_info->( $data_start, $step, $data, 0, $rrd_files->{$rrd_file}->{uom} );
                $line_info->{label} = $rrd_files->{$rrd_file}->{label};
                push @lines, $line_info;
            }
            elsif ( $output eq "csv" ) {

                # For csv data, we need to format it so that it returns:
                # [ [ time, a1, b1, c1, d1, ...], [ time2, a2, b2, c2, d2, ...], ... ]
                # This needs to work if there are multiple hsms returned
                my $have_data = scalar @csv_data;
                my $i         = 0;
                for my $line (@$data) {
                    if ( !$have_data ) {
                        if ( $i == 0 ) {
                            push @csv_data, [ "Time", $rrd_files->{$rrd_file}->{label} ];
                        }
                        my @time = localtime($data_start);
                        my $nice_time = sprintf( "%04d-%02d-%02d %02d:%02d:%02d", $time[5] + 1900, $time[4] + 1, $time[3], $time[2], $time[1], $time[0] );
                        unshift @$line, $nice_time;
                        $data_start += $step;
                        push @csv_data, $line;
                    }
                    else {
                        if ( $i == 0 ) {
                            push @{ $csv_data[$i] }, $rrd_files->{$rrd_file}->{label};
                            $i++;
                        }
                        push @{ $csv_data[$i] }, @$line;
                    }
                    $i++;
                }

            }

        }

    }

    # TODO Tests:
    #  something with warning, but no critical
    #  something with critical, but no warning
    #  something with warning and critical flipped

    $c->stash->{output} = $output;

    if ( $output eq "json" ) {
        $c->stash( ResultSet => { lines => \@lines } );
    }
    elsif ( $output eq "csv" ) {
        $c->stash->{'csv'} = { 'data' => \@csv_data };
        my $title = $c->req->param("title");
        $title =~ s/[^\w]/_/g;
        $c->res->header( 'Content-Disposition', qq[attachment; filename="$title.csv"] );
    }
}

1;
