#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::RRDgraph;

use strict;
use warnings;
use Opsview::Utils qw(convert_to_arrayref convert_uoms);
use Opsview::Systempreference;
use RRDs;
use File::Slurp;
use Clone qw(clone);

use base qw/Catalyst::Controller/;

sub auto : Private {
    my ( $self, $c, $id ) = @_;
    return $c->forward( "/check_public_or_user_access", ["RRDGRAPHS"] );
}

sub default : Path {
    my ( $self, $c ) = @_;

    # Basic RRD attributes
    my $params = clone( $c->req->params );
    $c->stash( query_params => $params );
    my $end = $params->{end} || time;
    my $start = $params->{start};
    if ( !defined $start ) {
        my $duration = $params->{duration} || "1d";
        $start = "end-$duration";
    }
    my $height = $params->{height} || 200;
    my $width  = $params->{width}  || 400;

    my $only_graph = $params->{'only-graph'};

    #Option not supported yet by rrd shipped with Opsview
    #my $full_size_mode = $params->{'full-size-mode'};
    my $upper_limit   = $params->{'upper-limit'};
    my $lower_limit   = $params->{'lower-limit'};
    my $rigid         = $params->{'rigid'};
    my $alt_autoscale = $params->{'alt-autoscale'};

    #Option not supported yet by rrd shipped with Opsview
    #my $alt_autoscale_min = $params->{'alt-autoscale-min'};
    my $alt_autoscale_max        = $params->{'alt-autoscale-max'};
    my $no_gridfit               = $params->{'no-gridfit'};
    my $x_grid                   = $params->{'x-grid'};
    my $y_grid                   = $params->{'y-grid'};
    my $alt_y_grid               = $params->{'alt-y-grid'};
    my $logarithmic              = $params->{'logarithmic'};
    my $units_exponent           = $params->{'units-exponent'};
    my $units_length             = $params->{'units-length'};
    my $units                    = $params->{'units'};
    my $zoom                     = $params->{'zoom'};
    my $font_render_mode         = $params->{'font-render-mode'};
    my $font_smoothing_threshold = $params->{'font-smoothing-threshold'};
    my $pango_markup             = $params->{'pango-markup'};
    my $graph_render_mode        = $params->{'graph-render-mode'};
    my $slope_mode               = $params->{'slope-mode'};
    my $interlaced               = $params->{'interlaced'};
    my $base                     = $params->{'base'};
    my $multiplier               = $params->{'multiplier'};
    my $column_width             = $params->{'column-width'} || 7;
    my $decimals                 = defined( $params->{'decimals'} ) ? $params->{'decimals'} : 2;

    my $graph_format = $params->{graph_format} || "PNG";
    my $cf           = $params->{cf}           || "AVERAGE";
    my $colours      = $params->{colours}      || "CC0000,FF9900,00CC00,0000CC,336699,993300,FF3300,99CC00,3366FF,660099,999999,CC6600";
    my @colours = split( ",", $colours );
    my $showlegend = $params->{legend};
    $showlegend = 1 if not defined $showlegend;
    my $fullname          = $params->{fullname} || 0;
    my $vertical_label    = $params->{'vertical-label'};
    my $title             = $params->{'title'};
    my $stacked           = $params->{stack} || 0;
    my $includethresholds = $params->{includethresholds} || 0;
    my $fixedscale        = ( exists $params->{fixedscale} and $params->{fixedscale} =~ m/\d+/ ) ? $params->{fixedscale} : undef;
    my $line              = $params->{line} || 1;
    my $type              = "LINE$line";
    if ($stacked) {
        $stacked = ":STACK";
        $type    = "AREA";
    }
    else {
        $stacked = "";
    }

    my $hosts    = convert_to_arrayref( $params->{host} );
    my $services = convert_to_arrayref( $params->{service} );

    # Also handles permissions
    my $results = $c->forward("/common/list_rrds");

    my @auth_errors = @{ $results->{errors} };
    if (@auth_errors) {
        $c->stash->{error} = join( "; ", @auth_errors );
        $c->detach("/soft_error");
    }
    if ( !$results->{summary}->{metrics} ) {
        $c->stash->{error} = "No datasets requested";
        $c->detach("/soft_error");
    }
    my $rrdoptions = [ "--start", $start, "--end", $end, "--height", $height, "--width", $width ];
    if ( defined $fixedscale )     { push @$rrdoptions, "-X",               $fixedscale }
    if ( defined $vertical_label ) { push @$rrdoptions, "--vertical-label", $vertical_label }
    if ( defined $title )          { push @$rrdoptions, "--title",          $title }
    if ( defined $only_graph )     { push @$rrdoptions, "--only-graph" }

    #if (defined $full_size_mode){ push @$rrdoptions, "--full-size-mode"; }
    if ( defined $upper_limit ) { push @$rrdoptions, "--upper-limit", $upper_limit }
    if ( defined $lower_limit ) { push @$rrdoptions, "--lower-limit", $lower_limit }
    if ( defined $rigid )       { push @$rrdoptions, "--rigid" }
    if ( defined $alt_autoscale ) { push @$rrdoptions, "--alt-autoscale" }

    #if (defined $alt_autoscale_min){          push @$rrdoptions, "--alt-autoscale-min", $alt_autoscale_min }
    if ( defined $alt_autoscale_max )        { push @$rrdoptions, "--alt-autoscale-max" }
    if ( defined $no_gridfit )               { push @$rrdoptions, "--no-gridfit" }
    if ( defined $x_grid )                   { push @$rrdoptions, "--x-grid", $x_grid }
    if ( defined $y_grid )                   { push @$rrdoptions, "--y-grid", $y_grid }
    if ( defined $alt_y_grid )               { push @$rrdoptions, "--alt-y-grid" }
    if ( defined $logarithmic )              { push @$rrdoptions, "--logarithmic" }
    if ( defined $units_exponent )           { push @$rrdoptions, "--units-exponent", $units_exponent }
    if ( defined $units_length )             { push @$rrdoptions, "--units-length", $units_length }
    if ( defined $units )                    { push @$rrdoptions, "--units", $units }
    if ( defined $zoom )                     { push @$rrdoptions, "--zoom", $zoom }
    if ( defined $font_render_mode )         { push @$rrdoptions, "--font-render-mode", $font_render_mode }
    if ( defined $font_smoothing_threshold ) { push @$rrdoptions, "--font-smoothing-threshold", $font_smoothing_threshold }
    if ( defined $pango_markup )             { push @$rrdoptions, "--pango-markup" }
    if ( defined $graph_render_mode )        { push @$rrdoptions, "--graph-render-mode", $graph_render_mode }
    if ( defined $slope_mode )               { push @$rrdoptions, "--slope-mode" }
    if ( defined $interlaced )               { push @$rrdoptions, "--interlaced" }
    if ( defined $base )                     { push @$rrdoptions, "--base", $base }

    my $ds                = $results->{metrics};
    my $colour_index      = 0;
    my $num_colours       = @colours;
    my $max_legend_length = 0;

    # first pass through metrics will complete data in $hash that isn't in $results
    foreach my $dsname ( @{ $results->{metric_order} } ) {
        foreach my $hash ( @{ $ds->{$dsname} } ) {

            # $vname is a unique datapoint name, stripped of invalid characters
            my $vname = "$hash->{host}-$hash->{service}-$dsname";
            $vname =~ s/[^a-zA-Z0-9-_]/_/g;
            $hash->{'vname'} = $vname;

            my $legend = "";
            if ( $showlegend == 1 or $showlegend == 2 ) {

                # $displayname is a calculated name, based on whether there are multiple hosts/services or not
                my $displayname = $dsname;
                if ( $fullname || scalar @$services > 1 ) {
                    $displayname = "$hash->{service}::$dsname";
                }
                if ( $fullname || scalar @$hosts > 1 ) {
                    $displayname = "$hash->{host}::$displayname";
                }
                $hash->{'legend'} = $displayname;
                $max_legend_length = length($displayname) if ( length($displayname) > $max_legend_length );

                $displayname =~ s/[^a-zA-Z0-9: -_]/_/g;
                $displayname =~ s/\:/\\:/g;
                $hash->{'displayname'} = $displayname;
            }
        }
    }

    my $format_specifier = "\%${column_width}.${decimals}lf";
    if ( $showlegend == 2 ) {
        my $text_metric = $c->loc("ui.graph.ongraph.short.metric");
        my $text_cur    = $c->loc("ui.graph.ongraph.short.current");
        my $text_max    = $c->loc("ui.graph.ongraph.short.max");
        my $text_min    = $c->loc("ui.graph.ongraph.short.min");
        my $text_avg    = $c->loc("ui.graph.ongraph.short.average");
        my $cap         = " $text_metric" . ( ' ' x ( $max_legend_length - length($text_metric) ) );
        $cap .= ( ' ' x ( $column_width - length($text_cur) + 2 ) ) . $text_cur;
        $cap .= ( ' ' x ( $column_width - length($text_max) + 2 ) ) . $text_max;
        $cap .= ( ' ' x ( $column_width - length($text_min) + 2 ) ) . $text_min;
        $cap .= ( ' ' x ( $column_width - length($text_avg) + 2 ) ) . $text_avg;
        push @$rrdoptions, "COMMENT: $cap\\n";
    }

    foreach my $dsname ( @{ $results->{metric_order} } ) {
        foreach my $hash ( @{ $ds->{$dsname} } ) {
            my $f             = $hash->{file};
            my $vname         = $hash->{vname};
            my $legend        = $hash->{'legend'} || "";
            my $fixedw_legend = ( $hash->{'displayname'} || "" ) . ( ' ' x ( $max_legend_length - length($legend) ) );
            my $nextcolour    = $colours[ $colour_index++ % $num_colours ];

            push @$rrdoptions, "DEF:$vname=$f:value:AVERAGE";

            my ( $uom_file, $uom, $new_uom, $uom_multiplier );
            ( $uom_file = $f ) =~ s/value.rrd$/uom/;
            if ( -f $uom_file ) {
                $uom = read_file($uom_file);
                ( $new_uom, $uom_multiplier ) = convert_uoms($uom);
                if ( $c->debug ) {
                    $c->log->_dump($hash);
                    $c->log->debug("uom_file=$uom_file");
                    $c->log->debug("uom=$uom");
                    $c->log->debug("new_uom=$new_uom");
                    $c->log->debug("uom_multiplier=$uom_multiplier");
                }
            }

            $multiplier = undef if ( $multiplier && $multiplier =~ m/\D/ );
            $uom_multiplier = $multiplier if ($multiplier);
            if ($uom_multiplier) {
                push @$rrdoptions, "CDEF:$vname-ds0bytes=$vname,$uom_multiplier,*";
                push @$rrdoptions, "$type:$vname-ds0bytes#$nextcolour:$fixedw_legend$stacked";
            }
            else {
                push @$rrdoptions, "$type:$vname#$nextcolour:$fixedw_legend$stacked";
            }

            #"GPRINT:$vname".':LAST:\tCur\: %6.2lf%s\n';

            if ( $showlegend == 2 ) {
                push @$rrdoptions, "GPRINT:$vname:LAST:$format_specifier", "GPRINT:$vname:MAX:$format_specifier", "GPRINT:$vname:MIN:$format_specifier", "GPRINT:$vname:AVERAGE:$format_specifier", "COMMENT:\\n";
            }

            $c->log->_dump($rrdoptions) if ( $c->debug );

            next unless ($includethresholds);

            $f = $hash->{threshold}->{file};
            foreach my $level (qw(warning critical)) {
                my $level_dsname = "${level}_end";
                if ( exists $hash->{threshold}->{$level_dsname} ) {
                    my $levelname    = "$vname-$level";
                    my $level_legend = "";
                    if ( $showlegend == 1 or $showlegend == 2 ) {
                        $level_legend = "$hash->{'displayname'}-$level";
                    }
                    $nextcolour = $colours[ $colour_index++ % $num_colours ];
                    push @$rrdoptions, "DEF:$levelname=$f:$level_dsname:AVERAGE", "$type:$levelname#$nextcolour:$level_legend$stacked", "COMMENT:\\n";
                }
            }
        }
    }
    $c->stash->{graph} = $rrdoptions;
    $c->forward("Opsview::Web::View::RRDGraph");
}

1;
