#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Root;

use strict;
use warnings;
use base qw/Catalyst::Controller/;

use Apache::AuthTkt;
use Opsview::Utils;
use Utils::Weberrors;
use Data::Dump qw(dump);
use DateTime;
use DateTime::Format::W3CDTF;

__PACKAGE__->mk_accessors(qw/start_url languages authtkt_ignoreip always_show_welcome_page/);

#
# Sets the actions in this controller to be registered with no prefix
# so they function identically to actions created in MyApp.pm
#
__PACKAGE__->config->{namespace} = '';

sub auto : Private {
    my ( $self, $c ) = @_;

    # Strip <> from cookie data
    # Need to be done very early before anything else reads the cookies, such as $c->user
    my $h = $c->req->cookies;
    foreach my $n ( keys %$h ) {
        my @vals = $h->{$n}->value();
        @vals = map { s/[<>]//g; $_ } @vals;
        $h->{$n}->value( \@vals );
    }

    my $f = {};
    $c->stash( functions => $f );
    $c->stash->{stylesheets}      = [];
    $c->stash->{extra_javascript} = [];
    $c->stash->{debug}            = $c->config->{static}->{debug};
    $c->stash->{useragent}        = $c->request->headers->{"user-agent"};
    $c->forward("/set_language");
    $c->session_expires(1);    # Reset the expiry for the session
    $c->stash->{page_title}        = "";
    $c->stash->{system_preference} = $c->model("OpsviewSchema::Systempreferences")->find(1);

    my $include_header = delete $c->req->params->{include_header};
    if ( !defined $include_header ) {
        $include_header = 1;
    }
    my $include_footer = delete $c->req->params->{include_footer};
    if ( !defined $include_footer ) {
        $include_footer = 1;
    }
    $c->stash->{include_header} = $include_header;
    $c->stash->{include_footer} = $include_footer;

    if ( $c->is_community_edition ) {
        $c->stash( footer_logo_url => "/images/OpsviewCommunityLogo.png" );
        $c->stash( logo_url        => "/images/OpsviewCommunityIcon.png" );
    }
    else {
        $c->stash( footer_logo_url => "/images/OpsviewEnterpriseLogo.png" );
        $c->stash( logo_url        => "/images/OpsviewEnterpriseIcon.png" );
    }

    # Override with custom logo if defined
    my $logo = Opsview::Config->logo_path;
    if ($logo) {
        $c->stash( logo_url => $logo );
    }

    1;    # Force continue to next auto
}

sub set_language : Private {
    my ( $self, $c ) = @_;

    # If ?lang= is set, use that as the language list
    # Otherwise use user setting
    # Otherwise C::P::I18N will use browser settings
    # Note: Need to check for $c->user because user_exists may pass
    # where the user info is saved in the session cookie, but the user does
    # not exist in the db
    if ( $_ = $c->req->param('lang')
        || ( $c->user_exists && $c->user && ( $_ = $c->user->language ) ) )
    {
        my $lang = convert_to_arrayref($_);
        $c->languages($lang);
    }
    if ( $c->debug ) {
        my $languages = $c->languages;
        $c->log->debug( "Languages setting: " . dump($languages) );
    }

    # Set date formatter too. At moment, only have one, but will plan to have more in future
    my $date_formatter = DateTime::Format::Strptime->new( pattern => "%F %T", time_zone => "local" );
    $c->stash(
        dt_formatter => sub {
            my $dt = shift;
            $dt->set_formatter($date_formatter);
            $dt->set_time_zone("local");
            "$dt";
        }
    );
}

sub authenticate : Private {
    my ( $self, $c ) = @_;

    my $need_login = 1;

    if ( $c->forward('/validate_ticket') ) {
        $need_login = 0;
    }

    if ($need_login) {
        $c->forward('login_page');
    }

    return !$need_login;
}

sub catch_all : Path {    # handle everything
    my ( $self, $c ) = @_;
    $c->res->redirect( $c->uri_for( $self->start_url ) );
}

# This redirects /nagios/* to /*. NOTE: it loses POST data, so
# it is best to avoid this altogether
sub nagios : Local {
    my ( $self, $c, @args ) = @_;
    $c->res->redirect( $c->uri_for( '/', @args, $c->req->query_params ) );
}

# Use this when a user hits an old URL, so we can display a message to say the URL should be changed
# Call with: $c->forward("/old_url_schema", [$c->uri_for("/status/hostgroup")])
sub old_url_schema : Private {
    my ( $self, $c, $uri ) = @_;
    $c->res->redirect($uri);
    $c->detach();
}

sub login : Local {
    my ( $self, $c ) = @_;
    if ( $c->req->path eq "login" && ( $c->req->method eq "POST" || $c->req->params->{one_time_token} ) ) {

        # Authenticate
        $c->forward("/authticket");

    }
    else {

        # If login_error is present, then an external app has put some information
        # re why the login failed. So for Opsview, we need to stop at this page, rather
        # than go to a catch all page
        if ( $c->req->param("login_error") ) {
            $c->stash( login_error => $c->req->param("login_error") );
            $c->forward('/login_page');
        }
        else {
            if ( $c->forward("/authenticate") ) {

                # If an external app redirects here and authentication passes in Opsview, then that means the
                # ticket is failing from the other app. This could be due to a bad shared secret key - check the external app
                if ( $c->req->param("app") ) {
                    $c->stash( login_error => $c->loc( "messages.authentication.invalidTicketFromOtherApp [_1]", $c->req->param("app") ) );
                    $c->forward('/login_page');
                }
                else {
                    $c->forward('catch_all');    # If already logged in, redirect to start
                }
            }
        }
    }
}

sub login_page : Private {
    my ( $self, $c ) = @_;
    $c->stash( template           => 'login' );
    $c->stash( ignore_footer_logo => 1 );

    if ( $c->req->params->{output} && $c->req->params->{output} eq "json" ) {
        $c->res->status(401);
    }

    # This does not include the referrer
    $c->stash->{back} = $c->req->params->{'back'};
    $c->stash( wrapper => "wrappers/blank" );
}

sub validate_ticket : Private {
    my ( $self, $c ) = @_;

    if ( $c->debug ) {
        my $h = $c->req->cookies;
        foreach my $n ( keys %$h ) {
            $c->log->debug( "Request cookies: " . $n . "=" . $h->{$n} );
        }
    }

    if ( not defined $c->request->cookies->{'auth_tkt'} ) {
        $c->stash( login_error => $c->loc("messages.authentication.noTicketFound") );

        # This error is normal, so do not show on login page. Need error for /rest/login_tkt
        $c->stash( do_not_show_login_error => 1 );
        $c->log->debug('No auth_tkt found');
        return 0;
    }

    $c->log->debug('auth_tkt ticket found');

    my $tkt = Apache::AuthTkt->new( secret => Opsview::Config->authtkt_shared_secret() );

    my $request_ip = $self->authtkt_ignoreip ? "0.0.0.0" : $c->req->address;
    my $info = $tkt->valid_ticket( $c->request->cookies->{'auth_tkt'}->value(), ip_addr => $request_ip );

    if ( not defined $info ) {
        $c->stash( login_error => $c->loc('messages.authentication.invalidTicket') );
        my $audit_text = $c->translate( "messages.auditlog.invalidTicketFromIp [_1]", $c->req->address );
        $c->log->info($audit_text);
        Opsview::Auditlog->system($audit_text);
        $c->forward("/expire_ticket");
        return 0;
    }

    # Calculate if the ticket has expired
    if ( ( $info->{'ts'} + $c->config->{'session'}->{'expires'} ) < time ) {
        $c->stash( login_error => $c->loc('messages.authentication.expiredTicket') );
        $c->log->info('auth_tkt ticket expired');
        $c->forward("/expire_ticket");
        return 0;
    }

    if ( my $callback = $c->stash->{validate_ticket_extra_checks} ) {
        unless ( $callback->( $c, $info ) ) {
            return 0;
        }
    }

    # Create user
    # If a session cookie is found, will restore the user from the session cookie
    # This also will check that the user exists in the database
    if ( !$c->user ) {
        $c->log->debug("Going to retrieve info for $info->{'uid'}");
        my $uobj = $c->find_user( { name => $info->{'uid'} }, "local" );
        my $authtkt_default_username = "";
        if ( !defined $uobj ) {
            $authtkt_default_username = $c->config->{authtkt_default_username};
            if ( !$authtkt_default_username ) {
                $c->stash( login_error => $c->loc('messages.authentication.userDoesNotExist') );
                my $audit_text = $c->translate( "messages.auditlog.userNotFoundInDB [_1]", $info->{uid} );
                $c->log->info($audit_text);
                Opsview::Auditlog->system($audit_text);
                $c->forward("/expire_ticket");
                return 0;
            }
            $uobj = $c->find_user( { name => $authtkt_default_username }, "local" );
            if ( !defined $uobj ) {
                $c->stash( login_error => $c->loc('messages.authentication.userDoesNotExist') );
                my $audit_text = $c->translate( "messages.auditlog.userNotFoundInDBAndAuthtktDefault [_1] [_2]", $info->{uid}, $authtkt_default_username );
                $c->log->error($audit_text);
                Opsview::Auditlog->system($audit_text);
                $c->forward("/expire_ticket");
                return 0;
            }
        }
        $c->set_authenticated($uobj);
        my $audit_text;
        if ($authtkt_default_username) {
            $audit_text = $c->translate( "messages.auditlog.loggedInAs [_1] [_2]", $info->{uid}, $authtkt_default_username );
        }
        else {
            $audit_text = $c->translate( "messages.auditlog.loggedIn [_1]", $info->{uid} );
        }
        $c->log->info($audit_text);
        Opsview::Auditlog->system($audit_text);
    }

    # User is valid - renew ticket if necessary
    if ( ( $info->{'ts'} + $c->config->{'session'}->{'renew_ticket_after'} ) <= time ) {
        $c->log->debug('auth_tkt ticket renewed');
        $c->forward('/extendticket');
    }

    $c->log->debug( 'User "', $c->user->username(), '" ticket validated' ) if $c->debug;

    return 1;
}

sub extendticket : Private {
    my ( $self, $c ) = @_;

    my $tkt = Apache::AuthTkt->new( secret => Opsview::Config->authtkt_shared_secret() );

    my $token;
    $token = "OPSVIEW_ADMIN" if ( $c->user->has_access("ADMINACCESS") );

    my $request_ip = $self->authtkt_ignoreip ? "0.0.0.0" : $c->req->address;
    $c->response->cookies->{'auth_tkt'} = {
        'value' => $tkt->ticket(
            uid     => $c->user->username,
            ip_addr => $request_ip,
            tokens  => $token,
        ),
        'path' => '/'
    };
    $c->log->debug( 'auth_tkt ticket extended to ' . $c->user->name );
}

# Need this because want to check if user already exists via session cookie
# API does not interrogate the authticket
sub login_api : Private {
    my ( $self, $c ) = @_;
    if ( $c->user_exists ) {
        return 1;
    }
    $c->stash( login_via => "API" );
    return $c->forward("/login_via_cat_plugins");
}

# This does the actual call to Catalyst::Authentication::* plugins to login
# This is called from /authticket (via the login screen)
# or from /login_api (which bypasses the authentication ticket)
# Returns 0 if not allowed (with audit text)
# or 1 if okay
sub login_via_cat_plugins : Private {
    my ( $self, $c ) = @_;

    # Have to do this in two steps because otherwise if login_username is
    # specified twice, get an array ref back instead of a scalar
    my $username = $c->stash->{".login_username"} || $c->req->param("login_username");
    delete $c->req->params->{login_username};
    my $password = $c->stash->{".login_password"} || $c->req->param("login_password");
    delete $c->req->params->{login_password};

    my $logged_in = 0;
    my $exception;
    if ( !defined $username ) {
        my $audit_text = $c->translate( "messages.auditlog.noUsernameFromIp [_1]", $c->req->address );
        $c->log->warn($audit_text);
        Opsview::Auditlog->system($audit_text);
    }
    else {
        my $login_via = $c->stash->{login_via} || "web";

        my $uobj = $c->find_user( { name => $username }, "local" );

        if ( my $one_time_token = $c->req->params->{one_time_token} ) {
            if ( my $api_session = $c->model("OpsviewSchema::ApiSessions")->search( { token => $one_time_token, username => $username, ip => $c->req->address, one_time_token => 1 } )->first ) {
                if ( $api_session->expires_at >= time() ) {

                    my $audit_text = $c->translate( "messages.auditlog.successfulLoginViaOneTimeToken [_1] [_2] [_3]", $username, $c->req->address, $login_via );
                    $c->log->info($audit_text);
                    Opsview::Auditlog->system($audit_text);

                    # Delete token so cannot be used again
                    $api_session->delete;

                    $c->set_authenticated( $uobj, "local" );
                    $logged_in = 1;
                }
                else {
                    $c->log->warn("One Time Token expired");
                }
            }
            else {
                $c->log->warn("Token not found");
            }
        }
        else {

            # Need an eval around here because $c->authenticate could throw an exception
            # if, eg, ldap server not available
            eval {
                if ( $uobj && $c->authenticate( { name => $username, password => $password }, $uobj->realm ) )
                {

                    # This part is required so that a user authenticated from a different store
                    # switches to the user defined in local Opsview database
                    if ( $c->user->auth_realm ne "local" ) {
                        $c->set_authenticated( $uobj, "local" );
                    }

                    $logged_in = 1;
                    my $audit_text = $c->translate( "messages.auditlog.successfulLoginFromVia [_1] [_2] [_3]", $username, $c->req->address, $login_via );
                    $c->log->info($audit_text);

                    # This should be a SYSTEM entry, as it relates to the system instead of the user
                    # This also means that we can count the number of user changes for a reload
                    Opsview::Auditlog->system($audit_text);

                }
            };
            $exception = $@;
        }

        if ( !$logged_in ) {
            my $login_error;
            my $audit_text = $c->translate( "messages.auditlog.unsuccessfulLoginFromVia [_1] [_2] [_3]", $username, $c->req->address, $login_via );
            if ($exception) {
                $login_error = $c->loc("messages.authentication.exceptionError");
                $audit_text .= " ($exception)";
            }
            else {
                $login_error = $c->loc("messages.authentication.invalid");
            }
            $c->stash( login_error => $login_error );
            $c->log->warn($audit_text);
            Opsview::Auditlog->system($audit_text);
        }
    }
    return $logged_in;
}

sub authticket : Private {
    my ( $self, $c ) = @_;

    # Check that JavaScript is enabled
    if ( $c->req->param('noscript') ) {
        $c->stash( login_error => 'JavaScript is disabled.' );
        $c->forward('/login_page');
    }
    else {

        # Attempt login
        if ( $c->forward("/login_via_cat_plugins") ) {

            # Record the use of this user agent
            $c->model("OpsviewSchema::Useragents")->update_or_create(
                {   id          => $c->request->user_agent,
                    last_update => DateTime->now()
                }
            );

            $c->forward('/extendticket');

            # Show user the welcome page, after first login
            my $url = $c->req->params->{'back'} || $c->request->referer();
            if ( !$url || $url =~ qr%/login($|\?)% || $url eq "/" ) {
                $url = $c->uri_for("/");
            }
            if ( $self->always_show_welcome_page || $c->user->show_welcome_page ) {
                $c->user->show_welcome_page(0);
                $c->user->update();
                $c->stash( continue_url => $url );
                $c->forward('/welcome');
            }
            else {
                $c->res->redirect($url);
            }
        }
        else {
            $c->forward('/login_page');
        }
    }
}

sub logout : Local {
    my ( $self, $c ) = @_;

    $c->forward("/expire_ticket");

    if ( $c->user_exists && $c->user ) {
        my $audit_text = $c->translate( "messages.auditlog.loggedOutFrom [_1] [_2]", $c->user->username, $c->req->address );
        $c->log->info($audit_text);
        Opsview::Auditlog->system($audit_text);
    }
    $c->logout;
    $c->forward('catch_all');
}

sub expire_ticket : Private {
    my ( $self, $c ) = @_;
    $c->response->cookies->{'auth_tkt'} = { 'value' => "expired", path => '/', expires => "-1d" };
}

sub refresh_page : Private {
    my ( $self, $c ) = @_;
    $c->stash( refresh_rate => Opsview::Systempreference->refresh_rate ) unless exists $c->stash->{refresh_rate};
    $c->stash( refresh_link => $c->uri_for_params( "", { output => "ajax" } ) );
}

sub soft_error : Private {
    my ( $self, $c ) = @_;
    $c->stash( template => "soft_error" );
    $c->stash->{error} = convert_to_arrayref( $c->stash->{error} );
}

sub end : ActionClass('RenderView') {
    my ( $self, $c ) = @_;
    if ( my @errors = @{ $c->error } ) {

        # Catch specific errors
        if ( $errors[0] =~ /Tried to set invalid session ID/ ) {
            $c->clear_errors;
            $c->stash( error => ["Invalid session ID"] );

            # We can't use $c->forward within the end process, so do a normal perl method call
            $self->soft_error($c);

            # Choose 400 because this is some client error
            $c->res->status(400);
        }
        else {
            $c->log->error( "Errors encountered: " . join( ', ', @errors ) );
            Opsview::Auditlog->system( "Errors encountered: " . join( ', ', @errors ) );

            # Force refresh_rate to 0 so no page refreshing
            $c->stash(
                errors       => \@errors,
                refresh_rate => 0,
                template     => 'error'
            );
            $c->stash( dokuwiki_link => Utils::Weberrors->lookup_errors( \@errors ) );
            $c->res->status(500);
            $c->clear_errors;
        }
    }

    if ( $c->debug ) {
        use Data::Dump qw(dump);
        my $h = $c->res->cookies;
        foreach my $n ( keys %$h ) {
            $c->log->debug( "Response cookies: " . $n . "=" . dump( $h->{$n} ) );
        }
    }

    # This is required to stop IE caching pages
    # Also makes Back button on browsers cause a reload of the page (except Safari - cannot force that to load new data)
    # See http://www.dev411.com/blog/2006/07/25/catalyst-customizing-the-view-stop-ie-from-caching-json for details
    # However, you don't want this for pages that have forms on it, as Back will lose all data
    unless ( $c->stash->{disable_cache} ) {
        $c->res->header( 'Cache-Control' => 'no-store, no-cache, must-revalidate, max-age=0, post-check=0, pre-check=0' );
        $c->res->header( 'Pragma'        => 'no-cache' );
        $c->res->header( 'Expires'       => 'Thu, 01 Jan 1970 00:00:00 GMT' );
    }

    # This is required for IE7/8 to allow cookies to be returned in the response when in an iframe
    # See http://adamyoung.net/IE-Blocking-iFrame-Cookies
    $c->res->header( 'P3P' => 'CP="IDC DSP COR ADM DEVi TAIi PSA PSD IVAi IVDi CONi HIS OUR IND CNT"' );

    # Set last server update time if json or ajax. This updates the status status bar text
    my $output = defined $c->stash->{output} ? $c->stash->{output} : ( $c->req->param("output") ? $c->req->param("output") : "" );

    if ( $output eq "ajax" || $output eq "json" ) {
        my $server_status = $c->model("RuntimeSchema::NagiosProgramstatus")->first;
        if ($server_status) {
            $c->res->header( "X-Opsview-Server-Status" => $c->stash->{dt_formatter}->( $server_status->status_update_time ) );
        }
    }

    # If json output, stop here
    if ( $output eq "json" ) {
        $c->forward("View::JSON");
    }
    elsif ( $output eq "csv" ) {
        $c->forward("View::Download::CSV");
    }
    elsif ( $output eq "excel" ) {
        $c->forward("View::Download::Excel");
    }
    elsif ( $output eq "xml" ) {
        $c->forward("View::Download::XML");
    }

    if ( $c->stash->{tabs} ) {
        push( @{ $c->stash->{extra_javascript} }, "fabtabulous" );
        $c->stash( edit_form_fields_template => "tab-wrapper" );
    }

    # Need to check $c->user too because user might be missing from db
    if ( $c->user_exists && $c->user ) {
        $c->stash( user_exists => 1 );
    }

    # Do sidenav stuff, unless ajax involved
    # If ajax request brings up login page, then "Timeout" output will trigger javascript to redirect to current url
    if ( $output && $output eq "ajax" ) {
        if ( $c->stash->{template} && $c->stash->{template} eq "login" ) {
            $c->res->body("Timeout");
        }
    }
    elsif ( ( $_ = $c->req->headers->{"x-requested-with"} ) && $_ eq "XMLHttpRequest" && ( $_ = $c->stash->{template} ) && $_ eq "login" ) {
        $c->res->body("AJAX error");
    }
    else {
        if ( $c->res->body ) {
            $c->stash( wrapper => "wrappers/blank" );
        }
        elsif ( !( $c->stash->{wrapper} ) ) {
            if ( !$c->stash->{ignore_sidenav} ) {
                $c->forward('/sidenav/setup_data');
            }
        }
    }

    if ( $c->stash->{debug} ) {
        my $tree   = $c->stats();
        my $dvisit = new Tree::Simple::Visitor();
        $tree->accept($dvisit);
        $c->stash->{'action_stats'} = $dvisit->getResults();
    }
}

# /rss is stored here.

use XML::RSS;

sub rss : Local {
    my ( $self, $c ) = @_;

    # No auth required for this page

    $c->res->content_type("application/rss+xml");

    my $rss = new XML::RSS( version => "2.0" );
    $rss->channel( title => $c->loc("ui.atom.title.rssDeprecated") );
    $rss->add_item(
        title       => "RSS Deprecated",
        link        => $c->uri_for("atom"),                                                 # REQ for firefox
        description => $c->loc( "ui.atom.help.rssDeprecated [_1]", $c->uri_for("atom") ),
    );
    $c->res->body( $rss->as_string );
}

# /atom for Atom feeds

use XML::Atom::SimpleFeed;
use Fcntl qw(:flock);
use POSIX qw(strftime);
use Storable qw(retrieve_fd store_fd);

# function to return time in a given format
sub w3cdtf {
    my $dt = DateTime->now(
        time_zone => 'local',
        formatter => DateTime::Format::W3CDTF->new,
    );

    # ensure stringified date returned
    return "$dt";
}

sub atom : Local {
    my ( $self, $c ) = @_;

    # Do not need to pass in user credentials as the HTTP credential module will pick off information from the headers
    # The realm in the hash is the Basic Auth realm as returned by the browser
    # TODO: allow cookies or LDAP to authenticate? See OPS-316
    if ( !$c->user_exists ) {

        # Set username and password, based on http auth info
        my ( $u, $p ) = $c->request->headers->authorization_basic;
        $c->stash( ".login_username" => $u ) if $u;
        $c->stash( ".login_password" => $p ) if $p;

        $c->stash( login_via => "RSS" );
        my $logged_in = $c->forward("/login_via_cat_plugins");

        # Return a basic auth request if login fails
        if ( !$logged_in ) {
            $c->authenticate( { realm => "Nagios Access" }, "http" );
        }
    }

    # We have to call again. This is because the user is not found in auto, until the above has been called
    $c->forward("/set_language");

    # Will only get here if user has authenticated
    my $server = $c->request->base;
    chomp($server);

    my $username = $c->user->username;

    my $uuid = $server . "=>" . $c->user->username;

    my $storefile = $c->config->{atom}->{dir} . "/$username.store";

    # Need to convert the $c->uri_for() into strings, becase on Sol10, XML::Atom::SimpleFeed crashes when
    # the regexp is run against an object. This looks like a perl bug (5.8.4)
    my $feed = XML::Atom::SimpleFeed->new(
        id        => $uuid,
        title     => $c->loc("ui.atom.notification.title"),
        link      => $c->uri_for("/")->as_string,
        author    => $c->config->{atom}->{name},
        generator => $c->config->{atom}->{name},
        icon      => $c->uri_for( $c->config->{atom}->{icon} )->as_string,
        logo      => $c->uri_for( $c->config->{atom}->{icon} )->as_string,
        updated   => w3cdtf(),
        subtitle  => '',
        category  => $c->config->{atom}->{name},
    );

    unless ( -r $storefile && -s $storefile ) {

        $feed->add_entry(
            id      => $uuid . '-1',
            title   => $c->loc("ui.atom.notification.title"),
            updated => w3cdtf(),
            link    => $c->uri_for("/")->as_string,
            content => $c->loc( "ui.atom.notification.noAlertsForUsername [_1]", $username ),
        );

        $c->res->body( $feed->as_string );

        return;
    }

    open( STORE, '<', $storefile );
    flock( STORE, LOCK_SH );
    my $entries = retrieve_fd(*STORE);
    close(STORE);

    foreach (@$entries) {
        delete( $_->{last_update} );
        delete( $_->{state} );
        delete( $_->{internal_id} );
        $_->{id} = 'server:' . $c->request->{headers}->{host} . ';' . $_->{id};
        $_->{author} ||= $c->config->{atom}->{author};
        $_->{link} = $c->uri_for( $c->config->{atom}->{extinfo} ) . $_->{link};
        $feed->add_entry(%$_);
    }
    $c->res->body( $feed->as_string );
}

sub about : Local {
    my ( $self, $c ) = @_;
    if ( $c->user && $c->user->has_access("ADMINACCESS") ) {
        my $num_services = $c->model("RuntimeSchema::OpsviewHostServices")->count;
        my $about_system = {
            uuid         => $c->model("OpsviewSchema::Systempreferences")->find(1)->uuid,
            num_hosts    => $c->model("OpsviewSchema::Hosts")->count(),
            num_services => $num_services,
            num_slaves   => $c->model("OpsviewSchema::Monitoringservers")->count() - 1,
        };
        $c->stash( about_system => $about_system );
    }
    $c->stash->{page_title} .= ' > ' . $c->loc("ui.about.title");
}

sub welcome : Private {
    my ( $self, $c ) = @_;
    $c->stash->{template} = "welcome";
    $c->stash->{page_title} .= ' > ' . $c->loc("ui.welcome.title");
}

sub nagvis : Local {
}

sub access_denied : Private {
    my ( $self, $c ) = @_;
    $c->stash( template => "access_denied" );
    my $username = ( $c->user_exists ? $c->user->username : "unknown" );
    Opsview::Auditlog->create( { username => $username, text => $c->translate( "messages.auditlog.accessDeniedToFrom [_1] [_2]", $c->req->uri, $c->req->address ) } );
    $c->res->status(403);
    $c->stash( "rest" => { message => "Access denied" } );
    $c->detach;
}

# This is to get an exception from the model translated up to the UI.
# With internationalisation, the error code will be the index to the translated information
sub translate_error : Private {
    my ( $self, $c, $error ) = @_;
    my ($errorcode) = ( $error =~ /(system\.messages\.[\w\.]+)/ );
    if ( $errorcode eq "system.messages.hostgroup.samenamesameparentclash" ) {
        $c->stash->{error} = "Cannot use same name with the same parent";
    }
    else {
        die $error;
    }
}

sub check_public_or_user_access : Private {
    my ( $self, $c, $access ) = @_;
    unless ( $c->public_role->has_access($access) ) {
        unless ( $c->forward("/authenticate") ) {
            $c->detach;
        }
        if ( !$c->user->has_access($access) ) {
            $c->detach("/access_denied");
            return 0;
        }
    }
    return 1;
}

# This is used for easyXDM's communication across domains
my $easyxdm_versions = {
    "2.4.9.102"  => "lowest",
    "2.4.15.118" => "highest",
};

sub restxdmxhr : Path('restxdmxhr.html') {
    my ( $self, $c ) = @_;
    my $version = $c->req->params->{version} || "";
    my %a = reverse %$easyxdm_versions;
    if ( $version eq "highest" ) {
        $version = $a{"highest"};
    }
    elsif ( !exists $easyxdm_versions->{$version} ) {
        $version = $a{"lowest"};
    }
    $c->stash( easyxdm_version => $version );
    $c->stash( template        => "easyxdm-$version/restxdmxhr.html" );
    $c->stash( wrapper         => "wrappers/blank" );
}

sub easyxdm_version : Private {
    my %a = reverse %$easyxdm_versions;
    return $a{highest};
}

sub testpages : Local {
}

1;
