#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Search;

use Moose;
use namespace::autoclean;

BEGIN { extends 'Catalyst::Controller::ActionRole' }

sub auto : Private {
    my ( $self, $c, $id ) = @_;

    return 0 unless $c->forward('/authenticate');

    $c->stash( additional_template_paths => [ $c->path_to( 'root', 'admin' )->stringify ] );

    return 1;
}

# We use this redirect for two reasons:
#  1) IE has a silly problem where getElementById will return elements with the same //name//, rather than id
#  2) This will allow us to do things like: "hg:reading" to go hg pages for Reading hostgroup in future
sub default : Path : Does(ACL) : AllowedRole(VIEWSOME) : AllowedRole(VIEWALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $search_value = $c->req->param("s");
    my ( $type, $term ) = $search_value =~ m/^(.*?) (.*)$/;
    if ( $type && $type eq "nd" ) {
        my $netdisco_url = $c->stash->{system_preference}->netdisco_url;
        unless ($netdisco_url) {
            $c->stash( error => $c->loc("ui.message.needNetdiscoUrl") );
            $c->detach("/soft_error");
        }
        $c->res->redirect("$netdisco_url/device.html?ip=$term");
    }
    elsif ( $type && $type eq "hg" ) {
        $c->res->redirect( $c->uri_for( "/status/hostgroup/name", $term ) );
    }
    else {
        $c->res->redirect( $c->uri_for( "/status/service", { host => $search_value } ) );
    }
}

# Sets a list of [ { name => display name, alias => display alias, type => short keyword for this type } ]
sub host : Local : Does(ACL) : AllowedRole(VIEWSOME) : AllowedRole(VIEWALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    my @hostlist;

    my $q = $c->req->params->{q};
    my ( $type, $term ) = $q =~ m/^(.*?) (.*)$/;
    $term = $q if !defined $term;

    unless ($type) {
        my $hostlist_ref = Runtime::Searches->list_hosts_by_contact( $c->user, $c->req->params );
        foreach my $host (@$hostlist_ref) {
            my $hash = { name => $host->{name} };
            if ( $host->{ip} ) {
                $hash->{alias} = $host->{ip};
            }
            push @hostlist, $hash;
        }
    }

    # Find host groups matching query
    if ( !defined $type || $type eq "hg" ) {
        my $rs = $c->model("RuntimeSchema::OpsviewHostgroups")->search( { name => { "-like" => "%$term%" } }, { order_by => "name" } );

        if ( $c->user->has_access("VIEWALL") ) {
        }
        elsif ( $c->user->has_access("VIEWSOME") ) {
            $rs = $rs->search(
                { "host_contacts.contactid" => $c->user->id, },
                {   join     => { hostgroup_hosts => "host_contacts" },
                    distinct => 1,
                }
            );
        }

        foreach my $hostgroup ( $rs->search ) {
            my $hash = { type => "hg", name => $hostgroup->name };
            push @hostlist, $hash;
        }
    }

    $c->stash( list    => \@hostlist );
    $c->stash( wrapper => "wrappers/blank" );
}

sub service : Local : Does(ACL) : AllowedRole(VIEWSOME) : AllowedRole(VIEWALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    $c->stash( wrapper => "wrappers/blank" );
    my $params = { %{ $c->req->params } };
    $params->{contact} = $c->user;
    $params->{type}    = "service";
    $c->stash( list     => Runtime::Searches->list_rrd_objects($params) );
    $c->stash( template => "search/host" );
}

use Opsview::Utils qw(convert_to_arrayref);
my $type_to_col = {
    "host"    => "hostname",
    "service" => "servicename",
    "metric"  => "metricname",
};

sub performancemetric : Local : Does(ACL) : AllowedRole(VIEWSOME) : AllowedRole(VIEWALL) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $params  = $c->req->params;
    my $host    = convert_to_arrayref( $params->{host} );
    my $service = convert_to_arrayref( $params->{service} );
    my $metric  = convert_to_arrayref( $params->{metric} );
    my $type    = $params->{type};

    if ( !$type ) {
        $c->stash( error => "No type defined" );
        $c->detach("/soft_error");
    }

    my $coltype = $type_to_col->{$type};

    if ( !$coltype ) {
        $c->stash( error => "Bad type: $type" );
        $c->detach("/soft_error");
    }

    my $search = {};
    $search->{hostname}    = { "-like" => $host }    if @$host;
    $search->{servicename} = { "-like" => $service } if @$service;
    $search->{metricname}  = { "-like" => $metric }  if @$metric;
    my $rs = $c->model("RuntimeSchema::OpsviewPerformancemetrics")->search( $search, { select => $coltype, distinct => 1 } );

    # Add permissions
    if ( $c->user->has_access("VIEWSOME") ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }

    my @list = $rs->get_column($coltype)->all;

    $c->stash( ResultSet => { list => \@list, count => scalar @list } );
    $c->stash( wrapper => "wrappers/blank" );
}

1;
