#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::State::Host;
use strict;
use warnings;
use MRO::Compat;

use Opsview::Utils qw(convert_state_to_text);
use Opsview::Utils::QueryHost;

use base qw(
    Opsview::Web::ControllerBase::Menus
    Opsview::Web::ControllerBase::Chained
    Opsview::Web::ControllerBase::Detail
    Opsview::Web::ControllerBase::Downtime
);

__PACKAGE__->config( model_name => "RuntimeDB::Host", );

sub anchor : PathPart('state/host') Chained('/') CaptureArgs(0) {
    shift->common(shift);
}

sub do_detail : Private {
    my ( $self, $c ) = @_;

    my $object = $c->stash->{object};
    $c->stash( title_description => $c->loc( "ui.state.host.title.summary [_1]", $object->name ) );

    # my @downtimes = $object->list_all_host_downtime_by_comment;

    $c->stash( downtime_params => { "hostid" => $object->id, rows => "all" } );
    $c->forward("/rest/downtime/downtime_GET");
    my $downtimes = $c->stash->{rest}->{list};

    $c->stash( downtimes => $downtimes );
    $self->next::method($c);
}

sub delete : Private {
    my ( $self, $c ) = @_;
    $c->stash->{hosts} = [ $c->model("RuntimeSchema::OpsviewHosts")->find( $c->stash->{object}->id ) ];
    $c->forward("/rest/downtime/downtime_DELETE");
    $c->res->redirect( $c->uri_for( "/status/service", { host => $c->stash->{object}->name } ) );
    $c->flash( message => $c->loc( "ui.downtime.help.downtimeDeletedFor [_1]", $c->stash->{object}->name ) );
}

sub queryhost_by_name_end : PathPart('queryhost') Chained('by_name') : Args(0) {
    my ( $self, $c ) = @_;
    $c->stash( template => "queryhost" );
    $self->queryhost($c);
}

sub queryhost : Chained('by_id') Args(0) {
    my ( $self, $c ) = @_;
    my $runtime_object = $c->stash->{object};
    my $config_object  = $runtime_object->configuration_host_object;

    unless ( defined $config_object ) {
        $c->stash( error => $c->loc("ui.message.hostNotInConfiguration") );
        $c->detach("/soft_error");
    }

    my $live_query_host = Opsview::Utils::QueryHost->run(
        {   hostaddress  => $config_object->ip,
            hostobject   => $config_object,
            snmp_version => $config_object->snmp_version,
            snmp_port    => $config_object->snmp_port,
            snmp_auth    => {
                community    => $config_object->snmp_community,
                username     => $config_object->snmpv3_username,
                authprotocol => $config_object->snmpv3_authprotocol,
                authpassword => $config_object->snmpv3_authpassword,
                privprotocol => $config_object->snmpv3_privprotocol,
                privpassword => $config_object->snmpv3_privpassword,
            },
            monitoringserver => $config_object->monitored_by,
            ignore_default   => 1,
        }
    );

    # Find all runtime services on this host that we want to graph
    my $rs = $c->model("RuntimeSchema::OpsviewHostServices")->search(
        {   host_object_id => $runtime_object->id,
            servicename    => { "-like" => [ "Interface: %", "Errors: %", "Discards: %" ] },
        },
        { prefetch => "servicestatus", }
    );

    # Add authentication
    if ( $c->user->has_access("VIEWSOME") ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }

    # Create lookup hash based on short interface name
    my $nagios_status_data_for_interface = {};
    while ( my $row = $rs->next ) {
        my $interfacename = $row->servicename;

        # TODO: I think this is the short interface name - not sure how to ensure these match up correctly
        $interfacename =~ s/^(.*?): //;
        my $type = $1;
        if ( $type eq "Interface" ) {
            $nagios_status_data_for_interface->{$interfacename}->{throughput_status}  = $row->servicestatus->current_state;
            $nagios_status_data_for_interface->{$interfacename}->{output}             = $row->servicestatus->output;
            $nagios_status_data_for_interface->{$interfacename}->{host}               = $row->hostname;
            $nagios_status_data_for_interface->{$interfacename}->{service}            = $row->servicename;
            $nagios_status_data_for_interface->{$interfacename}->{perfdata_available} = $row->perfdata_available;
        }
        elsif ( $type eq "Errors" || $type eq "Discards" ) {
            if ( $row->perfdata_available ) {
                push @{ $nagios_status_data_for_interface->{$interfacename}->{errors_discards_services_list} }, $row->servicename;
            }
        }
    }

    # Match name with interface row and merge state and available perf data
    my $interfaces = $live_query_host->{interfaces};
    unless ($interfaces) {
        foreach my $i ( $config_object->snmpinterfaces( { interfacename => { "!=" => "" } } ) ) {
            push @$interfaces,
                {
                id => $i->indexid || "-",
                interfacename      => $i->interfacename,
                shortinterfacename => $i->shortinterfacename,
                };
        }
    }

    foreach my $i (@$interfaces) {
        my $this_name = $i->{shortinterfacename} || $i->{interfacename};
        if ( my $d = $nagios_status_data_for_interface->{$this_name} ) {
            $i->{throughput_status}             = uc( convert_state_to_text( $d->{throughput_status} ) );
            $i->{output}                        = $d->{output};
            $i->{perfdata_available}            = $d->{perfdata_available};
            $i->{host}                          = $d->{host};
            $i->{service}                       = $d->{service};
            $i->{errors_discards_services_list} = $d->{errors_discards_services_list};
            $i->{graph_throughput}              = $d->{graph_throughput};
            $i->{graph_throughput_link}         = $d->{graph_throughput_link};
            $i->{graph_errors_discards}         = $d->{graph_errors_discards};
            $i->{graph_errors_discards_link}    = $d->{graph_errors_discards_link};
        }
    }
    $c->stash->{interfaces} = $interfaces;

    $c->stash->{nagios_status_data_for_interface} = $nagios_status_data_for_interface;
    $c->stash->{live_query_host}                  = $live_query_host;

    if ( my $netdisco_url = $c->model("OpsviewSchema::Systempreferences")->find(1)->netdisco_url ) {
        $c->stash->{netdisco_link} = "$netdisco_url/device.html?submit=Show+All+Ports&ip=" . $config_object->ip;
    }

    $c->stash( page_title => " > " . $c->loc("ui.objecttype.host") . ": " . $runtime_object->name . " > " . $c->loc("ui.state.host.queryhost.title.hostInterfaces") );
    push( @{ $c->stash->{stylesheets} }, 'dataTables.css' );
    push( @{ $c->stash->{extra_javascript} }, "jquery", "jquery.dataTables", "jquery.noConflict" );

    $c->stash( page_view_include => "view_options_links" );
    push @{ $c->stash->{page_view_options_links} },
        {
        link => $c->uri_for( "/status/service", { host => $c->stash->{object}->name } )->path_query,
        name => $c->loc("ui.menu.viewHostStatus")
        },
        {
        link => $c->uri_for( "/admin/host/edit",          $config_object->id )->path_query,
        name => $c->loc( "ui.admin.list.label.edit [_1]", $c->stash->{object}->name ),
        },
        {
        link => $c->uri_for( "/admin/host/interfaces", $config_object->id )->path_query,
        name => $c->loc("ui.admin.host.menu.editHostInterfaces")
        },
        ;

}

1;
