#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Status::Service;

use warnings;
use strict;
use base qw/Catalyst::Controller/;

sub auto : Private {
    my ( $self, $c ) = @_;
    $c->forward("/rest/status/service/apply_restrictions");
}

sub default : Path {
    my ( $self, $c ) = @_;

    my $heading_list = $c->stash->{heading_list} = [];
    push @$heading_list, $c->loc("ui.status.service.label.summary");

    my $params = { %{ $c->req->params } };
    $c->stash( status_params => $params );
    $c->forward("/rest/status/service/service_GET");
    $c->stash( status_data => $c->stash->{rest} );

    # Make a best guess for the page heading
    if ( ( exists $c->req->params->{downtime_start_time} ) && ( exists $c->req->params->{downtime_comment} ) ) {
        my $downtime_start_time = $c->req->param("downtime_start_time");
        my $downtime_comment    = $c->req->param("downtime_comment");

        # This downtime needs converting to local time
        my $dt = DateTime::Format::MySQL->parse_datetime($downtime_start_time);
        $dt->set_time_zone("UTC")->set_time_zone("local");
        my $downtime_output = $dt->ymd . " " . $dt->hms;
        push @$heading_list, $c->loc( "ui.status.service.label.summary.forDowntime [_1]", $downtime_output );
        $c->stash( subheading => $downtime_comment );
    }
    elsif ( $_ = $c->req->param("hostgroupid") ) {
        my $hg = $c->model("RuntimeSchema::OpsviewHostgroups")->search( { id => $_ } )->first;
        if ($hg) {
            $c->stash( hostgroup => $hg );
            $c->stash( breadcrumbs => $c->forward( "/status/hostgroup/breadcrumbs", [$hg] ) );
            push @$heading_list, $c->loc("ui.objecttype.hostgroup") . ": " . $hg->name;
        }
    }
    elsif ( $_ = $c->req->param("keyword") ) {
        push @$heading_list, $c->loc("ui.objecttype.keyword") . ": " . $_;
    }
    elsif ( $c->stash->{status_data}->{summary}->{host}->{total} == 1 ) {
        my $hostname    = $c->stash->{status_data}->{list}->[0]->{name};
        my $host        = $c->model("RuntimeSchema::OpsviewHosts")->find( { name => $hostname } );
        my $breadcrumbs = $c->forward( "/status/hostgroup/breadcrumbs", [ $host->hostgroup ] );
        push @$breadcrumbs, $c->forward( "/status/host/breadcrumb", [$host] );
        $c->stash( breadcrumbs => $breadcrumbs );
        push @$heading_list, $c->loc("ui.objecttype.host") . ": " . $hostname;
        $c->stash( only_one_host_found => 1 );
    }

    $c->stash( template => "service" );

    $c->stash->{functions}->{parseInterval} = \&Time::Interval::parseInterval;
}

sub acknowledge : Local : Does(ACL) : AllowedRole(ACTIONALL) : AllowedRole(ACTIONSOME) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    $self->sort_parameters($c);

    if ( $c->req->method eq "POST" ) {
        my $object_params = {};

        $c->forward("/rest/acknowledge/acknowledge_validate");

        if ( $_ = $c->stash->{validation_errors} ) {
            $c->stash->{error} = $_;
            $c->detach("/soft_error");
        }

        # Setup parameters as REST API expects
        foreach my $h ( $c->req->param("host_selection") ) {
            push @{ $object_params->{"hst.hostname"} }, $h;
        }
        foreach my $thing ( $c->req->param("service_selection") ) {
            my ( $h, $s ) = split( ";", $thing );
            push @{ $object_params->{"svc.hs"} }, $h . "::" . $s;
        }

        $c->stash( object_params => $object_params );

        $c->forward("/rest/acknowledge/find_objects");

        my $hosts_count    = scalar @{ $c->stash->{hosts} };
        my $services_count = scalar @{ $c->stash->{services} };

        if ( $hosts_count + $services_count == 0 ) {
            $c->flash( message => $c->loc("ui.acknowledgements.message.noItemsSelected") );
        }
        else {

            $c->forward("/rest/acknowledge/acknowledge_set");

            my @all_errors = @{ $c->stash->{all_errors} };
            if (@all_errors) {
                $c->stash->{error} = "Error setting acknowledgements: " . join( ", ", @all_errors );
                $c->detach("/soft_error");
            }
            else {
                $hosts_count    = scalar @{ $c->stash->{done_hosts} };
                $services_count = scalar @{ $c->stash->{done_services} };
                $c->log->info( "Mass acknowledgement: " . ( $hosts_count + $services_count ) );
                my $message = $c->loc( "ui.acknowledgements.message.submittedForHostsAndServices [_1] [_2]", $hosts_count, $services_count );
                $message .= "\n" . $c->loc("ui.acknowledgements.message.delay");
                $c->flash( message => $message );
            }
        }
        $c->res->redirect( $c->req->params->{from} || $c->req->referer );
    }
    else {

        $c->forward("/rest/acknowledge/acknowledge_GET");
        $c->stash( status_data => $c->stash->{rest} );

        my $from = $c->req->params->{from} || $c->req->referer;
        $c->stash( from => $from );
        my $heading_list = $c->stash->{heading_list} = [];
        push @$heading_list, $c->loc("ui.acknowledgements.title");
        $c->stash( template          => "acknowledge" );
        $c->stash( help_page_keyword => "acknowledgements" );
        $c->stash( refresh_rate      => 0 );
    }
}

sub sort_parameters {
    my ( $self, $c ) = @_;
    my $p = $c->req->params;
    delete $p->{output};

    # Need this little hack for pages from /status/hostgroup to come through correctly here
    if ( $_ = delete $p->{parentid} ) {
        $p->{hostgroupid} = $_;
    }

}

sub recheck : Local {
    my ( $self, $c ) = @_;

    $self->sort_parameters($c);

    my $ts = time;
    if ( $c->req->method eq 'POST' ) {
        my $object_params = {};

        # Setup parameters as REST API expects
        foreach my $h ( $c->req->param("host_selection") ) {
            push @{ $object_params->{"hst.hostname"} }, $h;
        }
        foreach my $thing ( $c->req->param("service_selection") ) {
            my ( $h, $s ) = split( ";", $thing );
            push @{ $object_params->{"svc.hs"} }, $h . "::" . $s;
        }

        $c->stash( object_params => $object_params );

        $c->forward("/rest/recheck/find_objects");

        my $hosts_count    = scalar @{ $c->stash->{hosts} };
        my $services_count = scalar @{ $c->stash->{services} };

        if ( $hosts_count + $services_count == 0 ) {
            $c->flash( message => $c->loc("ui.rechecks.message.noItemsSelected") );
        }
        else {

            $c->forward("/rest/recheck/recheck_do");

            my @all_errors = @{ $c->stash->{all_errors} };
            if (@all_errors) {
                $c->stash->{error} = "Error submitting rechecks:" . join( ", ", @all_errors );
                $c->detach("/soft_error");
            }
            else {
                $hosts_count    = scalar @{ $c->stash->{done_hosts} };
                $services_count = scalar @{ $c->stash->{done_services} };
                $c->log->info( "Mass recheck: " . ( $hosts_count + $services_count ) );
                my $message = $c->loc( "ui.rechecks.message.submittedForHostsAndServices [_1] [_2]", $hosts_count, $services_count );
                $message .= "\n" . $c->loc("ui.rechecks.message.delay");
                $c->flash( message => $message );
            }
        }
        $c->res->redirect( $c->req->params->{from} || $c->req->referer );
    }
    else {

        $c->forward("/rest/recheck/recheck_GET");
        $c->stash( status_data => $c->stash->{rest} );

        my $from = $c->req->params->{from} || $c->req->referer;
        $c->stash( from => $from );
        my $heading_list = $c->stash->{heading_list} = [];
        push @$heading_list, $c->loc("ui.rechecks.title");
        $c->stash( template          => "recheck" );
        $c->stash( help_page_keyword => "rechecks#massrecheck" );
        $c->stash( refresh_rate      => 0 );
    }

}

1;
