#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Controller::Viewport;

use strict;
use warnings;

use base qw/Catalyst::Controller/;

use Time::Interval;

sub auto : Private {
    my ( $self, $c, $id ) = @_;
    return $c->forward( "/check_public_or_user_access", ["VIEWPORTACCESS"] );
}

sub setup_keyword_rs {
    my ( $self, $c ) = @_;
    my $keyword_rs = $c->model("OpsviewSchema::Keywords")->search( { enabled => 1 } );

    # Apply keyword authentication
    if ( !$c->user_exists ) {
        $keyword_rs = $keyword_rs->search( { public => 1 } );
    }
    else {
        if ( $c->user->can_view_all || $c->user->role->all_keywords ) {
        }
        else {
            $keyword_rs->set_roleid( $c->user->role->id );
            $keyword_rs = $keyword_rs->search( { "-or" => [ { "keywordroles_or_public.roleid" => $c->user->role->id }, { "public" => 1 } ] }, { join => "keywordroles_or_public" } );
        }
    }

    $keyword_rs;
}

sub check : Private {
    my ( $self, $c ) = @_;
    my $name       = $c->req->params->{keyword};
    my $keyword_rs = $self->setup_keyword_rs($c);
    my $keyword    = $keyword_rs->search( { name => $name } )->first;
    unless ($keyword) {
        $c->stash( error => $c->loc( "messages.keyword.notFound [_1]", $name ) );
        $c->detach("/soft_error");
    }
    $c->stash( keyword => $keyword );
}

sub viewport_summary : Path : Args(0) {
    my ( $self, $c ) = @_;

    $c->forward("/rest/status/viewport/apply_restrictions");

    my $out = $c->req->param("output") || "";

    $c->stash(
        additional_template_paths => [
            $c->path_to( 'root', 'viewportsummary', 'style' )->stringify,    # Must come before viewportsummary to pick up most specific first
            $c->path_to( 'root', 'viewportsummary' )->stringify,
        ]
    );

    my $style;
    unless ( $style = $c->req->param("style") ) {
        $style = Opsview::Systempreference->viewport_summary_style;
    }
    $c->stash( style_type => "summary" );
    $c->forward( "/common/get_styles_list", [ "Viewport", "Summarystyle" ] );

    # Page related information
    unless ( exists $c->stash->{style_labels}->{$style} ) {
        $c->stash( error => $c->loc( "messages.viewportSummaryStyle.notFound [_1]", $style ) );
        $c->detach("/soft_error");
    }
    $c->stash( viewport_style => $style );

    if ( !$out ) {
        $self->set_view_options($c);
    }
    else {
        $c->stash( template => "viewport_wrapper" );
    }

    my $params = { %{ $c->req->params } };
    $c->stash( status_params => $params );
    $c->forward("/rest/status/viewport/viewport_GET");
    $c->stash( status_data  => $c->stash->{rest} );
    $c->stash( heading_list => [ $c->loc("ui.viewport.summary.title") ] );

    # Allow changing of templates or processing of data
    $c->forward("/viewport/summarystyle/$style/override");

}

sub set_view_options : Private {
    my ( $self, $c ) = @_;
    $c->stash( page_view_include => "view_options" );
    push( @{ $c->stash->{stylesheets} }, 'ajaxcommon.css' );
    push( @{ $c->stash->{extra_javascript} }, "jquery", "jquery.noConflict", "prototypextensions", "viewports" );

    my $keyword_rs             = $self->setup_keyword_rs($c);
    my @active_viewports       = $keyword_rs->search;
    my @keywords               = ();
    my %keywords_with_services = map { ( $_->keyword => 1 ) } (
        $c->model("RuntimeSchema::OpsviewViewports")->search(
            {},
            {   columns  => ["keyword"],
                distinct => 1,
            }
        )
    );
    foreach (@active_viewports) { push( @keywords, $_->name ) if $keywords_with_services{ $_->name }; }

    # info for all multiselects in filter options
    my $filter_columns = [
        {   "id"     => "keyword",
            "name"   => $c->loc("ui.event.label.keyword"),
            "values" => \@keywords
        }
    ];
    $c->stash->{filter_columns} = $filter_columns;

    if ( $c->user_exists ) {

        # If user exists, then filter by allowed, enabled keywords for that user
        my @keywords = map { $_->enabled && $keywords_with_services{ $_->name } ? $_->name : () } ( $c->user->valid_keywords );
        if (@keywords) {
            $c->req->params->{keyword} = \@keywords;
        }
    }
    $c->stash( template => "loading" );
}

sub status : Path : Args(1) {
    my ( $self, $c, $name ) = @_;
    my $out = $c->req->param("output") || "";

    $c->stash(
        additional_template_paths => [
            $c->path_to( 'root', 'viewport', 'style' )->stringify,    # Must come before viewport to pick up most specific first
            $c->path_to( 'root', 'viewport' )->stringify,
        ]
    );
    $c->req->params->{keyword}               = $name;
    $c->req->params->{includehandleddetails} = 1;
    $c->forward("/viewport/check");
    my $keyword = $c->stash->{keyword};
    unless ( $keyword->enabled ) {
        $c->stash( error => $c->loc( "messages.keyword.viewportNotEnabled [_1]", $name ) );
        $c->detach("/soft_error");
    }
    my $style;
    unless ( $style = ( scalar $c->req->body_params->{style} || $c->req->param("style") ) ) {
        $style = $keyword->style;
    }
    $c->forward( "/common/get_styles_list", ["Viewport"] );
    unless ( exists $c->stash->{style_labels}->{$style} ) {
        $c->stash( error => $c->loc( "messages.viewportStyle.notFound [_1]", $style ) );
        $c->detach("/soft_error");
    }
    $c->stash( viewport_style => $style );

    if ( !$out ) {
        $self->set_view_options($c);
    }
    else {
        $c->stash( template => "viewport_wrapper" );
    }

    my $description = $keyword->description;
    $description ||= $keyword->name;

    $c->stash( page_title => " > " . $c->loc("ui.viewport.title") . " > " . $description );
    $c->stash->{functions}->{parseInterval} = \&Time::Interval::parseInterval;

    # Allow changing of templates or processing of data
    my $custom = $c->forward_if_exists("/viewport/style/$style/custom");

    unless ($custom) {
        $c->forward( "/rest/status/viewport/viewport_one_GET", [ $keyword->name ] );
        $c->stash( status_data => $c->stash->{rest} );

        # Allow changing of templates or processing of data
        $c->forward("/viewport/style/$style/override");
    }

}

sub end : Private {
    my ( $self, $c ) = @_;
    my $out = $c->req->param("output") || "";
    if ( $out eq "ajax" ) {
        $c->stash( wrapper => "wrappers/blank" );
    }
    elsif ( $out eq "json" ) {

        # Have to create a new hash as status_data is a blessed object, which json barfs at
        $c->stash( ResultSet => { list => $c->stash->{status_data}->{list}, summary => $c->stash->{status_data}->{summary} } );
        $c->stash( output => "json" );
    }
    else {
        $c->forward("/refresh_page");
    }
    $c->detach("/end");
}

1;
