#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::ControllerBase::Admin;

use Moose;
use namespace::autoclean;

use Data::Page;
use Opsview::Common;
use Opsview::Utils;
use Opsview::Utils::Time;
use Data::Dump qw(dump);

BEGIN { extends 'Catalyst::Controller::ActionRole' }

__PACKAGE__->mk_accessors(
    qw/model_name set_many set_many_words set_commasep null_if_empty
        zero_if_empty may_create_new ignore_keys list_search_criteria
        draggables_list_method
        title_description
        title_new_string
        title_create_string
        set_many_on_off
        namespace
        search_column
        default_rows
        page_search_box
        jira_duration
        use_synchronise
        have_variables
        /
);

__PACKAGE__->config(
    set_many             => [],
    set_many_words       => [],
    set_many_on_off      => [],
    set_commasep         => [],
    null_if_empty        => [],
    zero_if_empty        => [],
    may_create_new       => [],
    ignore_keys          => [],
    list_search_criteria => {},
    jira_duration        => [],
);

sub auto : Private {
    my ( $self, $c, $id ) = @_;

    # This is not necessary as /admin protects the entire namespace
    # Need to remove because otherwise /configure/host cannot use this controller
    #  unless ($c->user_exists && (
    #    $c->user->has_access("ADMINACCESS") || $c->stash->{admin_override}
    #	)) {
    #    $c->stash(template => 'access_denied');
    #    return 0;
    #  }

    my $class      = ref($self);
    my $moniker    = ( split( '::', $class ) )[-1];    # last part of class name
    my $moniker_lc = lc $moniker;
    $c->stash( list_page => "admin/$moniker_lc" );
    my $model = $self->model_name;
    if ( $model =~ /OpsviewSchema/ ) {
        $c->stash( is_dbix_class => 1 );
    }
    else {
        $c->stash( is_dbix_class => 0 );
    }

    # Save this in the stash. This is because we might use DBIx::Class instead of Class::DBI
    # and saving in the stash allows you to switch between the two styles for some URLs within a class
    $c->stash( model_class => $c->model($model) );

    $moniker =~ s/([a-z])([A-Z])/$1 $2/g;    # "HostGroup" -> "Host Group" etc.
    my $translated_moniker = $c->loc("ui.objecttype.$moniker_lc");
    $c->stash( title_description => $translated_moniker ) unless $c->stash->{title_description};
    my $title_create_string = (
        exists $self->config->{title_create_string}
        ? $self->config->{title_create_string}
        : $c->loc( "ui.admin.list.createNew [_1]", $c->stash->{title_description} )
    );
    if ( $self->config->{page_add_new} ) {
        $self->set_page_view_options( $c, $title_create_string );
    }

    $c->stash( additional_template_paths => [ $c->path_to( 'root', $c->action->namespace )->stringify, $c->path_to( 'root', 'admin' )->stringify ] );

    # namespace used for contextual help
    if ( $_ = $self->namespace ) {
    }
    else {
        $_ = $c->action->namespace;
        s/.*\///;
    }
    $c->stash( namespace => $_ );

    push( @{ $c->stash->{stylesheets} }, 'admin.css' );

    my $from = $c->req->params->{from} || $c->req->referer;

    $c->stash( from   => $from );
    $c->stash( config => $self->config );

    if ( defined $id && $id =~ /^\d+$/ ) {    # Called with object id
        my $object;
        if ( $c->stash->{is_dbix_class} ) {
            $object = $c->stash->{model_class}->find($id);
        }
        else {
            $object = $c->stash->{model_class}->retrieve($id);
        }
        if ($object) {
            $c->stash( object => $object );
        }
        else {
            $c->detach('/catch_all');
        }
    }

    if ( $self->draggables_list_method ) {
        $c->stash( draggables_list_method => $self->draggables_list_method );
    }
    $c->stash( dragdrop_disabled => Opsview::Config->dragdrop_disabled );
    if ( Opsview::Config->nagios_interval_length_in_seconds ) {
        $c->stash( nagios_interval_units                => $c->loc("ui.admin.help.seconds") );
        $c->stash( nagios_interval_convert_from_minutes => 60 );
    }
    else {
        $c->stash( nagios_interval_units                => $c->loc("ui.admin.help.minutes") );
        $c->stash( nagios_interval_convert_from_minutes => 1 );
    }

    if ( my $snmptrapexceptionid = $c->req->param("snmptrapexception") ) {
        if ( my $trap = $c->model("OpsviewDB::Snmptrapexception")->retrieve($snmptrapexceptionid) ) {
            $c->stash( snmptrap => $trap );
        }
    }
    elsif ( my $trapid = $c->req->param("trap") ) {
        if ( my $trap = $c->model("OpsviewDB::Snmptrapdebug")->retrieve($trapid) ) {
            $c->stash( snmptrap => $trap );
        }
    }

    $c->stash( add_line_numbers => \&add_line_numbers );
    $c->stash( round            => \&round );

    my $ref;
    if ( $c->stash->{object} ) {
        $ref = $c->stash->{object};
    }
    else {
        $ref = $c->stash->{model_class};
    }
    if ( $c->stash->{title_description} ) {
        $c->stash->{page_title} .= ' > ' . $c->stash->{title_description};
    }
    elsif ( $ref->can('my_type_is') ) {
        $c->stash->{page_title} .= ' > ' . $translated_moniker;
    }

    $c->stash( page_search_box => $self->config->{page_search_box} || 0 );

    $c->stash( user_has_save_access => $c->user->has_access("CONFIGURESAVE") );

    return 1;
}

sub set_page_view_options {
    my ( $self, $c, $title_create_string ) = @_;
    $c->stash( page_view_include => "view_options_links" );
    $c->stash(
        page_view_options_links => [
            {   link => $c->uri_for_params("new")->path_query,
                name => $title_create_string,
            },
        ]
    );
}

sub round {
    my ( $number, $precision ) = @_;
    sprintf( "%0.${precision}f", $number );
}

sub add_line_numbers {
    my $data  = shift;
    my @lines = split( "\n", $data );
    my $c     = 0;
    my @new   = map { $c++; sprintf( "%2s: %s", $c, $_ ) } @lines;
    return join( "<br />", @new );
}

# Acts so that /admin/host is same as /admin/host/list
sub redirect_to_list : Path {
    my ( $self, $c ) = @_;

    if ( my $name = delete $c->req->params->{name} ) {
        my $it = $c->stash->{model_class}->search( name => $name );
        if ( $it->count == 1 ) {
            $c->res->redirect( $c->uri_for( "edit", $it->first->id ) );
        }
        elsif ( $it->count == 0 ) {
            $c->res->redirect( $c->uri_for( "list", { q => $name } ) );
        }
        else {
            $c->stash( error => "More than 1 $name found" );
            $c->forward("/soft_error");
        }
        $c->detach;
    }
    $c->forward("list");
}

sub list : Local : Args(0) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $params        = $c->req->params;
    my $req_page      = $params->{page} || 1;
    my $rows          = $params->{rows} || $self->default_rows || 200;
    my $search_column = $self->search_column || "name";
    if ( $req_page eq "all" ) {
        $req_page = 1;
        $rows     = 10000;
    }

    $c->stash->{page_title} .= ' > ' . $c->loc('ui.admin.title.list');

    my ( $page, $it );
    my $search    = $c->stash->{list_search_criteria} || {};
    my $page_attr = $c->stash->{page_attributes}      || {};
QUERY: {
        if ( exists $c->req->params->{q} ) {
            my $q = $c->req->param("q");
            last QUERY if ( $q eq "" );
            delete $c->req->params->{".btn"};
            my @search_array = split( " ", $q );
            my $array_ref = convert_to_arrayref($search_column);
            foreach my $colname (@$array_ref) {
                push @{ $search->{"-or"} }, { "me." . $colname => { 'like' => [ map { "%" . $_ . "%" } @search_array ] } };
            }
            $c->stash( search_array => \@search_array );
        }
    }
    if ( $c->stash->{is_dbix_class} ) {
        $it = $c->stash->{model_class}->search( $search, { page => $req_page, rows => $rows, %$page_attr } );
        $c->stash( count => $it->count );
        $c->stash( page  => $it->pager );
    }
    else {
        ( $page, $it ) = $c->stash->{model_class}->page( $search, { page => $req_page, rows => $rows, %$page_attr } );
        $c->stash( count => $page->total_entries );
        $c->stash( page  => $page );
    }
    $c->stash( data_iterator => $it );
    $c->stash( template      => 'list_layout' );
    $c->stash( page_link     => sub { my $page = shift; return create_page_link( $c, $page, $params ) } );
}

# See /usr/local/share/perl/5.8.4/Catalyst/Example/Controller/InstantCRUD.pm
# for more information about parameters. We assume no other parameters used
sub create_page_link {
    my ( $c, $page, $origparams ) = @_;
    my %params = %$origparams;
    $params{page} = $page;

    # Need eq because page could be "all"
    if ( $page eq $c->stash->{page}->current_page ) {
        return $page;
    }
    else {
        use HTML::Entities;
        my $page_text = ( $page eq 'all' ) ? $c->loc("ui.admin.list.pagesAll") : $page;
        return '<a href="' . decode_entities( $c->uri_for_params( "", { page => $page } )->path_query ) . '">' . $page_text . '</a>';
    }
}

sub delete : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    $c->forward("/admin/check_save_access");
    my $o = $c->stash->{object};
    my $audit_text = $c->translate( "messages.auditlog.admin.deletingObject [_1] [_2]", $o->class_title, $o->identity_string );
    $o->delete;
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $audit_text,
        }
    );
    $self->meta_redirect($c);
}

sub edit : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    # Need to set to blank because possible that there is no object (such as system preference page)
    my $object_name = "";
    if ( defined $c->stash->{page_title_override} ) {
        $object_name = $c->stash->{page_title_override};
    }
    elsif ( $c->stash->{object}->can('display_name') ) {
        $object_name = $c->stash->{object}->display_name;
    }
    elsif ( $c->stash->{object}->can('name') ) {
        $object_name = $c->stash->{object}->name;
    }

    my $action_description = "";
    if ( $object_name ne "" ) {
        $action_description = $c->loc("ui.admin.description.edit") . ": " . $object_name;
        $c->stash->{page_title} .= " > $action_description";
    }

    $c->stash( template      => 'edit_layout' );
    $c->stash( disable_cache => 1 );
    unless ( $c->stash->{message} ) {
        $c->stash( message => $action_description );
    }
    $c->stash( action_uri => $c->uri_for( 'save', $c->stash->{object}->id )->path );
    $c->stash( action => "edit" );
    $self->edit_common($c);
}

# Override in controllers for any custom stuff here
sub edit_common { }

sub create_new : Path('new') : Args(0) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    $c->stash->{page_title} .= " > " . $c->loc("ui.admin.title.new");
    $c->stash( template      => 'edit_layout' );
    $c->stash( disable_cache => 1 );
    $c->stash( message       => ( $c->stash->{title_new_string} || $c->loc( "ui.admin.description.new [_1]", $c->stash->{title_description} ) ) );
    $c->stash( action_uri    => $c->uri_for('save_new')->path );
    $c->stash( action        => "create" );
    $self->edit_common($c);
}

sub clone : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c, $cloned_from ) = @_;
    $c->stash( template      => 'edit_layout' );
    $c->stash( disable_cache => 1 );
    $c->stash->{page_title} .= " > " . $c->loc( "ui.admin.title.clone [_1]", $c->stash->{object}->name );
    my $message = $c->loc( "ui.admin.description.clone [_1] [_2]", $c->stash->{title_description}, $c->stash->{object}->name );
    $c->stash( message    => $message );
    $c->stash( action_uri => $c->uri_for('save_new')->path );
    $c->stash( clone      => 1 );
    $c->stash( action     => "clone" );                         # TODO: Need to remove references to clone in templates and use action instead
    $self->edit_common($c);
}

sub synchronise {
    my ( $self, $c ) = @_;
    my $attrs          = { %{ $self->use_synchronise } };
    my %set_commasep   = map { ( $_ => 1 ) } @{ $self->set_commasep };
    my %set_many_words = map { ( $_ => 1 ) } @{ $self->set_many_words };

    if ( $c->stash->{object} ) {
        $attrs->{id} = $c->stash->{object}->id;
    }
    my $params = $c->req->params;
PARAM: foreach my $p ( $c->req->param ) {
        next PARAM if $p =~ /^(?:action|type|from|server_ignore|\..*)$/;
        $attrs->{$p} = $params->{$p};
    }
    foreach my $commasep ( keys %set_commasep ) {
        if ( exists $attrs->{$commasep} ) {
            my $vals = convert_to_arrayref( $attrs->{$commasep} );
            $attrs->{$commasep} = join( ",", @$vals );
        }
    }
    foreach my $word ( keys %set_many_words ) {
        if ( exists $attrs->{$word} ) {
            my @v = map { { name => $_ } } ( split( /[ ,]/o, $attrs->{$word} ) );
            $attrs->{$word} = \@v;
        }
    }
    if ( $self->have_variables ) {

        # Convert variables[NAME]=VALUE
        # to variables => [ { name => NAME, value => VALUE } ]
        my @vars;
        foreach my $k ( keys %$attrs ) {
            if ( $k =~ /^variables\[(\w+)\]$/ ) {
                push @vars, { name => $1, value => delete $attrs->{$k} };
            }
        }
        $attrs->{variables} = \@vars;
    }
    $self->do_synchronise( $c, $attrs );
}

sub do_synchronise {
    my ( $self, $c, $attrs ) = @_;

    if ( $c->debug ) { warn( Data::Dump::dump($attrs) ) }
    $c->stash->{object} = $c->stash->{model_class}->synchronise( $attrs, { username => $c->user->username } );
}

sub save_new : Local : Args(0) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    $self->check_not_get($c);
    if ( $self->use_synchronise ) {
        if ( my $cloned_from = delete $c->req->params->{"cloned_from"} ) {
            if ( my $clone = $c->stash->{model_class}->find($cloned_from) ) {
                $c->stash( clone_object => $clone );
            }
        }
        $c->stash( object => $self->synchronise($c) );
        $self->meta_redirect($c);
    }
    elsif ( $c->stash->{is_dbix_class} ) {
        my $attrs  = {};
        my $params = $c->req->params;
    PARAM: foreach my $p ( $c->req->param ) {
            next PARAM if $p =~ /^(?:id|action|type|from|server_ignore|\..*)$/;
            $attrs->{$p} = $params->{$p};
        }
        if ( $c->stash->{model_class}->result_source->has_column('uncommitted') ) {
            $attrs->{uncommitted} = 1;
        }
        my $saved           = {%$attrs};
        my $o               = $c->stash->{model_class}->create($attrs);
        my $attributes_dump = Data::Dump::dump($saved);

        # This should probably be in the model, but I don't know how to get the user information down there
        Opsview::Auditlog->create(
            {   username => $c->user->username,
                text     => $c->translate( "messages.auditlog.admin.creatingObject [_1] [_2] [_3]", $o->class_title, $o->identity_string, $attributes_dump ),
            }
        );
        $c->stash( object => $o );
        $self->meta_redirect($c);
    }
    else {
        if ( my $cloned_from = delete $c->req->params->{"cloned_from"} ) {
            if ( my $clone = $c->stash->{model_class}->retrieve($cloned_from) ) {
                $c->stash( clone_object => $clone );
            }
        }
        $self->do_create_new($c);
        $c->forward('save');
    }
}

sub meta_redirect : Private {
    my ( $self, $c ) = @_;
    $c->forward('/admin/meta_redirect');
}

sub check_not_get : Private {
    my ( $self, $c ) = @_;
    if ( $c->req->method eq "GET" ) {
        $c->stash( error => "Not allowed to GET from this URL" );
        $c->detach("/soft_error");
    }
    $c->forward("/admin/check_save_access");
}

sub save : Local : Args(1) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;

    # This is run twice when clone is used, but that's okay for the moment
    $self->check_not_get($c);
    my $o = $c->stash->{object};
    if ( $self->use_synchronise ) {
        $self->synchronise($c);
    }
    elsif ( $c->stash->{is_dbix_class} ) {
        $self->do_dbix_update($c);
    }
    else {
        $self->do_update($c);
        $o->uncommitted(1) if $o->can('uncommitted');

        my $audit_fields = $c->stash->{audit_fields} || [];
        my $audit_lines = join( ", ", @$audit_fields );
        if ( exists $c->stash->{action} && $c->stash->{action} eq "create" ) {
            $_ = $c->translate( "messages.auditlog.admin.creatingObject [_1] [_2] [_3]", $o->class_title, $o->identity_string, $audit_lines );
        }
        else {
            $_ = $c->translate( "messages.auditlog.admin.updatingObject [_1] [_2] [_3]", $o->class_title, $o->identity_string, $audit_lines );
        }
        Opsview::Auditlog->create(
            {   username => $c->user->username,
                text     => $_,
            }
        );

        $o->update;
    }
    $self->meta_redirect($c);
}

sub do_create_new {
    my ( $self, $c ) = @_;
    my $audit_fields = [];

    my $model = $c->stash->{model_class};
    my %may_create_new = map { ( $_ => 1 ) } @{ $self->may_create_new };

    my %init;

    foreach my $col ( @{ $model->initial_columns } ) {

        if ( $may_create_new{$col} ) {    # handle category_new etc. on create

            if ( my $new = delete $c->req->params->{"${col}_new"} ) {

                my $class = $c->model('OpsviewDB')->convert_type_to_class($col);
                $init{$col} = $class->find_or_create( { name => $new } );
                push @$audit_fields, "$col='$new'";
            }
        }

        unless ( exists $init{$col} ) {
            if ( !defined $c->req->param($col) ) {
                die "${col} not set - cannot create";
            }
            else {
                $init{$col} = $c->req->param($col);
            }
            push @$audit_fields, "$col='" . $c->req->param($col) . "'";
        }

        $c->log->debug( "Create new: setting ${col}=" . $init{$col} ) if $c->debug;
        delete $c->req->params->{$col};    # don't update again later
    }

    $c->stash( object       => $model->create( \%init ) );
    $c->stash( audit_fields => $audit_fields );
    $c->stash( action       => "create" );
}

# Converts form parameters into a hash ref to update the object
sub do_dbix_update {
    my ( $self, $c ) = @_;
    my $o = $c->stash->{object};

    # For each column of object
    # Set if char
    # Set related if has_a or has_many
    my %zero_if_empty = map { ( $_ => 1 ) } @{ $self->zero_if_empty };
    my %set_many      = map { ( $_ => 1 ) } @{ $self->set_many };

    my $data = {};
PARAM: foreach my $p ( $c->req->param ) {
        next PARAM if $p =~ /^(?:id|action|type|from|server_ignore|\..*)$/;
        delete $zero_if_empty{$p};    # For later
        delete $set_many{$p};

        # This could be an arrayref
        my $val = $c->req->params->{$p};
        $data->{$p} = $val;
    }
    foreach my $zero ( keys %zero_if_empty ) {
        $data->{$zero} = 0;
    }
    foreach my $many ( keys %set_many ) {
        $data->{$many} = [];
    }
    if ( $o->can('uncommitted') ) {
        $data->{uncommitted} = 1;
    }
    my $saved = {%$data};
    $o->update($data);
    my $attributes_dump = Data::Dump::dump($saved);

    # This should probably be in the model, but I don't know how to get the user information down there
    Opsview::Auditlog->create(
        {   username => $c->user->username,
            text     => $c->translate( "messages.auditlog.admin.updatingObject [_1] [_2] [_3]", $o->class_title, $o->identity_string, $attributes_dump ),
        }
    );
}

sub do_update {
    my ( $self, $c ) = @_;
    my $audit_fields = $c->stash->{audit_fields};

    my %set_many       = map { ( $_ => 1 ) } @{ $self->set_many };
    my %set_many_words = map { ( $_ => 1 ) } @{ $self->set_many_words };
    my %set_commasep   = map { ( $_ => 1 ) } @{ $self->set_commasep };
    my %null_if_empty  = map { ( $_ => 1 ) } @{ $self->null_if_empty };
    my %zero_if_empty  = map { ( $_ => 1 ) } @{ $self->zero_if_empty };
    my %may_create_new = map { ( $_ => 1 ) } @{ $self->may_create_new };
    my %jira_duration  = map { ( $_ => 1 ) } @{ $self->jira_duration };

    # Pre process things like hostgroup_new. If set, delete hostgroup
    foreach my $p ( @{ $self->may_create_new } ) {
        my $new = "$p" . "_new";
        if ( $c->req->params->{$new} ) {
            delete $c->req->params->{$p};
        }
        else {
            delete $c->req->params->{$new};
        }
    }

    my $object = $c->stash->{object};

    # For set_many_on_off, "all" comes in {name}_all, and "on" comes in {name}.
    # Need to preprocess because cannot guarantee order below
    foreach my $h ( @{ $self->set_many_on_off } ) {
        my $p      = $h->{name};
        my $others = $h->{parameters} || [];
        my $all    = "$p" . "_all";
        if ( my @v = $c->req->param($all) ) {
            my @on = $c->req->param($p);
            my @extras;
            my $h;
            my $i = 1;
            foreach $_ (@$others) {
                my @a = $c->req->param($_);
                push @extras, \@a;
                $h->{$i} = \@a;
                $i++;
            }
            my $method = "set_${p}_on_off";
            if ( $c->debug ) {
                $c->log->debug( "Update: setting $all to " . join( ', ', @v ) );
                $c->log->debug( "Update: setting ${p} to " . join( ', ', @on ) );
                $c->log->debug( "Update: setting extras to " . dump(@extras) );
            }

            # Create a hash for better audit log display
            # Have to do before $object->$method, because that could be destructive
            my %on;
            map { $on{$_}++ } @on;
            my @a;
            for ( my $i = 0; $i < ( scalar @v ); $i++ ) {
                my @info = map { $h->{$_}->[$i] } ( 1 .. scalar @extras );
                push @a, $v[$i] . "=" . ( $on{ $v[$i] } ? "on" : "off" ) . "('" . join( "','", @info ) . "')";
            }
            push @$audit_fields, "${p}=(" . join( ",", @a ) . ")";

            $object->$method( \@v, \@on, @extras );
            delete $c->req->params->{$all};
            delete $c->req->params->{$p};
            map { delete $c->req->params->{$_} } (@$others);
        }
    }

PARAM: foreach my $p ( $c->req->param ) {
        next PARAM if $p =~ /^(?:id|action|type|from|server_ignore|\..*)$/;

        # no need to check for initial since they were deleted already

        foreach my $re ( @{ $self->ignore_keys } ) {
            $c->log->debug("Checking :$p: against :$re:");
            next if $p =~ m/$re/;
        }

        my @val = $c->req->param($p);
        $c->log->debug( "Update: setting ${p} to " . join( ', ', @val ) ) if $c->debug;

        if ( ( $p =~ /^(.*)_new$/ ) && $may_create_new{$1} ) {
            $p = $1;
            my $class = $c->model('OpsviewDB')->convert_type_to_class($p);
            @val = $class->find_or_create( { name => $val[0] } );
            push @$audit_fields, "(creating new $p=$val[0])";
        }

        if ( $self->can("do_update_${p}") ) {    # escape hatch

            my $meth = "do_update_${p}";
            $self->$meth( $c => @val );
            push @$audit_fields, "${p}='" . join( ",", @val ) . "'";

        }
        elsif ( delete $set_many{$p} ) {

            my $meth = "set_${p}_to";
            $object->$meth(@val);
            push @$audit_fields, "${p}=(" . join( ",", @val ) . ")";

        }
        elsif ( delete $set_many_words{$p} ) {

            my $meth = "set_${p}_to";
            my @v = split( /[ ,]/o, $val[0] );
            $object->$meth(@v);
            push @$audit_fields, "${p}=(" . ( join( ",", @v ) || "null" ) . ")";

        }
        elsif ( delete $set_commasep{$p} ) {

            $object->$p( join( ',', @val ) );
            push @$audit_fields, "${p}='" . join( ",", @val ) . "'";

        }
        elsif ( delete $null_if_empty{$p} ) {

            my $v = length $val[0] ? ( $val[0] ) : (undef);
            $object->$p($v);
            push @$audit_fields, "$p='" . ( !defined $v ? "null" : $v ) . "'";

        }
        elsif ( $object->can($p) ) {    # sanity check. Used to be find_column, but this doesn't work with a might_have relationship

            delete $zero_if_empty{$p};    # for later

            # only change db object if field is actually changed
            # The "if ! defined" is to stop warnings when a value could be undef because it is NULL in the db
            if ( ( !defined( $_ = $object->$p ) ) || $_ ne $val[0] ) {

                if ( $jira_duration{$p} ) {
                    eval { $val[0] = Opsview::Utils::Time->jira_duration_to_seconds( $val[0] ) };
                    if ($@) {
                        my $e = $@;
                        $e =~ s/\n//g;
                        $c->log->warn("Error with jira duration input $p: $e");
                        next PARAM;
                    }
                }
                $object->$p( $val[0] );
                push @$audit_fields, "$p='$val[0]'" unless ( $p eq "password" );
            }

        }
        else {
            $c->log->debug("Unknown parameter ${p} to update") if $c->debug;
        }
    }
    foreach my $many ( keys %set_many, keys %set_many_words ) {    # only unset remain
        my $meth = "set_${many}_to";
        $object->$meth();
        push @$audit_fields, "$many=()";
    }
    foreach my $commasep ( keys %set_commasep ) {
        $object->$commasep(undef);
        push @$audit_fields, "$commasep=null";
    }
    foreach my $null ( keys %null_if_empty ) {
        $c->log->debug("Update: setting $null to NULL") if $c->debug;
        $object->$null(undef);
        push @$audit_fields, "$null=undef";
    }
    foreach my $zero ( keys %zero_if_empty ) {
        $c->log->debug("Update: setting $zero to 0") if $c->debug;
        $object->$zero(0);
        push @$audit_fields, "$zero=0";
    }

    # Need to save again because if created here, will not be in stash
    $c->stash( audit_fields => $audit_fields );
}

sub unique : Local : Arg(0) : Does(ACL) : RequiresRole(CONFIGUREVIEW) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $dups = 0;
    foreach my $p ( keys %{ $c->req->params } ) {
        my $v = $c->req->param($p);
        if ( defined $v && $c->stash->{model_class}->has_column($p) && !$c->stash->{model_class}->count( $p => $v ) ) {
            $dups++;
        }
    }
    $c->stash( ResultSet => ( $dups == 0 ? 0 : 1 ) );
    $c->stash( output => "json" );
}

# This is used to try and catch Data::FormValidator errors. This really needs to be sorted
# out better by either Catalyst converting to Try::Tiny or D::FV::R not returning true on boolean
sub handle_exception {
    my ( $self, $c, $exception ) = @_;
    if ( $exception->isa("Data::FormValidator::Results") ) {
        my @errors;
        my $errors = $exception->msgs;
        foreach my $a ( keys %$errors ) {
            push @errors, "$a: " . $errors->{$a};
        }

        my $text = join( "; ", @errors );
        $c->error($text);
    }
    else {
        $c->error($exception);
    }
    $c->detach;
}

1;
