#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::ControllerBase::Config;

use Moose;
use namespace::autoclean;
use Opsview::Utils qw(convert_to_arrayref);
use Try::Tiny;
use JSON;

BEGIN { extends 'Catalyst::Controller::ActionRole', 'Opsview::Web::ControllerBase::REST' }

__PACKAGE__->mk_accessors(
    qw/
        model_name
        list_order_by
        api_path_prefix
        /
);

__PACKAGE__->config( api_path_prefix => "/rest/config", );

after 'begin' => sub {
    my ( $self, $c ) = @_;
    my $data = $c->req->data;
    if ( defined $data && ref $data eq "HASH" ) {

        # Delete any atttribute called "ref"
        try {
            $data = Opsview::Utils->remove_keys_from_hash( $data, ["ref"] );
        }
        catch {
            $c->res->status(400);
            $c->stash( "rest" => { message => "Error when parsing data", detail => $_ } );
            $c->detach;
        };
        $c->req->data($data);
    }

    my @requested_colunms = map { split( ",", $_ ) } ( $c->req->param("cols") );
    $c->stash->{requested_columns} = \@requested_colunms;

    $c->stash->{resultset} = $c->model( $self->model_name );

    # Need to delete the cookie to ensure that the user is not restored via cookies
    # as REST uses its own authentication routines
    delete $c->req->cookies->{opsview_web_session};

};

before 'end' => sub {
    my ( $self, $c ) = @_;

    # Any rest data with message->{detail} as an object will be stringified
    my $rest = $c->stash->{rest};
    if ( exists $rest->{detail} && ref( $rest->{detail} ) ne "" ) {
        $rest->{detail} = "" . $rest->{detail};
    }
};

sub list : Chained("objecttype") : PathPart("") : Args(0) : ActionClass('REST') {
}

sub list_GET {
    my ( $self, $c ) = @_;

    my $serialize_options = { columns => $c->stash->{requested_columns}, ref_prefix => $self->api_path_prefix };

    # Possible optimisation in future
    #my $search_args = { columns => \@allowed_columns, prefetch => [keys %has_a] };
    my $search_args = {};

    # order_by override required because could have ambigious duplicate column names
    $search_args->{order_by} = $self->list_order_by if $self->list_order_by;

    my $rows = ( exists $c->req->params->{rows} ) ? $c->req->params->{rows} : 50;
    my $page = $c->req->params->{page} || 1;

    if ( !( $rows eq "all" || $rows == 0 ) ) {
        $search_args->{rows} = $rows;
        $search_args->{page} = $page;
    }

    my $rs = $c->stash->{resultset};

    # Need to override search column of id, because can be ambiguous
    $c->req->params->{"s.me.id"} = delete $c->req->params->{"s.id"} if exists $c->req->params->{"s.id"};

    # Implement search based on parameters
    my %allowed_cols  = map { ( $_ => 1 ) } @{ $rs->result_class->allowed_columns };
    my $params        = $c->req->params;
    my $search_fields = {};
    if ( $params->{json_filter} && ref $params->{json_filter} eq "" ) {
        $search_fields = $self->search_args_from_json_filter( $c, $params->{json_filter} );
    }
    else {
        foreach my $p ( keys %$params ) {
            if ( $p =~ /^s\.([\w\.]+)$/ ) {

                # Do a direct pass through of search parameters through to DBIx::Class. Allow "me." prefix
                my $fullcolname  = $1;
                my $shortcolname = $1;
                $shortcolname =~ s/^me\.//;
                if ( $allowed_cols{$shortcolname} ) {
                    my $search_texts = convert_to_arrayref( $params->{$p} );
                    $search_fields->{$fullcolname} = { "-like" => $search_texts };
                }
            }
        }
    }

    my $result;
    $rs = $rs->search( $search_fields, $search_args );
    if ( $rows ne "0" ) {
        my @items;
        try {
            while ( my $it = $rs->next ) {
                my $hashref = $it->serialize_to_hash($serialize_options);
                $hashref->{ref} = $c->uri_for_path( $it->id );
                push @items, $hashref;
            }
        }
        catch {
            $c->res->status(400);
            $c->stash( "rest" => { message => "Error executing search", detail => $_ } );
            $c->detach;
        };
        $result->{list} = \@items;
    }
    $result->{summary} = {
        rows    => $rs->count,
        allrows => $c->stash->{resultset}->count,
    };
    if ( !( $rows eq "all" || $rows eq "0" ) ) {
        $result->{summary}->{totalrows}  = $rs->pager->total_entries;
        $result->{summary}->{totalpages} = $rs->pager->last_page;
        $result->{summary}->{page}       = $rs->pager->current_page;
    }
    $c->stash( 'rest' => $result );
}

sub list_POST : Does(ACL) : RequiresRole(CONFIGURESAVE) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $data = $self->check_data_is_valid( $c, "No create data found" );
    $self->list_synchronise_with_data( $c, $data, { create_only => 1 } );
}

sub list_PUT : Does(ACL) : RequiresRole(CONFIGURESAVE) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $data = $self->check_data_is_valid( $c, "No synchronise data found" );
    $self->list_synchronise_with_data( $c, $data );
}

sub list_synchronise_with_data {
    my ( $self, $c, $data, $options ) = @_;
    $options ||= {};

    if ( $data->{list} && ( ref( $data->{list} ) eq "ARRAY" ) ) {

        my $rs            = $c->stash->{resultset};
        my %ids_to_delete = ();
        if ( $c->req->params->{synchronise} ) {
            map { $ids_to_delete{ $_->id }++ } ( $rs->search( {}, { "columns" => ["id"] } ) );
        }

        # Roll back entire operation if anything fails
        my $guard = $rs->result_source->schema->txn_scope_guard;

        $rs->synchronise_list_start($options);

        foreach my $item ( @{ $data->{list} } ) {
            $c->stash->{list_object_number}++;
            $self->synchronise_with_data( $c, $item, $options );
            delete $ids_to_delete{ $c->stash->{object}->id };
        }

        $rs->synchronise_list_end($options);

        my @to_delete = keys %ids_to_delete;
        if (@to_delete) {
            try {
                $rs->search( { id => \@to_delete } )->delete;
            }
            catch {
                $c->res->status(400);
                $c->stash( "rest" => { message => "Rollback: Error when trying to delete old objects in a synchronise", detail => $_ } );
                $c->detach;
            };
        }

        $guard->commit;

        $c->stash( rest => { objects_updated => $c->stash->{list_object_number} } );
    }
    else {
        $self->synchronise_with_data( $c, $data, $options );
    }
}

sub object : Chained("objecttype") : PathPart("") : Args(1) : ActionClass('REST') {
    my ( $self, $c, $id ) = @_;
    my $object = $c->stash->{resultset}->find($id);

    unless ($object) {
        $c->res->status(404);
        $c->stash( "rest" => { message => "No object found with id=$id" } );
        $c->detach;
    }

    $c->stash( object => $object );
}

sub object_GET {
    my ( $self, $c ) = @_;
    my $object = $c->stash->{object};

    my $data = $object->serialize_to_hash( { ref_prefix => $self->api_path_prefix, columns => $c->stash->{requested_columns} } );

    $c->stash( 'rest' => { object => $data } );
}

sub object_PUT : Does(ACL) : RequiresRole(CONFIGURESAVE) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $id = $c->stash->{object}->id;
    my $data = $self->check_data_is_valid( $c, "No update data found" );

    # Add the id in to force this object. ->synchronise will use this in preference to data coming in
    $data->{id} = $id;

    # Strip out uncommitted flag if placed through
    delete $data->{uncommitted};

    $self->synchronise_with_data( $c, $data );
}

sub object_POST : Does(ACL) : RequiresRole(CONFIGURESAVE) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    my $data = $self->check_data_is_valid( $c, "No clone data found" );

    my $clone_from = $c->stash->{object}->serialize_to_hash;
    delete $clone_from->{id};
    delete $clone_from->{uncommitted};

    my $new_data = { %$clone_from, %$data };

    $self->synchronise_with_data( $c, $new_data, { create_only => 1 } );
}

sub object_DELETE : Does(ACL) : RequiresRole(CONFIGURESAVE) : ACLDetachTo(/access_denied) {
    my ( $self, $c ) = @_;
    try {
        $c->stash->{object}->my_delete( { username => $c->user->username } );
    }
    catch {
        $c->res->status(400);
        $c->stash( "rest" => { message => "Error when deleting object", detail => $_ } );
        $c->detach;
    };

    # We add some data in the rest json response. Having an empty response can result in unexpected behaviour
    $c->stash( 'rest' => { success => 1 } );
}

sub check_data_is_valid {
    my ( $self, $c, $message_if_data_not_set ) = @_;
    my $data = $c->req->data;
    unless ($data) {
        $c->res->status(400);
        $c->stash( "rest" => { message => $message_if_data_not_set } );
        $c->detach;
    }
    unless ( ref $data eq "HASH" ) {
        $c->res->status(400);
        $c->stash( "rest" => { message => "Data not in correct format" } );
        $c->detach;
    }

    # Helper. If someone does a POST/PUT from a GET without removing the "object" key, we just assume they forgot
    # So just collapse the hash down from there
    if ( exists $data->{object} ) {
        $c->req->data( $data->{object} );

        # Need to check again
        return $self->check_data_is_valid( $c, $message_if_data_not_set );
    }
    else {
        return $data;
    }
}

sub synchronise_with_data {
    my ( $self, $c, $data, $options ) = @_;
    $options ||= {};
    $options->{username} = $c->user->username;
    my $new_object;
    my $list_object_number = $c->stash->{list_object_number};
    try {
        $new_object = $c->stash->{resultset}->synchronise( $data, $options );
        $c->stash( object => $new_object );
    }
    catch {
        $c->res->status(400);
        my $message;
        my $output = { detail => $_ };
        if ($list_object_number) {
            $output->{objects_updated} = 0;
            if ( $options->{create_only} ) {
                $message = "Rollback. Error trying to create object: " . $list_object_number;
            }
            else {
                $message = "Rollback. Error trying to synchronise object: " . $list_object_number;
            }
        }
        else {
            if ( $options->{create_only} ) {
                $message = "Error trying to create object";
            }
            else {
                $message = "Error trying to synchronise object";
            }
        }
        $output->{message} = $message;
        $c->stash( "rest" => $output );
        $c->detach;
    };
    unless ($list_object_number) {
        $self->object_GET($c);
    }
}

# Returns a SQL::Abstract hash, based on json_filter input
# TODO: Should validate input in future - just pass through for moment
sub search_args_from_json_filter {
    my ( $self, $c, $json_filter ) = @_;
    my $json = JSON->new->relaxed;
    my $filter;
    try {
        $filter = $json->decode($json_filter);
    }
    catch {
        $c->res->status(400);
        $c->stash( "rest" => { message => "Error when parsing json_filter", detail => $_ } );
        $c->detach;
    };

    # Add "me." to every key name, to make it clear searching only this table
    $filter = Opsview::Utils->add_me_to_columns($filter);

    if ( $c->debug ) {
        $c->log->debug( "JSON filter = " . Data::Dump::dump($filter) );
    }
    $filter;
}

1;
