#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::ControllerBase::REST::NagiosAction;

use Moose;
use namespace::autoclean;
use Opsview::Utils qw(convert_to_arrayref unique_objects);
use Try::Tiny;

BEGIN { extends 'Catalyst::Controller' }

sub action_results : Private {
    my ( $self, $c ) = @_;
    my @done_hosts    = @{ $c->stash->{done_hosts} };
    my @done_services = @{ $c->stash->{done_services} };

    my $result = {
        summary => {
            num_hosts    => scalar @done_hosts,
            num_services => scalar @done_services,
        },
    };

    $result->{list} = {
        hosts    => [],
        services => [],
    };
    foreach my $h (@done_hosts) {
        push @{ $result->{list}->{hosts} }, { hostname => $h->name, id => $h->id };
    }
    foreach my $s (@done_services) {
        push @{ $result->{list}->{services} }, { hostname => $s->hostname, servicename => $s->servicename, id => $s->id };
    }
    $c->stash( 'rest' => $result );
}

sub find_objects : Private {
    my ( $self, $c, $access_required_for_all ) = @_;
    $access_required_for_all ||= "ACTIONALL";

    # Find objects
    my @hostgroups = @{ $c->stash->{hostgroups} || [] };
    my @hosts      = @{ $c->stash->{hosts}      || [] };
    my @services   = @{ $c->stash->{services}   || [] };
    my $search     = {};
    my $args = $c->stash->{object_params} || $c->req->params;
    my $rs;
    my @errors;

    # Services via /status/service filtering parameters
    $rs = $c->model("RuntimeSchema::OpsviewHostObjects");
    if ( !$c->user->has_access($access_required_for_all) ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }
    $rs = $rs->filter_objects( $args, { inclusive => 1, smart_object_search => 1 } );
    while ( my $object = $rs->next ) {
        if ( $object->is_host ) {
            push @hosts, $object->host;
        }
        else {
            push @services, $object->hostservice;
        }
    }

    # Hostgroups
    if ( $c->user->has_access($access_required_for_all) ) {
        $rs = $c->model("RuntimeSchema::OpsviewHostgroups");
        my $hostgroupnames = convert_to_arrayref( $args->{"hg.hostgroupname"} );
        foreach my $hgname (@$hostgroupnames) {
            my @hgs = $rs->search( { name => $hgname } );
            if (@hgs) {
                push @hostgroups, @hgs;
            }
            else {

                # Missing
                push @errors, "Cannot find host group with name $hgname";
            }
        }
        my $hostgroupids = convert_to_arrayref( $args->{"hg.hostgroupid"} );
        foreach my $hgid (@$hostgroupids) {
            my $hg = $rs->find( { id => $hgid } );
            if ($hg) {
                push @hostgroups, $hg;
            }
            else {

                # Missing
                push @errors, "Cannot find host group with id $hgid";
            }
        }
        @hostgroups = unique_objects( \@hostgroups );
    }

    # Hosts
    $rs = $c->model("RuntimeSchema::OpsviewHosts");
    if ( !$c->user->has_access($access_required_for_all) ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }
    $search = {};
    my $hostnames = convert_to_arrayref( $args->{"hst.hostname"} );
    foreach my $hname (@$hostnames) {
        my $host = $rs->find( { name => $hname } );
        if ($host) {
            push @hosts, $host;
        }
        else {

            # Missing
            push @errors, "Cannot find host with name $hname";
        }
    }
    my $hostids = convert_to_arrayref( $args->{"hst.hostid"} );
    foreach my $hid (@$hostids) {
        my $host = $rs->find( { id => $hid } );
        if ($host) {
            push @hosts, $host;
        }
        else {

            # Missing
            push @errors, "Cannot find host with id $hid";
        }
    }
    my $hostgroupnames = convert_to_arrayref( $args->{"hst.hostgroupname"} );
    my $hostgroupids   = convert_to_arrayref( $args->{"hst.hostgroupid"} );
    if ( @$hostgroupids || @$hostgroupnames ) {

        # Maybe this should be based on the host group hierarchy?
        $search->{"hostgroup.name"} = $hostgroupnames if @$hostgroupnames;
        $search->{"hostgroup.id"}   = $hostgroupids   if @$hostgroupids;
        my @host_search = $rs->search( $search, { join => "hostgroup" } );
        push @hosts, @host_search;
    }
    @hosts = unique_objects( \@hosts );

    # Services
    $rs = $c->model("RuntimeSchema::OpsviewHostServices");
    if ( !$c->user->has_access($access_required_for_all) ) {
        $rs = $rs->search( { "contacts.contactid" => $c->user->id }, { join => "contacts" } );
    }
    $search = {};
    my $hs_list = convert_to_arrayref( $args->{"svc.hs"} );
    foreach my $hs (@$hs_list) {
        my ( $hostname, $servicename ) = split( "::", $hs );
        my $service = $rs->find( { hostname => $hostname, servicename => $servicename } );
        if ($service) {
            push @services, $service;
        }
        else {

            # Missing
            push @errors, "Cannot find service with hs $hs";
        }
    }
    my $sid_list = convert_to_arrayref( $args->{"svc.serviceid"} );
    foreach my $sid (@$sid_list) {
        my $service = $rs->find( { service_object_id => $sid } );
        if ($service) {
            push @services, $service;
        }
        else {

            # Missing
            push @errors, "Cannot find service with serviceid $sid";
        }
    }

    # Services search
    {
        my $do_search = 0;
        my $hostnames = convert_to_arrayref( $args->{"svc.hostname"} );
        if (@$hostnames) {
            $do_search++;
            $rs = $rs->search( { "me.hostname" => { "-like" => $hostnames } } );
        }
        my $servicenames = convert_to_arrayref( $args->{"svc.servicename"} );
        if (@$servicenames) {
            $do_search++;
            $rs = $rs->search( { "me.servicename" => { "-like" => $servicenames } } );
        }
        my $hostgroupnames = convert_to_arrayref( $args->{"svc.hostgroupname"} );
        if (@$hostgroupnames) {
            $do_search++;
            $rs = $rs->search( { "hostgroup.name" => $hostgroupnames }, { join => { "host" => "hostgroup" } } );
        }
        my $hostgroupids = convert_to_arrayref( $args->{"svc.hostgroupid"} );
        if (@$hostgroupids) {
            $do_search++;
            $rs = $rs->search( { "hostgroup.id" => $hostgroupids }, { join => { "host" => "hostgroup" } } );
        }
        my $keywords = convert_to_arrayref( $args->{"svc.keywordname"} );
        if (@$keywords) {
            $do_search++;
            $rs = $rs->search( { "keywords.keyword" => $keywords }, { join => "keywords" } );
        }
        my $keywordids = convert_to_arrayref( $args->{"svc.keywordid"} );
        if (@$keywordids) {
            $do_search++;
            $rs = $rs->search( { "keywords.viewportid" => $keywordids }, { join => "keywords" } );
        }
        my $servicegroupnames = convert_to_arrayref( $args->{"svc.servicegroupname"} );
        if (@$servicegroupnames) {
            $do_search++;
            $rs = $rs->search( { "servicegroup.name" => $servicegroupnames }, { join => "servicegroup" } );
        }
        my $servicecheckids = convert_to_arrayref( $args->{"svc.servicecheckid"} );
        if (@$servicecheckids) {
            $do_search++;
            $rs = $rs->search( { "me.servicecheck_id" => $servicecheckids } );
        }
        if ($do_search) {
            my @svc_search = $rs->search($search);
            push @services, @svc_search;
        }
    }
    @services = unique_objects( \@services );

    if (@errors) {
        $c->res->status(403);
        $c->stash(
            "rest" => {
                message => "Error when searching for objects",
                detail  => join( "; ", @errors )
            }
        );
        $c->detach;
    }

    $c->stash->{hostgroups} = \@hostgroups;
    $c->stash->{hosts}      = \@hosts;
    $c->stash->{services}   = \@services;
}

1;
