#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::ControllerBase::WrapCGI;

use strict;
use warnings;
use base qw/Catalyst::Controller/;

use HTTP::Request::AsCGI;
use HTTP::Request;
use Path::Class ();
use IPC::Open2;
use IO::Handle;
use IO::File;
use HTML::TreeBuilder;
use URI;
use URI::Escape;
use Encode qw/decode/;

# Hack-around because Catalyst::Engine::HTTP goes and changes
# them to be the remote socket, and FCGI.pm does even dumber things.

open( *REAL_STDIN,  "<&=" . fileno(*STDIN) );
open( *REAL_STDOUT, ">>&=" . fileno(*STDOUT) );

__PACKAGE__->mk_accessors(qw/cgi_base_path/);

__PACKAGE__->config( cgi_base_path => '/usr/local/cgi-bin', );

sub path_prefix {'cgi-bin'}

sub auto : Private {
    my ( $self, $c ) = @_;
    $c->stash( template => 'cgi_output' );
    return $c->forward("/authenticate");
}

sub cgi_to_process {
    my ( $self, $c, $script ) = @_;
    if ( $c->req->param("output") && $c->req->param("output") eq "ajax" ) {
        $self->cgi_to_ajax( $c => $script );
    }
    else {
        $self->cgi_to_stash( $c => $script );
    }
}

sub cgi_to_response {
    my ( $self, $c, $script ) = @_;
    my $res = $self->wrap_cgi( $c, $script );
    $c->res->content_type( $res->content_type );
    $c->res->body( $res->content );
    $c->res->headers( $res->headers );
}

# Get just body data from Nagios CGIs
sub cgi_to_ajax {
    my ( $self, $c, $script ) = @_;
    my $res = $self->wrap_cgi( $c, $script );
    my $data = $res->content;
    my ($body_data) = ( $data =~ m/<body.*?>(.*)<\/body>/si );
    if ( $c->stash->{div_id} ) {
        $body_data = '<div id="' . $c->stash->{div_id} . '">' . $body_data . "</div>";
    }
    $c->res->body($body_data);
}

sub cgi_to_stash {
    my ( $self, $c, $script ) = @_;
    my $res = $self->wrap_cgi( $c, $script );

    # Needs to be a regexp
    if ( ( my $ct = $res->content_type ) !~ m%text/html% ) {
        $c->res->content_type($ct);
        $c->res->body( $res->content );
        return;
    }

    my $data = $res->content;

    my ($head_data) = ( $data =~ m/<head>(.*)<\/head>/si );
    my ($body_data) = ( $data =~ m/<body.*?>(.*)<\/body>/si );

    $body_data ||= $data;    # handle admin_apply.cgi error (returns invalid html)

    my $tree = HTML::TreeBuilder->new;

    $tree->parse($head_data);
    $tree->eof;

    my ($head) = $tree->content_list;

    my $stylesheets = $c->stash->{stylesheets};

    my @extra_head_info;
    my $refresh;

    foreach my $child ( $head->content_list ) {
        if ( $child->tag eq 'link' && $child->attr('rel') eq 'stylesheet' ) {
            my $href = $child->attr('href');
            $href =~ s/^.*stylesheets\///;

            # Need to remove prefix upto stylesheets/ (could have a prefix added at beginning)
            # so /ov3/stylesheet/skins/aqua/theme.css
            # is munged into skins/aqua/theme.css
            push( @$stylesheets, $href );
        }
        if ( $child->tag eq 'meta' ) {
            if ( $child->attr('http-equiv') =~ /^refresh$/i ) {
                $refresh = $child->attr('content');
            }
            elsif ( $child->attr('http-equiv') =~ /^content-type$/i ) {

                # ignore content-types as covered in wrappers
            }
            else {
                push( @extra_head_info, $child->as_HTML );
            }
        }
        if ( $child->tag eq "script" ) {    # Javascripts are saved too
            push @extra_head_info, $child->as_HTML;
        }
    }

    if ( ($refresh) || ( $refresh = $res->header('Refresh') ) ) {
        $c->stash->{refresh_rate} = $refresh;

        # Nagios CGIs do not read GET & POST parameters - only one or other
        # We force all parameters to be POST in the refresh link
        my $p = $c->req->params;
        my $h = {%$p};                      # Not sure why, but must be copied
        $h->{output} = "ajax";
        my $url = URI->new('http:');
        $url->query_form(%$h);
        $c->stash->{post_body}      = $url->query;
        $c->stash->{refresh_method} = "post";
        $c->forward("/refresh_page");
    }

    if (@extra_head_info) {
        $c->stash( extra_head_info => join( "\n", @extra_head_info ) );
    }

    if ( $c->stash->{div_id} ) {
        $body_data = '<div id="' . $c->stash->{div_id} . '">' . $body_data . "</div>";
    }

    # Need to decode output to retain UTF8 characters
    $c->stash( cgi_output => decode( 'utf8', $body_data ) );

    $tree->delete;
}

sub wrap_cgi {
    my ( $self, $c, $script ) = @_;
    my $req          = HTTP::Request->new( $c->req->method => $c->req->uri );
    my $body         = $c->req->body;
    my $body_content = '';

    $req->content_type( $c->req->content_type );    # set this now so we can override

    if ($body) {                                    # Slurp from body filehandle
        local $/;
        $body_content = <$body>;
    }
    else {
        my $body_params = $c->req->body_parameters;
        if ( keys %$body_params ) {
            my @parts;
            foreach my $key ( keys %$body_params ) {
                my $raw = $body_params->{$key};
                foreach my $value ( ref $raw ? @$raw : ($raw) ) {
                    push( @parts, join( '=', map { uri_escape($_) } ( $key, $value ) ) );
                }
            }
            $body_content = join( '&', @parts );
            $req->content_type('application/x-www-form-urlencoded');
        }
    }

    #warn "Body type: ".$req->content_type;
    #warn "Body: ${body_content}";

    $req->content($body_content);
    $req->content_length( length($body_content) );
    my $user = ( $c->user_exists ? $c->user->username : '' );
    my $env = HTTP::Request::AsCGI->new(
        $req,
        PATH_INFO   => join( "/", "",                              @{ $c->req->args } ),
        REMOTE_USER => $user,
        PERL5LIB    => $ENV{PERL5LIB} # propagate custom perl lib paths
    );

    my $cgi_path = Path::Class::File->new( $self->cgi_base_path, $script )->stringify;
    my $saved_error;
    {
        local *STDIN  = \*REAL_STDIN;     # restore the real ones so the filenos
        local *STDOUT = \*REAL_STDOUT;    # are 0 and 1 for the env setup

        $env->setup;
        if ( $c->debug ) {
            my $h = $env->enviroment;
            my $envvars = join( $/, map {"$_=$ENV{$_}"} ( sort keys %$h ) );
            $c->log->debug( 'CGI Command: ', $cgi_path );
            $c->log->debug( 'CGI Environment: ', $/, $envvars );
        }
        system($cgi_path);
        $saved_error = $!;
        $env->restore;

        warn "CGI invoke failed: $@" if $@;

    }

    my $code = $env->response->code;
    $c->res->status($code);
    if ( $code == 500 ) {
        $c->log->error("Internal server error invoking $script: $saved_error");
    }
    return $env->response;
}

1;
