#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::Utils;

use strict;
use warnings;

# This package is for utility functions that do not
# require any paths

sub run_external_command {
    my ( $self, $c, $cmd ) = @_;
    local $SIG{CHLD} = 'DEFAULT';
    my @output;
    $c->log->debug("Running: @$cmd");
    unless ( open C, "-|", @$cmd ) {
        $c->res->body( $c->loc("messages.error.cannotRunCommand") );
        return;
    }
    @output = <C>;
    close C;
    $c->res->body( $output[0] || $c->loc("messages.error.noOutput") );
}

# Given a $c->req->params, will convert into a hash
# Rules are:
#   name[keyA] = [valueA1, valueA2]
#   name[keyB] = [valueB1, valueB2]
# will convert to
#   { name => [ { keyA => valueA1, keyB => valueB1 }, { keyA => value A2, keyB => valueB2 } ] }
# Order is retained
# Assumes that there are same number of elements in name[keyA] and name[keyB]
sub convert_multilist_params_to_hash {
    my ( $class, $params, $args ) = @_;
    $args ||= {};
    my $data = {};
    foreach my $p ( keys %$params ) {
        next unless ( $p =~ /^(\w+)\[/ );
        my $parameter = $1;
        my $list = $data->{$parameter} ||= [];
        $p =~ /^.*?\[(\w+)\]/;
        my $key    = $1;
        my $values = $params->{$p};
        delete $params->{$p} if $args->{delete};
        $values = [$values] if ( ref $values ne "ARRAY" );    # Since Catalyst may return single values instead of an array

        for ( my $i = 0; $i < scalar @$values; $i++ ) {
            $list->[$i] = {} unless $list->[$i];
            $list->[$i]->{$key} = $values->[$i];
        }
    }
    $data;
}

# Given a $c->req->params, will convert into a hash
# Rules are:
#   name[1][keyA] = valueA1,
#   name[1][keyB] = valueB1,
#   name[2][keyA] = valueA2,
#   name[2][keyB] = valueB2,
# will convert to
#   { name => [ { keyA => valueA1, keyB => valueB2 }, { keyA => valueA2, keyB => valueB2 } ] }
# The order is sorted by the id column
sub convert_indexed_params_to_hash {
    my ( $class, $params, $args ) = @_;
    $args ||= {};
    my $items = {};
    foreach my $p ( keys %$params ) {
        next unless ( $p =~ /^(\w+)\[(\w+)\]\[(\w+)\]$/ );
        my $parameter = $1;
        my $id        = $2;
        my $key       = $3;
        $items->{$parameter}->{$id}->{$key} = $params->{$p};
        delete $params->{$p} if $args->{delete};
    }
    $items;
}

sub setup_specific_page_titles {
    my ( $self, $c ) = @_;
    my $host         = $c->req->param('host');
    my $hostgroup    = $c->req->param('hostgroup');
    my $service      = $c->req->param('service');
    my $servicegroup = $c->req->param('servicegroup');

    if ( $host && $host ne 'null' ) {
        $c->stash->{page_title} .= ' > ' . $c->loc( 'ui.title.host [_1]', $host );
    }
    if ( $hostgroup && $hostgroup ne 'null' ) {
        $c->stash->{page_title} .= ' > ' . $c->loc( 'ui.title.hostgroup [_1]', $hostgroup );
    }
    if ( $service && $service ne 'null' ) {
        $c->stash->{page_title} .= ' > ' . $c->loc( 'ui.title.service [_1]', $service );
    }
    if ( $servicegroup && $servicegroup ne 'null' ) {
        $c->stash->{page_title} .= ' > ' . $c->loc( 'ui.title.servicegroup [_1]', $servicegroup );
    }
}

1;
