#
# AUTHORS:
#       Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Web::View::Download::XML;

use strict;
use warnings;
use parent 'Catalyst::View';

use XML::Writer;

=head1 NAME

Opsview::Web::View::Download::XML - Catalyst Download XML View

=head1 DESCRIPTION

Catalyst View outputting $c->stash->{xml}->{data} as XML using $c->stash->{xml}->{template}
to give the tag and attribute names to use, and which values to include from the data.

e.g.

$c->stash->{xml}->{data} = {
  'summary' => { # a hash
    'maxeventid' => '2892' # a scalar
  },
  'list' => [ # an array or hashes
    { # a hash
      'id' => '4321',
      'host' => 'host1'
    },
    { # a hash
      'id' => '1234',
      'host' => 'host2'
    }
  ],
  'list2' => [ # an array of scalars
    'foo', 'bar', 'foobar'
  ]
};

# and

$c->stash->{xml}->{template} = {
  'type' => 'hash',
  'name' => 'root',
  'children' => {
    'summary' => {
      'type' => 'hash',
      'name' => 'summary',
      'children' => {
        'maxeventid' => {
          'type' => 'scalar',
          'name' => 'maxid'
        }
      }
    },
    'list' => {
      'type' => 'array',
      'name' => 'list',
      'item' => {
        'type' => 'hash',
        'name' => 'event',
        'attrs' => {
          'id' => { 'name' => 'id' }
        },
        'children' => {
          'hostname' => {
            'type' => 'scalar',
            'name' => 'host'
          }
        }
      }
    },
    'list2' => {
      'type' => 'array',
      'name' => 'names',
      'item' => {
        'type' => 'scalar',
        'name' => 'name'
      }
    }
  }
};

# and

$c->forward("View::Download::XML");

# should give

<?xml version="1.0" encoding="UTF-8"?>
<root>
<summary><maxid>2892</maxid></summary>
<list>
<event id="4321"><host>host1</host></event>
<event id="1234"><host>host2</host></event>
</list>
<names>
<name>foo</name>
<name>bar</name>
<name>foobar</name>
</names>
</root>

=head1 AUTHOR

Tristan Aubrey-Jones OBO Opsera LTD.

=cut

__PACKAGE__->config( 'stash_key' => 'xml' );

sub process {
    my $self = shift;
    my ($c) = @_;

    my $content = $self->render( $c, $c->stash );

    $c->res->headers->header( "Content-Type" => "application/xml" );
    $c->res->body($content);
}

sub write_element {
    my $self = shift;
    my ( $writer, $t, $n ) = @_;

    my $type = $t->{type} || 'scalar';
    my $name = $t->{name};

    if ( $type eq 'scalar' ) {

        # start tag
        $writer->startTag($name);

        # write scalar
        $writer->characters($n);
    }
    elsif ( $type eq 'hash' ) {

        # add any attributes
        my @attrs = ();
        if ( exists $t->{attrs} ) {
            my $atttems = $t->{attrs};
            foreach my $attname ( keys %{$atttems} ) {
                push( @attrs, $atttems->{$attname}->{name}, $n->{$attname} ) if ( exists $n->{$attname} );
            }
        }

        # start tag
        if ( $#attrs > 0 ) { $writer->startTag( $name, @attrs ); }
        else               { $writer->startTag($name); }

        # write children
        my $children = $t->{children};
        foreach my $child ( keys %{$children} ) {
            $self->write_element( $writer, $children->{$child}, $n->{$child} ) if ( exists $n->{$child} );
        }
    }
    elsif ( $type eq 'array' ) {

        # start tag
        $writer->startTag($name);

        # write array items
        my $itemtem = $t->{item};
        foreach my $item ( @{$n} ) {
            $self->write_element( $writer, $itemtem, $item );
        }
    }

    # end tag
    $writer->endTag();
}

sub render {
    my $self = shift;
    my ( $c, $args ) = @_;

    my $data;
    my $t;
    my $stash_key = $self->config->{'stash_key'};
    my $content   = "";

    if (   defined( $args->{$stash_key} )
        && ref( $args->{$stash_key} ) =~ /HASH/
        && defined( $args->{$stash_key}{data} ) )
    {
        $data = $args->{$stash_key}{data};
        $t    = $args->{$stash_key}{template};
    }

    # Create new XML writer
    my $writer = new XML::Writer( OUTPUT => \$content );    #NEWLINES => 1);

    # Write XML using template
    $writer->xmlDecl("UTF-8");
    $self->write_element( $writer, $t, $data );

    # Return XML string
    $writer->end();
    return $content;
}

1;

__END__
